@@ -1,8 +1,10 @@
 /*
- * Copyright (c) 1999-2018 Douglas Gilbert.
+ * Copyright (c) 1999-2019 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 /*
@@ -21,6 +23,8 @@
 #include <errno.h>
 #include <string.h>
 #include <unistd.h>
+#define __STDC_FORMAT_MACROS 1
+#include <inttypes.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -38,7 +42,7 @@
 #endif
 
 
-static const char * const version_str = "1.90 20180712";
+static const char * const version_str = "1.95 20191219";
 
 
 #define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
@@ -91,12 +95,22 @@ sg_cmds_close_device(int device_fd)
 
 static const char * const pass_through_s = "pass-through";
 
-static int
-sg_cmds_process_helper(const char * leadin, int mx_di_len, int resid,
-                       const uint8_t * sbp, int slen, bool noisy,
-                       int verbose, int * o_sense_cat)
+static void
+sg_cmds_resid_print(const char * leadin, bool is_din, int num_req,
+                    int num_got)
 {
-    int scat, got;
+    pr2ws("    %s: %s requested %d bytes (data-%s  got %d "
+          "bytes%s\n", leadin, pass_through_s,num_req,
+          (is_din ? "in), got" : "out) but reported"), num_got,
+          (is_din ? "" : " sent"));
+}
+
+static int
+sg_cmds_process_helper(const char * leadin, int req_din_x, int act_din_x,
+                       int req_dout_x, int act_dout_x, const uint8_t * sbp,
+                       int slen, bool noisy, int verbose, int * o_sense_cat)
+{
+    int scat;
     bool n = false;
     bool check_data_in = false;
     char b[512];
@@ -136,11 +150,27 @@ sg_cmds_process_helper(const char * leadin, int mx_di_len, int resid,
         sg_get_sense_str(NULL, sbp, slen, (verbose > 1),
                          sizeof(b), b);
         pr2ws("%s", b);
-        if ((mx_di_len > 0) && (resid > 0)) {
-            got = mx_di_len - resid;
-            if ((verbose > 2) || check_data_in || (got > 0))
-                pr2ws("    %s requested %d bytes (data-in) but got %d "
-                      "bytes\n", pass_through_s, mx_di_len, got);
+        if (req_din_x > 0) {
+            if (act_din_x != req_din_x) {
+                if ((verbose > 2) || check_data_in || (act_din_x > 0))
+                    sg_cmds_resid_print(leadin, true, req_din_x, act_din_x);
+                if (act_din_x < 0) {
+                    if (verbose)
+                        pr2ws("    %s: %s can't get negative bytes, say it "
+                              "got none\n", leadin, pass_through_s);
+                }
+            }
+        }
+        if (req_dout_x > 0) {
+            if (act_dout_x != req_dout_x) {
+                if ((verbose > 1) && (act_dout_x > 0))
+                    sg_cmds_resid_print(leadin, false, req_dout_x, act_dout_x);
+                if (act_dout_x < 0) {
+                    if (verbose)
+                        pr2ws("    %s: %s can't send negative bytes, say it "
+                              "sent none\n", leadin, pass_through_s);
+                }
+            }
         }
     }
     if (o_sense_cat)
@@ -153,19 +183,18 @@ sg_cmds_process_helper(const char * leadin, int mx_di_len, int resid,
  * sense data is found it is decoded and output to sg_warnings_strm (def:
  * stderr); depending on the 'noisy' and 'verbose' settings. Returns -2 for
  * "sense" category (may not be fatal), -1 for failed, 0, or a positive
- * number. If 'mx_di_len > 0' then asks pass-through for resid and returns
- * (mx_di_len - resid); otherwise returns 0. So for data-in it should return
- * the actual number of bytes received. For data-out (to device) or no data
- * call with 'mx_di_len' set to 0 or less. If -2 returned then sense category
+ * number. If din type command (or bidi) returns actual number of bytes read
+ * (din_len - resid); otherwise returns 0. If -2 returned then sense category
  * output via 'o_sense_cat' pointer (if not NULL). Note that several sense
  * categories also have data in bytes received; -2 is still returned. */
 int
 sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
-                     int pt_res, int mx_di_len, const uint8_t * sbp,
-                     bool noisy, int verbose, int * o_sense_cat)
+                     int pt_res, bool noisy, int verbose, int * o_sense_cat)
 {
-    int got, cat, duration, slen, resid, resp_code, sstat;
     bool transport_sense;
+    int cat, slen, resp_code, sstat, req_din_x, req_dout_x;
+    int act_din_x, act_dout_x;
+    const uint8_t * sbp;
     char b[1024];
 
     if (NULL == leadin)
@@ -196,10 +225,22 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
         pr2ws("%s: %s timeout\n", leadin, pass_through_s);
         return -1;
     }
-    if ((verbose > 2) && ((duration = get_scsi_pt_duration_ms(ptvp)) >= 0))
-        pr2ws("      duration=%d ms\n", duration);
-    resid = (mx_di_len > 0) ? get_scsi_pt_resid(ptvp) : 0;
+    if (verbose > 2) {
+        uint64_t duration = get_pt_duration_ns(ptvp);
+
+        if (duration > 0)
+            pr2ws("      duration=%" PRIu64 " ns\n", duration);
+        else {
+            int d = get_scsi_pt_duration_ms(ptvp);
+
+            if (d != -1)
+                pr2ws("      duration=%u ms\n", (uint32_t)d);
+        }
+    }
+    get_pt_req_lengths(ptvp, &req_din_x, &req_dout_x);
+    get_pt_actual_lengths(ptvp, &act_din_x, &act_dout_x);
     slen = get_scsi_pt_sense_len(ptvp);
+    sbp = get_scsi_pt_sense_buf(ptvp);
     switch ((cat = get_scsi_pt_result_category(ptvp))) {
     case SCSI_PT_RESULT_GOOD:
         if (sbp && (slen > 7)) {
@@ -215,21 +256,31 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
                 }
             }
         }
-        if (mx_di_len > 0) {
-            got = mx_di_len - resid;
-            if ((verbose > 1) && (resid != 0))
-                pr2ws("    %s: %s requested %d bytes (data-in) but got %d "
-                      "bytes\n", leadin, pass_through_s, mx_di_len, got);
-            if (got >= 0)
-                return got;
-            else {
-                if (verbose)
-                    pr2ws("    %s: %s can't get negative bytes, say it got "
-                          "none\n", leadin, pass_through_s);
-                return 0;
+        if (req_din_x > 0) {
+            if (act_din_x != req_din_x) {
+                if ((verbose > 1) && (act_din_x >= 0))
+                    sg_cmds_resid_print(leadin, true, req_din_x, act_din_x);
+                if (act_din_x < 0) {
+                    if (verbose)
+                        pr2ws("    %s: %s can't get negative bytes, say it "
+                              "got none\n", leadin, pass_through_s);
+                    act_din_x = 0;
+                }
             }
-        } else
-            return 0;
+        }
+        if (req_dout_x > 0) {
+            if (act_dout_x != req_dout_x) {
+                if ((verbose > 1) && (act_dout_x >= 0))
+                    sg_cmds_resid_print(leadin, false, req_dout_x, act_dout_x);
+                if (act_dout_x < 0) {
+                    if (verbose)
+                        pr2ws("    %s: %s can't send negative bytes, say it "
+                              "sent none\n", leadin, pass_through_s);
+                    act_dout_x = 0;
+                }
+            }
+        }
+        return act_din_x;
     case SCSI_PT_RESULT_STATUS: /* other than GOOD and CHECK CONDITION */
         sstat = get_scsi_pt_status_response(ptvp);
         if (o_sense_cat) {
@@ -262,7 +313,8 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
         }
         return -1;
     case SCSI_PT_RESULT_SENSE:
-        return sg_cmds_process_helper(leadin, mx_di_len, resid, sbp, slen,
+        return sg_cmds_process_helper(leadin, req_din_x, act_din_x,
+                                      req_dout_x, act_dout_x, sbp, slen,
                                       noisy, verbose, o_sense_cat);
     case SCSI_PT_RESULT_TRANSPORT_ERR:
         if (verbose || noisy) {
@@ -276,8 +328,9 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
                             get_scsi_pt_status_response(ptvp)) && (slen > 0));
 #endif
         if (transport_sense)
-            return sg_cmds_process_helper(leadin, mx_di_len, resid, sbp,
-                                          slen, noisy, verbose, o_sense_cat);
+            return sg_cmds_process_helper(leadin, req_din_x, act_din_x,
+                                          req_dout_x, act_dout_x, sbp, slen,
+                                          noisy, verbose, o_sense_cat);
         else
             return -1;
     case SCSI_PT_RESULT_OS_ERR:
@@ -311,7 +364,6 @@ create_pt_obj(const char * cname)
 static const char * const inquiry_s = "inquiry";
 
 
-
 /* Returns 0 on success, while positive values are SG_LIB_CAT_* errors
  * (e.g. SG_LIB_CAT_MALFORMED). If OS error, returns negated errno or -1. */
 static int
@@ -319,11 +371,16 @@ sg_ll_inquiry_com(struct sg_pt_base * ptvp, bool cmddt, bool evpd, int pg_op,
                   void * resp, int mx_resp_len, int timeout_secs,
                   int * residp, bool noisy, int verbose)
 {
-    int res, ret, k, sense_cat, resid;
+    int res, ret, sense_cat, resid;
     uint8_t inq_cdb[INQUIRY_CMDLEN] = {INQUIRY_CMD, 0, 0, 0, 0, 0};
     uint8_t sense_b[SENSE_BUFF_LEN];
     uint8_t * up;
 
+    if (resp == NULL) {
+        if (verbose)
+            pr2ws("Got NULL `resp` pointer");
+        return SG_LIB_CAT_MALFORMED;
+    }
     if (cmddt)
         inq_cdb[1] |= 0x2;
     if (evpd)
@@ -332,10 +389,11 @@ sg_ll_inquiry_com(struct sg_pt_base * ptvp, bool cmddt, bool evpd, int pg_op,
     /* 16 bit allocation length (was 8, increased in spc3r09, 200209) */
     sg_put_unaligned_be16((uint16_t)mx_resp_len, inq_cdb + 3);
     if (verbose) {
-        pr2ws("    %s cdb: ", inquiry_s);
-        for (k = 0; k < INQUIRY_CMDLEN; ++k)
-            pr2ws("%02x ", inq_cdb[k]);
-        pr2ws("\n");
+        char b[128];
+
+        pr2ws("    %s cdb: %s\n", inquiry_s,
+              sg_get_command_str(inq_cdb, INQUIRY_CMDLEN, false, sizeof(b),
+                                 b));
     }
     if (resp && (mx_resp_len > 0)) {
         up = (uint8_t *)resp;
@@ -349,8 +407,8 @@ sg_ll_inquiry_com(struct sg_pt_base * ptvp, bool cmddt, bool evpd, int pg_op,
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     set_scsi_pt_data_in(ptvp, (uint8_t *)resp, mx_resp_len);
     res = do_scsi_pt(ptvp, -1, timeout_secs, verbose);
-    ret = sg_cmds_process_resp(ptvp, inquiry_s, res, mx_resp_len, sense_b,
-                               noisy, verbose, &sense_cat);
+    ret = sg_cmds_process_resp(ptvp, inquiry_s, res, noisy, verbose,
+                               &sense_cat);
     resid = get_scsi_pt_resid(ptvp);
     if (residp)
         *residp = resid;
@@ -462,7 +520,7 @@ sg_simple_inquiry(int sg_fd, struct sg_simple_inquiry_resp * inq_data,
         inq_data->peripheral_qualifier = 0x3;
         inq_data->peripheral_type = 0x1f;
     }
-    inq_resp = sg_memalign(SAFE_STD_INQ_RESP_LEN, 0, &free_irp, verbose > 4);
+    inq_resp = sg_memalign(SAFE_STD_INQ_RESP_LEN, 0, &free_irp, false);
     if (NULL == inq_resp) {
         pr2ws("%s: out of memory\n", __func__);
         return sg_convert_errno(ENOMEM);
@@ -504,7 +562,7 @@ sg_simple_inquiry_pt(struct sg_pt_base * ptvp,
         inq_data->peripheral_qualifier = 0x3;
         inq_data->peripheral_type = 0x1f;
     }
-    inq_resp = sg_memalign(SAFE_STD_INQ_RESP_LEN, 0, &free_irp, verbose > 4);
+    inq_resp = sg_memalign(SAFE_STD_INQ_RESP_LEN, 0, &free_irp, false);
     if (NULL == inq_resp) {
         pr2ws("%s: out of memory\n", __func__);
         return sg_convert_errno(ENOMEM);
@@ -541,15 +599,15 @@ sg_ll_test_unit_ready_progress_pt(struct sg_pt_base * ptvp, int pack_id,
                                   int * progress, bool noisy, int verbose)
 {
     static const char * const tur_s = "test unit ready";
-    int res, ret, k, sense_cat;
+    int res, ret, sense_cat;
     uint8_t tur_cdb[TUR_CMDLEN] = {TUR_CMD, 0, 0, 0, 0, 0};
     uint8_t sense_b[SENSE_BUFF_LEN];
 
     if (verbose) {
-        pr2ws("    %s cdb: ", tur_s);
-        for (k = 0; k < TUR_CMDLEN; ++k)
-            pr2ws("%02x ", tur_cdb[k]);
-        pr2ws("\n");
+        char b[128];
+
+        pr2ws("    %s cdb: %s\n", tur_s,
+              sg_get_command_str(tur_cdb, TUR_CMDLEN, false, sizeof(b), b));
     }
 
     clear_scsi_pt_obj(ptvp);
@@ -557,8 +615,7 @@ sg_ll_test_unit_ready_progress_pt(struct sg_pt_base * ptvp, int pack_id,
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     set_scsi_pt_packet_id(ptvp, pack_id);
     res = do_scsi_pt(ptvp, -1, DEF_PT_TIMEOUT, verbose);
-    ret = sg_cmds_process_resp(ptvp, tur_s, res, SG_NO_DATA_IN, sense_b,
-                               noisy, verbose, &sense_cat);
+    ret = sg_cmds_process_resp(ptvp, tur_s, res, noisy, verbose, &sense_cat);
     if (-1 == ret)
         ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
     else if (-2 == ret) {
@@ -632,7 +689,7 @@ sg_ll_request_sense_com(struct sg_pt_base * ptvp, int sg_fd, bool desc,
                         void * resp, int mx_resp_len, bool noisy, int verbose)
 {
     bool ptvp_given = false;
-    int k, ret, res, sense_cat;
+    int ret, res, sense_cat;
     static const char * const rq_s = "request sense";
     uint8_t rs_cdb[REQUEST_SENSE_CMDLEN] =
         {REQUEST_SENSE_CMD, 0, 0, 0, 0, 0};
@@ -646,12 +703,12 @@ sg_ll_request_sense_com(struct sg_pt_base * ptvp, int sg_fd, bool desc,
     }
     rs_cdb[4] = mx_resp_len & 0xff;
     if (verbose) {
-        pr2ws("    %s cmd: ", rq_s);
-        for (k = 0; k < REQUEST_SENSE_CMDLEN; ++k)
-            pr2ws("%02x ", rs_cdb[k]);
-        pr2ws("\n");
-    }
+        char b[128];
 
+        pr2ws("    %s cdb: %s\n", rq_s,
+              sg_get_command_str(rs_cdb, REQUEST_SENSE_CMDLEN, false,
+                                 sizeof(b), b));
+    }
     if (ptvp)
         ptvp_given = true;
     else {
@@ -663,8 +720,7 @@ sg_ll_request_sense_com(struct sg_pt_base * ptvp, int sg_fd, bool desc,
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     set_scsi_pt_data_in(ptvp, (uint8_t *)resp, mx_resp_len);
     res = do_scsi_pt(ptvp, -1, DEF_PT_TIMEOUT, verbose);
-    ret = sg_cmds_process_resp(ptvp, rq_s, res, mx_resp_len, sense_b, noisy,
-                               verbose, &sense_cat);
+    ret = sg_cmds_process_resp(ptvp, rq_s, res, noisy, verbose, &sense_cat);
     if (-1 == ret)
         ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
     else if (-2 == ret) {
@@ -716,7 +772,7 @@ sg_ll_report_luns_com(struct sg_pt_base * ptvp, int sg_fd, int select_report,
 {
     static const char * const report_luns_s = "report luns";
     bool ptvp_given = false;
-    int k, ret, res, sense_cat;
+    int ret, res, sense_cat;
     uint8_t rl_cdb[REPORT_LUNS_CMDLEN] =
                          {REPORT_LUNS_CMD, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
     uint8_t sense_b[SENSE_BUFF_LEN];
@@ -724,12 +780,12 @@ sg_ll_report_luns_com(struct sg_pt_base * ptvp, int sg_fd, int select_report,
     rl_cdb[2] = select_report & 0xff;
     sg_put_unaligned_be32((uint32_t)mx_resp_len, rl_cdb + 6);
     if (verbose) {
-        pr2ws("    %s cdb: ", report_luns_s);
-        for (k = 0; k < REPORT_LUNS_CMDLEN; ++k)
-            pr2ws("%02x ", rl_cdb[k]);
-        pr2ws("\n");
-    }
+        char b[128];
 
+        pr2ws("    %s cdb: %s\n", report_luns_s,
+              sg_get_command_str(rl_cdb, REPORT_LUNS_CMDLEN, false,
+                                 sizeof(b), b));
+    }
     if (ptvp)
         ptvp_given = true;
     else if (NULL == ((ptvp = create_pt_obj(report_luns_s))))
@@ -738,8 +794,8 @@ sg_ll_report_luns_com(struct sg_pt_base * ptvp, int sg_fd, int select_report,
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     set_scsi_pt_data_in(ptvp, (uint8_t *)resp, mx_resp_len);
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
-    ret = sg_cmds_process_resp(ptvp, report_luns_s, res, mx_resp_len,
-                               sense_b, noisy, verbose, &sense_cat);
+    ret = sg_cmds_process_resp(ptvp, report_luns_s, res, noisy, verbose,
+                               &sense_cat);
     if (-1 == ret)
         ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
     else if (-2 == ret) {
