@@ -1,8 +1,10 @@
 /*
- * Copyright (c) 2009-2018 Douglas Gilbert.
+ * Copyright (c) 2009-2019 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <stdio.h>
@@ -29,7 +31,7 @@
 #include "sg_pt_nvme.h"
 #endif
 
-static const char * scsi_pt_version_str = "3.09 20180712";
+static const char * scsi_pt_version_str = "3.12 20190612";
 
 
 const char *
@@ -62,7 +64,7 @@ static const char * nvme_scsi_vendor_str = "NVMe    ";
 #define F_INV_OP                0x200
 
 /* Table of SCSI operation code (opcodes) supported by SNTL */
-struct sg_opcode_info_t sg_opcode_info_arr[] =
+static struct sg_opcode_info_t sg_opcode_info_arr[] =
 {
     {0x0, 0, 0, {6,              /* TEST UNIT READY */
       0, 0, 0, 0, 0xc7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0} },
@@ -90,7 +92,13 @@ struct sg_opcode_info_t sg_opcode_info_arr[] =
       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0} },
 };
 
-
+/* Returns pointer to array of struct sg_opcode_info_t of SCSI commands
+ * translated to NVMe. */
+const struct sg_opcode_info_t *
+sg_get_opcode_translation(void)
+{
+    return sg_opcode_info_arr;
+}
 
 /* Given the NVMe Identify controller response and optionally the NVMe
  * Identify namespace response (NULL otherwise), generate the SCSI VPD
