@@ -1,5 +1,5 @@
 /*
-*  Copyright (c) 2012-2018, Kaminario Technologies LTD
+*  Copyright (c) 2012-2019, Kaminario Technologies LTD
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
@@ -22,6 +22,8 @@
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
  *
  * This command performs a SCSI COMPARE AND WRITE. See SBC-3 at
  * http://www.t10.org
@@ -54,7 +56,7 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
-static const char * version_str = "1.25 20180625";
+static const char * version_str = "1.27 20191220";
 
 #define DEF_BLOCK_SIZE 512
 #define DEF_NUM_BLOCKS (1)
@@ -347,7 +349,7 @@ sg_ll_compare_and_write(int sg_fd, uint8_t * buff, int blocks,
                         bool noisy, int verbose)
 {
         bool valid;
-        int k, sense_cat, slen, res, ret;
+        int sense_cat, slen, res, ret;
         uint64_t ull = 0;
         struct sg_pt_base * ptvp;
         uint8_t cawCmd[COMPARE_AND_WRITE_CDB_SIZE];
@@ -368,10 +370,11 @@ sg_ll_compare_and_write(int sg_fd, uint8_t * buff, int blocks,
         set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
         set_scsi_pt_data_out(ptvp, buff, xfer_len);
         if (verbose > 1) {
-                pr2serr("    Compare and write cdb: ");
-                for (k = 0; k < COMPARE_AND_WRITE_CDB_SIZE; ++k)
-                        pr2serr("%02x ", cawCmd[k]);
-                pr2serr("\n");
+                char b[128];
+
+                pr2serr("    Compare and write cdb: %s\n",
+                sg_get_command_str(cawCmd, COMPARE_AND_WRITE_CDB_SIZE, false,
+                                   sizeof(b), b));
         }
         if ((verbose > 2) && (xfer_len > 0)) {
                 pr2serr("    Data-out buffer contents:\n");
@@ -379,8 +382,7 @@ sg_ll_compare_and_write(int sg_fd, uint8_t * buff, int blocks,
         }
         res = do_scsi_pt(ptvp, sg_fd, DEF_TIMEOUT_SECS, verbose);
         ret = sg_cmds_process_resp(ptvp, "COMPARE AND WRITE", res,
-                                   SG_NO_DATA_IN, sense_b, noisy, verbose,
-                                   &sense_cat);
+                                   noisy, verbose, &sense_cat);
         if (-1 == ret)
                 ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
         else if (-2 == ret) {
