@@ -6,13 +6,15 @@
  *
  * Copyright (C) 2003  Grant Grundler    grundler at parisc-linux dot org
  * Copyright (C) 2003  James Bottomley       jejb at parisc-linux dot org
- * Copyright (C) 2005-2018  Douglas Gilbert   dgilbert at interlog dot com
+ * Copyright (C) 2005-2020  Douglas Gilbert   dgilbert at interlog dot com
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2, or (at your option)
  *   any later version.
  *
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
  * See http://www.t10.org for relevant standards and drafts. The most recent
  * draft is SBC-4 revision 2.
  */
@@ -38,7 +40,7 @@
 #include "sg_pr2serr.h"
 #include "sg_pt.h"
 
-static const char * version_str = "1.55 20180830";
+static const char * version_str = "1.61 20200123";
 
 
 #define RW_ERROR_RECOVERY_PAGE 1  /* can give alternate with --mode=MP */
@@ -70,21 +72,27 @@ static const char * version_str = "1.55 20180830";
 /* FORMAT UNIT (SBC) and FORMAT MEDIUM (SSC) share the same opcode */
 #define SG_FORMAT_MEDIUM_CMD 0x4
 #define SG_FORMAT_MEDIUM_CMDLEN 6
+
+/* FORMAT WITH PRESET (new in sbc4r18) */
+#define SG_FORMAT_WITH_PRESET_CMD 0x38
+#define SG_FORMAT_WITH_PRESET_CMDLEN 10
+
 #define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
 
 struct opts_t {
         bool cmplst;            /* -C value */
         bool cmplst_given;
-        bool dcrt;              /* -D */
         bool dry_run;           /* -d */
         bool early;             /* -e */
-        bool fwait;             /* -w (negate for immed) */
+        bool fmtmaxlba;         /* -b (only with F_WITH_PRESET) */
+        bool fwait;             /* -w (negated form IMMED) */
         bool ip_def;            /* -I */
         bool long_lba;          /* -l */
         bool mode6;             /* -6 */
         bool pinfo;             /* -p, deprecated, prefer fmtpinfo */
         bool poll_type;         /* -x 0|1 */
         bool poll_type_given;
+        bool preset;            /* -E */
         bool quick;             /* -Q */
         bool do_rcap16;         /* -l */
         bool resize;            /* -r */
@@ -92,10 +100,12 @@ struct opts_t {
         bool verbose_given;
         bool verify;            /* -y */
         bool version_given;
+        int dcrt;              /* -D (can be given once or twice) */
         int lblk_sz;            /* -s value */
         int ffmt;               /* -t value; fast_format if > 0 */
         int fmtpinfo;
         int format;             /* -F */
+        uint32_t p_id;          /* set by argument of --preset=id  */
         int mode_page;          /* -M value */
         int pfu;                /* -P value */
         int pie;                /* -q value */
@@ -118,6 +128,7 @@ static struct option long_options[] = {
         {"dry_run", no_argument, 0, 'd'},
         {"early", no_argument, 0, 'e'},
         {"ffmt", required_argument, 0, 't'},
+        {"fmtmaxlba", no_argument, 0, 'b'},
         {"fmtpinfo", required_argument, 0, 'f'},
         {"format", no_argument, 0, 'F'},
         {"help", no_argument, 0, 'h'},
@@ -129,6 +140,7 @@ static struct option long_options[] = {
         {"pfu", required_argument, 0, 'P'},
         {"pie", required_argument, 0, 'q'},
         {"poll", required_argument, 0, 'x'},
+        {"preset", required_argument, 0, 'E'},
         {"quick", no_argument, 0, 'Q'},
         {"resize", no_argument, 0, 'r'},
         {"rto_req", no_argument, 0, 'R'},
@@ -151,15 +163,17 @@ usage()
         printf("Usage:\n"
                "  sg_format [--cmplst=0|1] [--count=COUNT] [--dcrt] "
                "[--dry-run] [--early]\n"
-               "            [--ffmt=FFMT] [--fmtpinfo=FPI] [--format] "
-               "[--help] [--ip-def]\n"
-               "            [--long] [--mode=MP] [--pfu=PFU] [--pie=PIE] "
-               "[--pinfo]\n"
-               "            [--poll=PT] [--quick] [--resize] [--rto_req] "
-               "[--security]\n"
-               "            [--six] [--size=LB_SZ] [--tape=FM] "
-               "[--timeout=SECS] [--verbose]\n"
-               "            [--verify] [--version] [--wait] DEVICE\n"
+               "            [--ffmt=FFMT] [--fmtmaxlba] [--fmtpinfo=FPI] "
+               "[--format] [--help]\n"
+               "            [--ip-def] [--long] [--mode=MP] [--pfu=PFU] "
+               "[--pie=PIE]\n"
+               "            [--pinfo] [--poll=PT] [--preset=ID] [--quick] "
+               "[--resize]\n"
+               "            [--rto_req] [--security] [--six] [--size=LB_SZ] "
+               "[--tape=FM]\n"
+               "            [--timeout=SECS] [--verbose] [--verify] "
+               "[--version] [--wait]\n"
+               "            DEVICE\n"
                "  where:\n"
                "    --cmplst=0|1\n"
                "      -C 0|1        sets CMPLST bit in format cdb "
@@ -170,6 +184,8 @@ usage()
                "same as current\n"
                "    --dcrt|-D       disable certification (doesn't "
                "verify media)\n"
+               "                    use twice to enable certification and "
+               "set FOV bit\n"
                "    --dry-run|-d    bypass device modifying commands (i.e. "
                "don't format)\n"
                "    --early|-e      exit once format started (user can "
@@ -186,6 +202,8 @@ usage()
                "count and size)\n"
                "                    use thrice for FORMAT UNIT command "
                "only\n"
+               "    --fmtmaxlba|-b    sets FMTMAXLBA field in FORMAT WITH "
+               "PRESET\n"
                "    --help|-h       prints out this usage message\n"
                "    --ip-def|-I     use default initialization pattern\n"
                "    --long|-l       allow for 64 bit lbas (default: assume "
@@ -200,8 +218,12 @@ usage()
                "    --poll=PT|-x PT    PT is poll type, 0 for test unit "
                "ready\n"
                "                       1 for request sense (def: 0 (1 "
-               "for tape))\n");
-        printf("    --quick|-Q      start format without pause for user "
+               "for tape and\n"
+               "                       format with preset))\n");
+        printf("    --preset=ID|-E ID    do FORMAT WITH PRESET command "
+               "with PRESET\n"
+               "                         IDENTIFIER field set to ID\n"
+               "    --quick|-Q      start format without pause for user "
                "intervention\n"
                "                    (i.e. no time to reconsider)\n"
                "    --resize|-r     resize (rather than format) to COUNT "
@@ -227,17 +249,19 @@ usage()
                "    --verbose|-v    increase verbosity\n"
                "    --verify|-y     sets VERIFY bit in FORMAT MEDIUM (tape)\n"
                "    --version|-V    print version details and exit\n"
-               "    --wait|-w       format command waits until format "
-               "operation completes\n"
+               "    --wait|-w       format commands wait until format "
+               "operations complete\n"
                "                    (default: set IMMED=1 and poll with "
                "Test Unit Ready)\n\n"
                "\tExample: sg_format --format /dev/sdc\n\n"
                "This utility formats a SCSI disk [FORMAT UNIT] or resizes "
                "it. Alternatively\nif '--tape=FM' is given formats a tape "
-               "[FORMAT MEDIUM].\n");
-        printf("WARNING: This utility will destroy all the data on "
-               "DEVICE when '--format'\n\t or '--tape' is given. Check that "
-               "you have specified the correct\n\t DEVICE.\n");
+               "[FORMAT MEDIUM]. Another alternative\nis doing the FORMAT "
+               "WITH PRESET command when '--preset=ID' is given.\n\n");
+        printf("WARNING: This utility will destroy all the data on the "
+               "DEVICE when\n\t '--format', '--tape=FM' or '--preset=ID' "
+               "is given. Double check\n\t that you have specified the "
+               "correct DEVICE.\n");
 }
 
 /* Invokes a SCSI FORMAT MEDIUM command (SSC).  Return of 0 -> success,
@@ -247,7 +271,7 @@ sg_ll_format_medium(int sg_fd, bool verify, bool immed, int format,
                     void * paramp, int transfer_len, int timeout, bool noisy,
                     int verbose)
 {
-        int k, ret, res, sense_cat;
+        int ret, res, sense_cat;
         uint8_t fm_cdb[SG_FORMAT_MEDIUM_CMDLEN] =
                                   {SG_FORMAT_MEDIUM_CMD, 0, 0, 0, 0, 0};
         uint8_t sense_b[SENSE_BUFF_LEN];
@@ -262,10 +286,11 @@ sg_ll_format_medium(int sg_fd, bool verify, bool immed, int format,
         if (transfer_len > 0)
                 sg_put_unaligned_be16(transfer_len, fm_cdb + 3);
         if (verbose) {
-                pr2serr("    Format medium cdb: ");
-                for (k = 0; k < SG_FORMAT_MEDIUM_CMDLEN; ++k)
-                        pr2serr("%02x ", fm_cdb[k]);
-                pr2serr("\n");
+                char b[128];
+
+                pr2serr("    Format medium cdb: %s\n",
+                        sg_get_command_str(fm_cdb, SG_FORMAT_MEDIUM_CMDLEN,
+                                           false, sizeof(b), b));
         }
 
         ptvp = construct_scsi_pt_obj();
@@ -277,8 +302,63 @@ sg_ll_format_medium(int sg_fd, bool verify, bool immed, int format,
         set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
         set_scsi_pt_data_out(ptvp, (uint8_t *)paramp, transfer_len);
         res = do_scsi_pt(ptvp, sg_fd, timeout, verbose);
-        ret = sg_cmds_process_resp(ptvp, "format medium", res, transfer_len,
-                                   sense_b, noisy, verbose, &sense_cat);
+        ret = sg_cmds_process_resp(ptvp, "format medium", res, noisy,
+                                   verbose, &sense_cat);
+        if (-1 == ret)
+                ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
+        else if (-2 == ret) {
+                switch (sense_cat) {
+                case SG_LIB_CAT_RECOVERED:
+                case SG_LIB_CAT_NO_SENSE:
+                        ret = 0;
+                        break;
+                default:
+                        ret = sense_cat;
+                        break;
+                }
+        } else
+                ret = 0;
+        destruct_scsi_pt_obj(ptvp);
+        return ret;
+}
+
+/* Invokes a SCSI FORMAT WITH PRESET command (SBC).  Return of 0 -> success,
+ * various SG_LIB_CAT_* positive values or -1 -> other errors */
+static int
+sg_ll_format_with_preset(int sg_fd, bool immed, bool fmtmaxlba,
+                         uint32_t preset_id, int timeout, bool noisy,
+                         int verbose)
+{
+        int ret, res, sense_cat;
+        uint8_t fwp_cdb[SG_FORMAT_WITH_PRESET_CMDLEN] =
+                     {SG_FORMAT_WITH_PRESET_CMD, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+        uint8_t sense_b[SENSE_BUFF_LEN];
+        struct sg_pt_base * ptvp;
+
+        if (immed)
+                fwp_cdb[1] |= 0x80;
+        if (fmtmaxlba)
+                fwp_cdb[1] |= 0x40;
+        if (preset_id > 0)
+                sg_put_unaligned_be32(preset_id, fwp_cdb + 2);
+        if (verbose) {
+                char b[128];
+
+                pr2serr("    Format with preset cdb: %s\n",
+                        sg_get_command_str(fwp_cdb,
+                                           SG_FORMAT_WITH_PRESET_CMDLEN,
+                                           false, sizeof(b), b));
+        }
+        ptvp = construct_scsi_pt_obj();
+        if (NULL == ptvp) {
+                pr2serr("%s: out of memory\n", __func__);
+                return sg_convert_errno(ENOMEM);
+        }
+        set_scsi_pt_cdb(ptvp, fwp_cdb, sizeof(fwp_cdb));
+        set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
+        res = do_scsi_pt(ptvp, sg_fd, timeout, verbose);
+        ret = sg_cmds_process_resp(ptvp, "format with preset", res, noisy,
+                                   verbose, &sense_cat);
         if (-1 == ret)
                 ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
         else if (-2 == ret) {
@@ -301,7 +381,7 @@ sg_ll_format_medium(int sg_fd, bool verify, bool immed, int format,
 static int
 scsi_format_unit(int fd, const struct opts_t * op)
 {
-        bool need_hdr, longlist, ip_desc;
+        bool need_param_lst, longlist, ip_desc;
         bool immed = ! op->fwait;
         int res, progress, pr, rem, param_sz, off, resp_len, tmout;
         int poll_wait_secs;
@@ -337,8 +417,10 @@ scsi_format_unit(int fd, const struct opts_t * op)
         off = longlist ? LONG_FORMAT_HEADER_SZ : SH_FORMAT_HEADER_SZ;
         param[0] = op->pfu & 0x7;  /* PROTECTION_FIELD_USAGE (bits 2-0) */
         param[1] = (immed ? 0x2 : 0); /* FOV=0, [DPRY,DCRT,STPF,IP=0] */
-        if (op->dcrt)
+        if (1 == op->dcrt)
                 param[1] |= 0xa0;     /* FOV=1, DCRT=1 */
+        else if (op->dcrt > 1)
+                param[1] |= 0x80;     /* FOV=1, DCRT=0 */
         if (ip_desc) {
                 param[1] |= 0x88;     /* FOV=1, IP=1 */
                 if (op->sec_init)
@@ -348,9 +430,9 @@ scsi_format_unit(int fd, const struct opts_t * op)
                 param[3] = (op->pie & 0xf);/* PROTECTION_INTERVAL_EXPONENT */
         /* with the long parameter list header, P_I_INFORMATION is always 0 */
 
-        need_hdr = (immed || op->cmplst || op->dcrt || ip_desc ||
-                    (op->pfu > 0) || (op->pie > 0));
-        param_sz = need_hdr ?
+        need_param_lst = (immed || op->cmplst || (op->dcrt > 0) || ip_desc ||
+                          (op->pfu > 0) || (op->pie > 0));
+        param_sz = need_param_lst ?
                     (off + (ip_desc ? INIT_PATTERN_DESC_SZ : 0)) : 0;
 
         if (op->dry_run) {
@@ -358,7 +440,7 @@ scsi_format_unit(int fd, const struct opts_t * op)
                 pr2serr("Due to --dry-run option bypassing FORMAT UNIT "
                         "command\n");
                 if (vb) {
-                        if (need_hdr) {
+                        if (need_param_lst) {
                                 pr2serr("  FU would have received parameter "
                                         "list: ");
                                 hex2stderr(param, max_param_sz, -1);
@@ -368,13 +450,14 @@ scsi_format_unit(int fd, const struct opts_t * op)
                         pr2serr("  FU cdb fields: fmtpinfo=0x%x, "
                                 "longlist=%d, fmtdata=%d, cmplst=%d, "
                                 "ffmt=%d [timeout=%d secs]\n",
-                                op->fmtpinfo, longlist, need_hdr, op->cmplst,
-                                op->ffmt, tmout);
+                                op->fmtpinfo, longlist, need_param_lst,
+                                op->cmplst, op->ffmt, tmout);
                 }
         } else
                 res = sg_ll_format_unit_v2(fd, op->fmtpinfo, longlist,
-                                           need_hdr, op->cmplst, 0, op->ffmt,
-                                           tmout, param, param_sz, true, vb);
+                                           need_param_lst, op->cmplst, 0,
+                                           op->ffmt, tmout, param, param_sz,
+                                           true, vb);
         if (free_param)
             free(free_param);
 
@@ -485,7 +568,7 @@ scsi_format_medium(int fd, const struct opts_t * op)
                 tmout = op->timeout;
         if (op->dry_run) {
                 res = 0;
-                pr2serr("Due to --dry-run option bypassing FORMAT UNIT "
+                pr2serr("Due to --dry-run option bypassing FORMAT MEDIUM "
                         "command\n");
         } else
                 res = sg_ll_format_medium(fd, op->verify, immed,
@@ -572,6 +655,114 @@ scsi_format_medium(int fd, const struct opts_t * op)
         return 0;
 }
 
+/* Return 0 on success, else see sg_ll_format_medium() above */
+static int
+scsi_format_with_preset(int fd, const struct opts_t * op)
+{
+        int res, progress, pr, rem, resp_len, tmout;
+        int vb = op->verbose;
+        bool immed = ! op->fwait;
+        char b[80];
+
+        if (immed)
+                tmout = SHORT_TIMEOUT;
+        else {
+                if (op->total_byte_count > EIGHT_TBYTE)
+                        tmout = VLONG_FORMAT_TIMEOUT;
+                else if (op->total_byte_count > FOUR_TBYTE)
+                        tmout = LONG_FORMAT_TIMEOUT;
+                else
+                        tmout = FORMAT_TIMEOUT;
+        }
+        if (op->timeout > tmout)
+                tmout = op->timeout;
+        if (op->dry_run) {
+                res = 0;
+                pr2serr("Due to --dry-run option bypassing FORMAT WITH "
+                        "PRESET command\n");
+        } else
+                res = sg_ll_format_with_preset(fd, immed, op->fmtmaxlba,
+                                               op->p_id, tmout, true, vb);
+        if (res) {
+                sg_get_category_sense_str(res, sizeof(b), b, vb);
+                pr2serr("Format with preset command: %s\n", b);
+                return res;
+        }
+        if (! immed)
+                return 0;
+
+        if (! op->dry_run)
+                printf("\nFormat with preset has started\n");
+        if (op->early) {
+                if (immed)
+                        printf("Format continuing,\n    Request sense can "
+                               "be used to monitor progress\n");
+                return 0;
+        }
+
+        if (op->dry_run) {
+                printf("No point in polling for progress, so exit\n");
+                return 0;
+        }
+        if (! op->poll_type) {
+                for(;;) {
+                        sleep_for(POLL_DURATION_SECS);
+                        progress = -1;
+                        res = sg_ll_test_unit_ready_progress(fd, 0, &progress,
+                                             true, (vb > 1) ? (vb - 1) : 0);
+                        if (progress >= 0) {
+                                pr = (progress * 100) / 65536;
+                                rem = ((progress * 100) % 65536) / 656;
+                                printf("Format in progress, %d.%02d%% done\n",
+                                       pr, rem);
+                        } else
+                                break;
+                }
+        }
+        if (op->poll_type || (SG_LIB_CAT_NOT_READY == res)) {
+                uint8_t * reqSense;
+                uint8_t * free_reqSense = NULL;
+
+                reqSense = sg_memalign(MAX_BUFF_SZ, 0, &free_reqSense, false);
+                if (NULL == reqSense) {
+                        pr2serr("%s: unable to obtain heap for Request "
+                                "Sense\n", __func__);
+                        return sg_convert_errno(ENOMEM);
+                }
+                for(;;) {
+                        sleep_for(POLL_DURATION_SECS);
+                        memset(reqSense, 0x0, MAX_BUFF_SZ);
+                        res = sg_ll_request_sense(fd, false, reqSense,
+                                                  MAX_BUFF_SZ, false,
+                                                  (vb > 1) ? (vb - 1) : 0);
+                        if (res) {
+                                pr2serr("polling with Request Sense command "
+                                        "failed [res=%d]\n", res);
+                                break;
+                        }
+                        resp_len = reqSense[7] + 8;
+                        if (vb > 1) {
+                                pr2serr("Parameter data in hex:\n");
+                                hex2stderr(reqSense, resp_len, 1);
+                        }
+                        progress = -1;
+                        sg_get_sense_progress_fld(reqSense, resp_len,
+                                                  &progress);
+                        if (progress >= 0) {
+                                pr = (progress * 100) / 65536;
+                                rem = ((progress * 100) % 65536) / 656;
+                                printf("Format in progress, %d.%02d%% done\n",
+                                       pr, rem);
+                        } else
+                                break;
+                }
+                if (free_reqSense)
+                        free(free_reqSense);
+        }
+        printf("FORMAT WITH PRESET Complete\n");
+        return 0;
+}
+
 #define VPD_DEVICE_ID 0x83
 #define VPD_ASSOC_LU 0
 #define VPD_ASSOC_TPORT 1
@@ -699,15 +890,9 @@ print_dev_id(int fd, uint8_t * sinq_resp, int max_rlen,
         if (n > (SAFE_STD_INQ_RESP_LEN - 4))
                 n = (SAFE_STD_INQ_RESP_LEN - 4);
         for (k = 0, has_sn = 0, has_di = 0; k < n; ++k) {
-                if (VPD_UNIT_SERIAL_NUM == b[4 + k]) {
-                        if (has_di) {
-                                if (op->verbose)
-                                        pr2serr("VPD_SUPPORTED_VPDS "
-                                                "dis-ordered\n");
-                                goto out;
-                        }
+                if (VPD_UNIT_SERIAL_NUM == b[4 + k])
                         ++has_sn;
-                } else if (VPD_DEVICE_ID == b[4 + k]) {
+                else if (VPD_DEVICE_ID == b[4 + k]) {
                         ++has_di;
                         break;
                 }
@@ -1031,6 +1216,7 @@ static int
 parse_cmd_line(struct opts_t * op, int argc, char **argv)
 {
         int j;
+        int64_t ll;
 
         op->cmplst = true;      /* will be set false if FFMT > 0 */
         op->mode_page = RW_ERROR_RECOVERY_PAGE;
@@ -1041,12 +1227,15 @@ parse_cmd_line(struct opts_t * op, int argc, char **argv)
                 int c;
 
                 c = getopt_long(argc, argv,
-                                "c:C:dDef:FhIlm:M:pP:q:QrRs:St:T:vVwx:y6",
+                                "bc:C:dDeE:f:FhIlm:M:pP:q:QrRs:St:T:vVwx:y6",
                                 long_options, &option_index);
                 if (c == -1)
                         break;
 
                 switch (c) {
+                case 'b':
+                        op->fmtmaxlba = true;
+                        break;
                 case 'c':
                         if (0 == strcmp("-1", optarg))
                                 op->blk_count = -1;
@@ -1072,11 +1261,22 @@ parse_cmd_line(struct opts_t * op, int argc, char **argv)
                         op->dry_run = true;
                         break;
                 case 'D':
-                        op->dcrt = true;
+                        ++op->dcrt;
                         break;
                 case 'e':
                         op->early = true;
                         break;
+                case 'E':
+                        ll = sg_get_llnum(optarg);
+                        if ((ll < 0) || (ll > UINT32_MAX)) {
+                                pr2serr("bad argument to '--preset', need 32 "
+                                        "bit integer\n");
+                                return SG_LIB_SYNTAX_ERROR;
+                        }
+                        op->p_id = (uint32_t)ll;
+                        op->preset = true;
+                        op->poll_type = 1;      /* poll with REQUEST SENSE */
+                        break;
                 case 'f':
                         op->fmtpinfo = sg_get_num(optarg);
                         if ((op->fmtpinfo < 0) || ( op->fmtpinfo > 3)) {
@@ -1238,9 +1438,10 @@ parse_cmd_line(struct opts_t * op, int argc, char **argv)
                 usage();
                 return SG_LIB_SYNTAX_ERROR;
         }
-        if (op->format && (op->tape >= 0)) {
-                pr2serr("Cannot choose both '--format' and '--tape='; disk "
-                        "or tape, choose one only\n");
+        if (((int)(op->format > 0) + (int)(op->tape >= 0) + (int)op->preset)
+            > 1) {
+                pr2serr("Can choose only one of: '--format', '--tape=' and "
+                        "'--preset='\n");
                 return SG_LIB_CONTRADICT;
         }
         if (op->ip_def && op->sec_init) {
@@ -1326,9 +1527,14 @@ main(int argc, char **argv)
                 goto format_only;
 
         ret = print_dev_id(fd, inq_resp, inq_resp_sz, op);
-        if (ret)
-                goto out;
-        pdt = 0x1f & inq_resp[0];
+        if (ret) {
+                if (op->dry_run) {
+                        pr2serr("INQUIRY failed, assume device is a disk\n");
+                        pdt = 0;
+                } else
+                        goto out;
+        } else
+                pdt = 0x1f & inq_resp[0];
         if (op->format) {
                 if ((PDT_DISK != pdt) && (PDT_OPTICAL != pdt) &&
                     (PDT_RBC != pdt)) {
@@ -1345,12 +1551,18 @@ main(int argc, char **argv)
                         goto out;
                 }
                 goto format_med;
-        }
+        } else if (op->preset)
+                goto format_with_pre;
 
         ret = fetch_block_desc(fd, dbuff, &calc_len, &bd_lb_sz, op);
-        if (ret)
-                goto out;
-
+        if (ret) {
+                if (op->dry_run) {
+                        /* pick some numbers ... */
+                        calc_len = 1024 * 1024 * 1024;
+                        bd_lb_sz = 512;
+                } else
+                        goto out;
+        }
         rq_lb_sz = op->lblk_sz;
         if (op->resize || (op->format && ((op->blk_count != 0) ||
               ((rq_lb_sz > 0) && (rq_lb_sz != bd_lb_sz))))) {
@@ -1416,7 +1628,6 @@ again_sp_false:
 
         if (op->format) {
 format_only:
-#if 1
                 if (op->quick)
                         goto skip_f_unit_reconsider;
                 printf("\nA FORMAT UNIT will commence in 15 seconds\n");
@@ -1443,9 +1654,6 @@ skip_f_unit_reconsider:
                                 pr2serr("    try '-v' for more "
                                         "information\n");
                 }
-#else
-                pr2serr("FORMAT UNIT ignored, testing\n");
-#endif
         }
         goto out;
 
@@ -1475,8 +1683,35 @@ skip_f_med_reconsider:
         if (res) {
                 pr2serr("FORMAT MEDIUM failed\n");
                 if (0 == vb)
-                        pr2serr("    try '-v' for more "
-                                "information\n");
+                        pr2serr("    try '-v' for more information\n");
+        }
+        goto out;
+
+format_with_pre:
+        if (op->quick)
+                goto skip_f_with_pre_reconsider;
+        printf("\nA FORMAT WITH PRESET will commence in 15 seconds\n");
+        printf("    ALL data on %s will be DESTROYED\n",
+               op->device_name);
+        printf("        Press control-C to abort\n");
+        sleep_for(5);
+        printf("\nA FORMAT WITH PRESET will commence in 10 seconds\n");
+        printf("    ALL data on %s will be DESTROYED\n",
+               op->device_name);
+        printf("        Press control-C to abort\n");
+        sleep_for(5);
+        printf("\nA FORMAT WITH PRESET will commence in 5 seconds\n");
+        printf("    ALL data on %s will be DESTROYED\n",
+               op->device_name);
+        printf("        Press control-C to abort\n");
+        sleep_for(5);
+skip_f_with_pre_reconsider:
+        res = scsi_format_with_preset(fd, op);
+        ret = res;
+        if (res) {
+                pr2serr("FORMAT WITH PRESET failed\n");
+                if (0 == vb)
+                        pr2serr("    try '-v' for more information\n");
         }
 
 out:
