@@ -1,14 +1,16 @@
-/* A utility program originally written for the Linux OS SCSI subsystem.
-*  Copyright (C) 2000-2016 D. Gilbert
-*  This program is free software; you can redistribute it and/or modify
-*  it under the terms of the GNU General Public License as published by
-*  the Free Software Foundation; either version 2, or (at your option)
-*  any later version.
-
-   This is an auxiliary file holding data tables for the sg_inq utility.
-   It is mainly based on the SCSI SPC-4 document at http://www.t10.org .
-
-*/
+/*
+ * A utility program originally written for the Linux OS SCSI subsystem.
+ *     Copyright (C) 2000-2018 D. Gilbert
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This is an auxiliary file holding data tables for the sg_inq utility.
+ * It is mainly based on the SCSI SPC-5 document at http://www.t10.org .
+ */
 
 #include <unistd.h>
 #include <fcntl.h>
@@ -19,9 +21,11 @@
 #include "config.h"
 #endif
 
+#include "sg_lib.h"
+#include "sg_lib_data.h"
 
 /* Assume index is less than 16 */
-const char * sg_ansi_version_arr[] =
+const char * sg_ansi_version_arr[16] =
 {
     "no conformance claimed",
     "SCSI-1",           /* obsolete, ANSI X3.131-1986 */
@@ -41,19 +45,13 @@ const char * sg_ansi_version_arr[] =
     "reserved [Fh]",
 };
 
-
-/* Copy of structure found in sg_inq.c */
-struct sg_version_descriptor {
-    int value;
-    const char * name;
-};
-
-/* table from SPC-5 revision 08 [sorted numerically (from Annex E.9)] */
-/* Can also be obtained from : http://www.t10.org/lists/stds.txt 20160125 */
-
+/* table from SPC-5 revision 16 [sorted numerically (from Annex E.9)] */
+/* Can also be obtained from : http://www.t10.org/lists/stds.txt 20170114 */
+/* Corrected against spc5r21 on 20190312 */
 
 #ifdef SG_SCSI_STRINGS
-struct sg_version_descriptor sg_version_descriptor_arr[] = {
+
+struct sg_lib_simple_value_name_t sg_version_descriptor_arr[] = {
     {0x0, "Version Descriptor not supported or No standard identified"},
     {0x20, "SAM (no version claimed)"},
     {0x3b, "SAM T10/0994-D revision 18"},
@@ -77,6 +75,7 @@ struct sg_version_descriptor sg_version_descriptor_arr[] = {
     {0xa2, "SAM-5 T10/2104-D revision 4"},
     {0xa4, "SAM-5 T10/2104-D revision 20"},
     {0xa6, "SAM-5 T10/2104-D revision 21"},
+    {0xa8, "SAM-5 ANSI INCITS 515-2016"},
     {0xc0, "SAM-6 (no version claimed)"},
     {0x120, "SPC (no version claimed)"},
     {0x13b, "SPC T10/0995-D revision 11a"},
@@ -198,7 +197,7 @@ struct sg_version_descriptor sg_version_descriptor_arr[] = {
     {0x46c, "SPC-4 ANSI INCITS 513-2015"},
     {0x480, "SMC-3 (no version claimed)"},
     {0x482, "SMC-3 T10/1730-D revision 15"},
-    {0x482, "SMC-3 T10/1730-D revision 16"},
+    {0x484, "SMC-3 T10/1730-D revision 16"},
     {0x486, "SMC-3 ANSI INCITS 484-2012"},
     {0x4a0, "ADC-2 (no version claimed)"},
     {0x4a7, "ADC-2 T10/1741-D revision 7"},
@@ -225,17 +224,21 @@ struct sg_version_descriptor sg_version_descriptor_arr[] = {
     {0x527, "SSC-4 SSC-4 ANSI INCITS 516-2013"},
     {0x560, "OSD-3 (no version claimed)"},
     {0x580, "SES-3 (no version claimed)"},
+    {0x582, "SES-3 T10/BSR INCITS 518 revision 13"},
+    {0x584, "SES-3 T10/BSR INCITS 518 revision 14"},
     {0x5a0, "SSC-5 (no version claimed)"},
     {0x5c0, "SPC-5 (no version claimed)"},
     {0x5e0, "SFSC (no version claimed)"},
     {0x5e3, "SFSC BSR INCITS 501 revision 01"},
     {0x5e5, "SFSC BSR INCITS 501 revision 02"},
+    {0x5e8, "SFSC ANSI INCITS 501-2016"},
     {0x600, "SBC-4 (no version claimed)"},
     {0x620, "ZBC (no version claimed)"},
     {0x622, "ZBC BSR INCITS 536 revision 02"},
     {0x624, "ZBC BSR INCITS 536 revision 05"},
     {0x640, "ADC-4 (no version claimed)"},
     {0x660, "ZBC-2 (no version claimed)"},
+    {0x680, "SES-4 (no version claimed)"},
     {0x820, "SSA-TL2 (no version claimed)"},
     {0x83b, "SSA-TL2 T10/1147-D revision 05b"},
     {0x83c, "SSA-TL2 ANSI INCITS 308-1998"},
@@ -361,6 +364,7 @@ struct sg_version_descriptor sg_version_descriptor_arr[] = {
     {0xc65, "SAS-3 T10/BSR INCITS 519 revision 06"},
     {0xc68, "SAS-3 ANSI INCITS 519-2014"},
     {0xc80, "SAS-4 (no version claimed)"},
+    {0xc82, "SAS-4 T10/BSR INCITS 534 revision 08a"},
     {0xd20, "FC-PH (no version claimed)"},
     {0xd3b, "FC-PH ANSI INCITS 230-1994"},
     {0xd3c, "FC-PH ANSI INCITS 230-1994 with Amnd 1 ANSI INCITS "
@@ -402,6 +406,7 @@ struct sg_version_descriptor sg_version_descriptor_arr[] = {
     {0xe6e, "FC-PI-3 ANSI INCITS 460-2011"},
     {0xe80, "FC-PI-4 (no version claimed)"},
     {0xe82, "FC-PI-4 T11/1647-D revision 8.0"},
+    {0xe88, "FC-PI-4 ANSI INCITS 450 -2009"},
     {0xea0, "FC 10GFC (no version claimed)"},
     {0xea2, "FC 10GFC ANSI INCITS 364-2003"},
     {0xea3, "FC 10GFC ISO/IEC 14165-116"},
@@ -472,9 +477,6 @@ struct sg_version_descriptor sg_version_descriptor_arr[] = {
              "claimed)"},
     {0x1628, "ATA/ATAPI-8 ATA-AAM ANSI INCITS 451-2008"},
     {0x162a, "ATA/ATAPI-8 ATA8-ACS ANSI INCITS 452-2009 w/ Amendment 1"},
-    {0x1721, "ACS-2 (no version claimed)"},
-    {0x1722, "ACS-2 ANSI INCITS 482-2013"},
-    {0x1726, "ACS-3 (no version claimed)"},
     {0x1728, "Universal Serial Bus Specification, Revision 1.1"},
     {0x1729, "Universal Serial Bus Specification, Revision 2.0"},
     {0x1730, "USB Mass Storage Class Bulk-Only Transport, Revision 1.0"},
@@ -483,6 +485,10 @@ struct sg_version_descriptor sg_version_descriptor_arr[] = {
     {0x1747, "UAS T10/2095-D revision 04"},
     {0x1748, "UAS ANSI INCITS 471-2010"},
     {0x1749, "UAS ISO/IEC 14776-251:2014"},
+    {0x1761, "ACS-2 (no version claimed)"},
+    {0x1762, "ACS-2 ANSI INCITS 482-2013"},
+    {0x1765, "ACS-3 INCITS 522-2014"},
+    {0x1767, "ACS-4 INCITS 529-2018"},
     {0x1780, "UAS-2 (no version claimed)"},
     {0x1ea0, "SAT (no version claimed)"},
     {0x1ea7, "SAT T10/1711-D rev 8"},
@@ -497,6 +503,8 @@ struct sg_version_descriptor sg_version_descriptor_arr[] = {
     {0x1ee4, "SAT-3 T10/BSR INCITS 517 revision 7"},
     {0x1ee8, "SAT-3 ANSI INCITS 517-2015"},
     {0x1f00, "SAT-4 (no version claimed)"},
+    {0x1f02, "SAT-4 T10/BSR INCITS 491 revision 5"},
+    {0x1f04, "SAT-4 T10/BSR INCITS 491 revision 6"},
     {0x20a0, "SPL (no version claimed)"},
     {0x20a3, "SPL T10/2124-D revision 6a"},
     {0x20a5, "SPL T10/2124-D revision 7"},
@@ -512,6 +520,10 @@ struct sg_version_descriptor sg_version_descriptor_arr[] = {
     {0x20e6, "SPL-3 T10/BSR INCITS 492 revision 7"},
     {0x20e8, "SPL-3 ANSI INCITS 492-2015"},
     {0x2100, "SPL-4 (no version claimed)"},
+    {0x2102, "SPL-4 T10/BSR INCITS 538 revision 08a"},
+    {0x2104, "SPL-4 T10/BSR INCITS 538 revision 10"},
+    {0x2105, "SPL-4 T10/BSR INCITS 538 revision 11"},
+    {0x2120, "SPL-5 (no version claimed)"},
     {0x21e0, "SOP (no version claimed)"},
     {0x21e4, "SOP T10/BSR INCITS 489 revision 4"},
     {0x21e6, "SOP T10/BSR INCITS 489 revision 5"},
@@ -523,13 +535,19 @@ struct sg_version_descriptor sg_version_descriptor_arr[] = {
     {0x2220, "SOP-2 (no draft published)"},
     {0x2240, "PQI-2 (no version claimed)"},
     {0x2242, "PQI-2 T10/BSR INCITS 507 revision 01"},
+    {0x2244, "PQI-2 PQI-2 ANSI INCITS 507-2016"},
     {0xffc0, "IEEE 1667 (no version claimed)"},
     {0xffc1, "IEEE 1667-2006"},
     {0xffc2, "IEEE 1667-2009"},
-    {0xffff, NULL},
+    {0xffc3, "IEEE 1667-2015"},
+    {0xffc4, "IEEE 1667-2018"},
+    {0xffff, NULL},     /* sentinel, leave at end */
 };
+
 #else
-struct sg_version_descriptor sg_version_descriptor_arr[] = {
-    {0xffff, NULL},
+
+struct sg_lib_simple_value_name_t sg_version_descriptor_arr[] = {
+    {0xffff, NULL},     /* sentinel, leave at end */
 };
+
 #endif
