@@ -1,15 +1,19 @@
 /*
- * Copyright (c) 2004-2015 Douglas Gilbert.
+ * Copyright (c) 2004-2020 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <unistd.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
+#include <errno.h>
 #include <ctype.h>
 #include <getopt.h>
 #define __STDC_FORMAT_MACROS 1
@@ -30,7 +34,7 @@
  * and decodes the response.
  */
 
-static const char * version_str = "1.30 20151219";
+static const char * version_str = "1.44 20200115";
 
 #define MAX_RLUNS_BUFF_LEN (1024 * 1024)
 #define DEF_RLUNS_BUFF_LEN (1024 * 8)
@@ -44,6 +48,7 @@ static struct option long_options[] = {
         {"linux", no_argument, 0, 'l'},
 #endif
         {"lu_cong", no_argument, 0, 'L'},
+        {"lu-cong", no_argument, 0, 'L'},
         {"maxlen", required_argument, 0, 'm'},
         {"quiet", no_argument, 0, 'q'},
         {"raw", no_argument, 0, 'r'},
@@ -74,7 +79,8 @@ usage()
             "                  [--verbose] [--version] DEVICE\n");
 #endif
     pr2serr("     or\n"
-            "       sg_luns    --test=ALUN [--hex] [--lu_cong] [--verbose]\n"
+            "       sg_luns    --test=ALUN [--decode] [--hex] [--lu_cong] "
+            "[--verbose]\n"
             "  where:\n"
             "    --decode|-d        decode all luns into component parts\n"
             "    --help|-h          print out usage message\n"
@@ -122,29 +128,27 @@ usage()
  * defines its own "bridge addressing method" in place of the SAM-3
  * "logical addressing method".  */
 static void
-decode_lun(const char * leadin, const unsigned char * lunp, int lu_cong,
+decode_lun(const char * leadin, const uint8_t * lunp, bool lu_cong,
            int do_hex, int verbose)
 {
-    int k, j, x, a_method, bus_id, target, lun, len_fld, e_a_method;
-    int next_level, lu_cong_admin;
-    unsigned char not_spec[8] = {0xff, 0xff, 0xff, 0xff,
-                                 0xff, 0xff, 0xff, 0xff};
+    bool next_level, admin_lu_cong;
+    int k, x, a_method, bus_id, target, lun, len_fld, e_a_method;
+    uint64_t ull;
     char l_leadin[128];
     char b[256];
-    uint64_t ull;
 
-    if (0 == memcmp(lunp, not_spec, sizeof(not_spec))) {
-        printf("%sLogical unit not specified\n", leadin);
+    if (0xff == lunp[0]) {
+        printf("%sLogical unit _not_ specified\n", leadin);
         return;
     }
-    lu_cong_admin = lu_cong;
+    admin_lu_cong = lu_cong;
     memset(l_leadin, 0, sizeof(l_leadin));
     for (k = 0; k < 4; ++k, lunp += 2) {
-        next_level = 0;
+        next_level = false;
         strncpy(l_leadin, leadin, sizeof(l_leadin) - 3);
         if (k > 0) {
             if (lu_cong) {
-                lu_cong_admin = 0;
+                admin_lu_cong = false;
                 if ((0 == lunp[0]) && (0 == lunp[1])) {
                     printf("%s>>>> Administrative LU\n", l_leadin);
                     if (do_hex || verbose)
@@ -172,8 +176,8 @@ decode_lun(const char * leadin, const unsigned char * lunp, int lu_cong,
                     printf("%s0x%04x\n", b, x);
                 else
                     printf("%s%d\n", b, x);
-                if (lu_cong_admin)
-                    next_level = 1;
+                if (admin_lu_cong)
+                    next_level = true;
             } else {
                 bus_id = lunp[0] & 0x3f;
                 snprintf(b, sizeof(b), "%sPeripheral device addressing: ",
@@ -192,7 +196,7 @@ decode_lun(const char * leadin, const unsigned char * lunp, int lu_cong,
                                (bus_id ? "target" : "lun"), lunp[1]);
                 }
                 if (bus_id)
-                    next_level = 1;
+                    next_level = true;
             }
             break;
         case 1:         /* flat space addressing method */
@@ -248,6 +252,9 @@ decode_lun(const char * leadin, const unsigned char * lunp, int lu_cong,
                 case 5:
                     printf("%sMANAGEMENT PROTOCOL %s\n", l_leadin, b);
                     break;
+                case 6:
+                    printf("%sTARGET COMMANDS %s\n", l_leadin, b);
+                    break;
                 default:
                     if (do_hex)
                         printf("%s%s 0x%02x\n", l_leadin, b, x);
@@ -264,12 +271,7 @@ decode_lun(const char * leadin, const unsigned char * lunp, int lu_cong,
                     printf("%sExtended flat space addressing: lun=%d\n",
                            l_leadin, x);
             } else if ((2 == len_fld) && (2 == e_a_method)) {
-                ull = 0;
-                for (j = 0; j < 5; ++j) {
-                    if (j > 0)
-                        ull <<= 8;
-                    ull |= lunp[1 + j];
-                }
+                ull = sg_get_unaligned_be(5, lunp + 1);
                 if (do_hex)
                     printf("%sLong extended flat space addressing: "
                            "lun=0x%010" PRIx64 "\n", l_leadin, ull);
@@ -292,13 +294,8 @@ decode_lun(const char * leadin, const unsigned char * lunp, int lu_cong,
                                "length=%d, e.a. method=%d, value=%d\n",
                                l_leadin, len_fld, e_a_method, x);
                 } else {
-                    ull = 0;
-                    x = (2 == len_fld) ? 5 : 7;
-                    for (j = 0; j < x; ++j) {
-                        if (j > 0)
-                            ull <<= 8;
-                        ull |= lunp[1 + j];
-                    }
+                    ull = sg_get_unaligned_be(((2 == len_fld) ? 5 : 7),
+                                              lunp + 1);
                     if (do_hex) {
                         printf("%sExtended logical unit addressing: "
                                "length=%d, e. a. method=%d, ", l_leadin,
@@ -314,9 +311,6 @@ decode_lun(const char * leadin, const unsigned char * lunp, int lu_cong,
                 }
             }
             break;
-        default:
-            printf("%s<<decode_lun: faulty logic>>\n", l_leadin);
-            break;
         }
         if (next_level)
             continue;
@@ -329,7 +323,7 @@ decode_lun(const char * leadin, const unsigned char * lunp, int lu_cong,
 
 #ifdef SG_LIB_LINUX
 static void
-linux2t10_lun(uint64_t linux_lun, unsigned char t10_lun[])
+linux2t10_lun(uint64_t linux_lun, uint8_t t10_lun[])
 {
     int k;
 
@@ -338,10 +332,10 @@ linux2t10_lun(uint64_t linux_lun, unsigned char t10_lun[])
 }
 
 static uint64_t
-t10_2linux_lun(const unsigned char t10_lun[])
+t10_2linux_lun(const uint8_t t10_lun[])
 {
     int k;
-    const unsigned char * cp;
+    const uint8_t * cp;
     uint64_t res;
 
     res = sg_get_unaligned_be16(t10_lun + 6);
@@ -349,64 +343,51 @@ t10_2linux_lun(const unsigned char t10_lun[])
         res = (res << 16) + sg_get_unaligned_be16(cp);
     return res;
 }
-
-/* Copy of t10_lun --> Linux unsigned int (i.e. 32 bit ) present in Linux
- * kernel, up to least lk 3.8.0, extended to 64 bits.
- * BEWARE: for sizeof(int==4) this function is BROKEN and is left here as
- * as example and may soon be removed. */
-static uint64_t
-t10_2linux_lun64bitBR(const unsigned char t10_lun[])
-{
-    int i;
-    uint64_t lun;
-
-    lun = 0;
-    for (i = 0; i < (int)sizeof(lun); i += 2)
-        lun = lun | (((t10_lun[i] << 8) | t10_lun[i + 1]) << (i * 8));
-    return lun;
-}
 #endif  /* SG_LIB_LINUX */
 
 
 static void
-dStrRaw(const char* str, int len)
+dStrRaw(const char * str, int len)
 {
     int k;
 
-    for (k = 0 ; k < len; ++k)
+    for (k = 0; k < len; ++k)
         printf("%c", str[k]);
 }
 
 int
 main(int argc, char * argv[])
 {
-    int sg_fd, k, m, off, res, c, list_len, len_cap, luns, trunc;
-    int decode = 0;
-    int do_hex = 0;
 #ifdef SG_LIB_LINUX
-    int do_linux = 0;
+    bool do_linux = false;
 #endif
-    int lu_cong = 0;
-    int lu_cong_given = 0;
+    bool do_quiet = false;
+    bool do_raw = false;
+    bool lu_cong_arg_given = false;
+    bool o_readonly = false;
+#ifdef SG_LIB_LINUX
+    bool test_linux_in = false;
+    bool test_linux_out = false;
+#endif
+    bool trunc;
+    bool verbose_given = false;
+    bool version_given = false;
+    int sg_fd, k, m, off, res, c, list_len, len_cap, luns;
+    int decode_arg = 0;
+    int do_hex = 0;
+    int lu_cong_arg = 0;
     int maxlen = 0;
-    int do_quiet = 0;
-    int do_raw = 0;
-    int o_readonly = 0;
+    int ret = 0;
     int select_rep = 0;
     int verbose = 0;
-#ifdef SG_LIB_LINUX
-    int test_linux_in = 0;
-    int test_linux_out = 0;
-    int test_linux_out2 = 0;
-#endif
     unsigned int h;
     const char * test_arg = NULL;
     const char * device_name = NULL;
     const char * cp;
-    unsigned char lun_arr[8];
+    uint8_t * reportLunsBuff = NULL;
+    uint8_t * free_reportLunsBuff = NULL;
+    uint8_t lun_arr[8];
     struct sg_simple_inquiry_resp sir;
-    unsigned char * reportLunsBuff = NULL;
-    int ret = 0;
 
     while (1) {
         int option_index = 0;
@@ -423,7 +404,7 @@ main(int argc, char * argv[])
 
         switch (c) {
         case 'd':
-            decode = 1;
+            ++decode_arg;
             break;
         case 'h':
         case '?':
@@ -434,12 +415,12 @@ main(int argc, char * argv[])
             break;
 #ifdef SG_LIB_LINUX
         case 'l':
-            ++do_linux;
+            do_linux = false;
             break;
 #endif
         case 'L':
-            ++lu_cong;
-            ++lu_cong_given;
+            ++lu_cong_arg;
+            lu_cong_arg_given = true;
             break;
         case 'm':
             maxlen = sg_get_num(optarg);
@@ -450,13 +431,13 @@ main(int argc, char * argv[])
             }
             break;
         case 'q':
-            ++do_quiet;
+            do_quiet = true;
             break;
         case 'r':
-            ++do_raw;
+            do_raw = true;
             break;
         case 'R':
-            ++o_readonly;
+            o_readonly = true;
             break;
         case 's':
            select_rep = sg_get_num(optarg);
@@ -469,11 +450,12 @@ main(int argc, char * argv[])
             test_arg = optarg;
             break;
         case 'v':
+            verbose_given = true;
             ++verbose;
             break;
         case 'V':
-            pr2serr("version: %s\n", version_str);
-            return 0;
+            version_given = true;
+            break;
         default:
             pr2serr("unrecognised option code 0x%x ??\n", c);
             usage();
@@ -492,6 +474,26 @@ main(int argc, char * argv[])
             return SG_LIB_SYNTAX_ERROR;
         }
     }
+#ifdef DEBUG
+    pr2serr("In DEBUG mode, ");
+    if (verbose_given && version_given) {
+        pr2serr("but override: '-vV' given, zero verbose and continue\n");
+        verbose_given = false;
+        version_given = false;
+        verbose = 0;
+    } else if (! verbose_given) {
+        pr2serr("set '-vv'\n");
+        verbose = 2;
+    } else
+        pr2serr("keep verbose=%d\n", verbose);
+#else
+    if (verbose_given && version_given)
+        pr2serr("Not in DEBUG mode, so '-vV' has no special action\n");
+#endif
+    if (version_given) {
+        pr2serr("version: %s\n", version_str);
+        return 0;
+    }
 
     if (test_arg) {
         memset(lun_arr, 0, sizeof(lun_arr));
@@ -501,29 +503,36 @@ main(int argc, char * argv[])
         if ('L' == toupper(cp[0])) {
             uint64_t ull;
 
-            if (1 != sscanf(cp + 1, " %" SCNu64, &ull)) {
+            if (('0' == cp[1]) && ('X' == toupper(cp[2])))
+                k = sscanf(cp + 3, " %" SCNx64, &ull);
+            else
+                k = sscanf(cp + 1, " %" SCNu64, &ull);
+            if (1 != k) {
                 pr2serr("Unable to read Linux style LUN integer given to "
                         "--test=\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
             linux2t10_lun(ull, lun_arr);
-            test_linux_in = 1;
+            test_linux_in = true;
         } else
 #endif
         {
-            /* Check if trailing 'L' or 'W' */
-            m = strlen(cp);    /* must be at least 1 char in test_arg */
+            /* Check if trailing 'L' */
 #ifdef SG_LIB_LINUX
+            m = strlen(cp);    /* must be at least 1 char in test_arg */
             if ('L' == toupper(cp[m - 1]))
-                test_linux_out = 1;
-            else if ('W' == toupper(cp[m - 1]))
-                test_linux_out2 = 1;
+                test_linux_out = true;
 #endif
             if (('0' == cp[0]) && ('X' == toupper(cp[1])))
                 cp += 2;
-            if (strchr(cp, ' ') || strchr(cp, '\t')) {
-                for (k = 0; k < 8; ++k, cp += m) {
-                    if (1 != sscanf(cp, " %2x%n", &h, &m))
+            if (strchr(cp, ' ') || strchr(cp, '\t') || strchr(cp, '-')) {
+                for (k = 0; k < 8; ++k, cp += 2) {
+                    c = *cp;
+                    if ('\0' == c)
+                        break;
+                    else if (! isxdigit(c))
+                        ++cp;
+                    if (1 != sscanf(cp, "%2x", &h))
                         break;
                     lun_arr[k] = h & 0xff;
                 }
@@ -541,14 +550,21 @@ main(int argc, char * argv[])
             }
         }
 #ifdef SG_LIB_LINUX
-        if (verbose || test_linux_in || test_linux_out2)
+        if (verbose || test_linux_in || decode_arg)
 #else
-        if (verbose)
+        if (verbose || decode_arg)
 #endif
         {
-            printf("64 bit LUN in T10 preferred (hex) format: ");
-            for (k = 0; k < 8; ++k)
-                printf(" %02x", lun_arr[k]);
+            if (decode_arg > 1) {
+                printf("64 bit LUN in T10 (hex, dashed) format: ");
+                for (k = 0; k < 8; k += 2)
+                    printf("%c%02x%02x", (k ? '-' : ' '), lun_arr[k],
+                           lun_arr[k + 1]);
+            } else {
+                printf("64 bit LUN in T10 preferred (hex) format: ");
+                for (k = 0; k < 8; ++k)
+                    printf(" %02x", lun_arr[k]);
+            }
             printf("\n");
         }
 #ifdef SG_LIB_LINUX
@@ -562,20 +578,10 @@ main(int argc, char * argv[])
             else
                 printf("Linux 'word flipped' integer LUN representation: %"
                        PRIu64 "\n", t10_2linux_lun(lun_arr));
-        } else if (test_linux_out2) {
-            if (do_hex > 1)
-                printf("Linux internal 64 bit LUN representation: 0x%016"
-                       PRIx64 "\n", t10_2linux_lun64bitBR(lun_arr));
-            else if (do_hex)
-                printf("Linux internal 64 bit LUN representation: 0x%"
-                       PRIx64 "\n", t10_2linux_lun64bitBR(lun_arr));
-            else
-                printf("Linux internal 64 bit LUN representation: %"
-                       PRIu64 "\n", t10_2linux_lun64bitBR(lun_arr));
         }
 #endif
         printf("Decoded LUN:\n");
-        decode_lun("  ", lun_arr, (lu_cong % 2), do_hex, verbose);
+        decode_lun("  ", lun_arr, (lu_cong_arg % 2), do_hex, verbose);
         return 0;
     }
     if (NULL == device_name) {
@@ -594,31 +600,35 @@ main(int argc, char * argv[])
     sg_fd = sg_cmds_open_device(device_name, o_readonly, verbose);
     if (sg_fd < 0) {
         pr2serr("open error: %s: %s\n", device_name, safe_strerror(-sg_fd));
-        return SG_LIB_FILE_ERROR;
+        return sg_convert_errno(-sg_fd);
     }
-    if (decode && (! lu_cong_given)) {
+    if (decode_arg && (! lu_cong_arg_given)) {
+        if (verbose > 1)
+            pr2serr("in order to decode LUN and since --lu_cong not given, "
+                    "do standard\nINQUIRY to find LU_CONG bit\n");
         /* check if LU_CONG set in standard INQUIRY response */
-        res = sg_simple_inquiry(sg_fd, &sir, 0, verbose);
+        res = sg_simple_inquiry(sg_fd, &sir, false, verbose);
         ret = res;
         if (res) {
             pr2serr("fetching standard INQUIRY response failed\n");
             goto the_end;
         }
-        lu_cong = !!(0x40 & sir.byte_1);
-        if (verbose && lu_cong)
+        lu_cong_arg = !!(0x40 & sir.byte_1);
+        if (verbose && lu_cong_arg)
             pr2serr("LU_CONG bit set in standard INQUIRY response\n");
     }
 
     if (0 == maxlen)
         maxlen = DEF_RLUNS_BUFF_LEN;
-    reportLunsBuff = (unsigned char *)calloc(1, maxlen);
+    reportLunsBuff = (uint8_t *)sg_memalign(maxlen, 0, &free_reportLunsBuff,
+                                            verbose > 3);
     if (NULL == reportLunsBuff) {
-        pr2serr("unable to malloc %d bytes\n", maxlen);
-        return SG_LIB_CAT_OTHER;
+        pr2serr("unable to sg_memalign %d bytes\n", maxlen);
+        return sg_convert_errno(ENOMEM);
     }
-    trunc = 0;
+    trunc = false;
 
-    res = sg_ll_report_luns(sg_fd, select_rep, reportLunsBuff, maxlen, 1,
+    res = sg_ll_report_luns(sg_fd, select_rep, reportLunsBuff, maxlen, true,
                             verbose);
     ret = res;
     if (0 == res) {
@@ -631,26 +641,25 @@ main(int argc, char * argv[])
             goto the_end;
         }
         if (1 == do_hex) {
-            dStrHex((const char *)reportLunsBuff, len_cap, 1);
+            hex2stdout(reportLunsBuff, len_cap, 1);
             goto the_end;
         }
         luns = (list_len / 8);
-        if (0 == do_quiet)
+        if (! do_quiet)
             printf("Lun list length = %d which imples %d lun entr%s\n",
                    list_len, luns, ((1 == luns) ? "y" : "ies"));
         if ((list_len + 8) > maxlen) {
             luns = ((maxlen - 8) / 8);
-            trunc = 1;
+            trunc = true;
             pr2serr("  <<too many luns for internal buffer, will show %d "
                     "lun%s>>\n", luns, ((1 == luns) ? "" : "s"));
         }
         if (verbose > 1) {
             pr2serr("\nOutput response in hex\n");
-            dStrHexErr((const char *)reportLunsBuff,
-                       (trunc ? maxlen : list_len + 8), 1);
+            hex2stderr(reportLunsBuff, (trunc ? maxlen : list_len + 8), 1);
         }
         for (k = 0, off = 8; k < luns; ++k, off += 8) {
-            if (0 == do_quiet) {
+            if (! do_quiet) {
                 if (0 == k)
                     printf("Report luns [select_report=0x%x]:\n", select_rep);
                 printf("    ");
@@ -669,9 +678,9 @@ main(int argc, char * argv[])
             }
 #endif
             printf("\n");
-            if (decode)
-                decode_lun("      ", reportLunsBuff + off, (lu_cong % 2),
-                           do_hex, verbose);
+            if (decode_arg)
+                decode_lun("      ", reportLunsBuff + off,
+                           (bool)(lu_cong_arg % 2), do_hex, verbose);
         }
     } else if (SG_LIB_CAT_INVALID_OP == res)
         pr2serr("Report Luns command not supported (support mandatory in "
@@ -688,13 +697,18 @@ main(int argc, char * argv[])
     }
 
 the_end:
-    if (reportLunsBuff)
-        free(reportLunsBuff);
+    if (free_reportLunsBuff)
+        free(free_reportLunsBuff);
     res = sg_cmds_close_device(sg_fd);
     if (res < 0) {
         pr2serr("close error: %s\n", safe_strerror(-res));
         if (0 == ret)
-            return SG_LIB_FILE_ERROR;
+            return sg_convert_errno(-res);
+    }
+    if (0 == verbose) {
+        if (! sg_if_can2stderr("sg_luns failed: ", ret))
+            pr2serr("Some error occurred, try again with '-v' or '-vv' for "
+                    "more information\n");
     }
     return (ret >= 0) ? ret : SG_LIB_CAT_OTHER;
 }
