@@ -1,10 +1,12 @@
 /*
- *  Copyright (C) 2000-2018 D. Gilbert
+ *  Copyright (C) 2000-2020 D. Gilbert
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
  *  This program outputs information provided by a SCSI MODE SENSE command.
  *  Does 10 byte MODE SENSE commands by default, Trent Piepho added a "-6"
  *  switch for force 6 byte mode sense commands.
@@ -30,7 +32,7 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
-static const char * version_str = "1.66 20180909";
+static const char * version_str = "1.71 20200121";
 
 #define DEF_ALLOC_LEN (1024 * 4)
 #define DEF_6_ALLOC_LEN 252
@@ -116,6 +118,8 @@ static struct page_code_desc pc_desc_common[] = {
     {0xa, 0x1, "coe", "Control extension"},
     {0xa, 0x3, "cdla", "Command duration limit A"},
     {0xa, 0x4, "cdlb", "Command duration limit B"},
+    {0xa, 0x7, "cdt2a", "Command duration limit T2A"},  /* spc6r01 */
+    {0xa, 0x8, "cdt2b", "Command duration limit T2B"},  /* spc6r01 */
     {0x15, 0x0, "ext_", "Extended"},
     {0x16, 0x0, "edts", "Extended device-type specific"},
     {0x18, 0x0, "pslu", "Protocol specific lu"},
@@ -133,7 +137,7 @@ static struct page_code_desc pc_desc_disk[] = {
     {0x1, 0x0, "rw", "Read-Write error recovery"},
     {0x3, 0x0, "fo", "Format (obsolete)"},
     {0x4, 0x0, "rd", "Rigid disk geometry (obsolete)"},
-    {0x5, 0x0, "fg", "Flexible geometry (obsolete)"},
+    {0x5, 0x0, "fd", "Flexible disk (obsolete)"},
     {0x7, 0x0, "ve", "Verify error recovery"},
     {0x8, 0x0, "ca", "Caching"},
     {0xa, 0x2, "atag", "Application tag"},
@@ -249,6 +253,16 @@ static struct page_code_desc pc_desc_t_adc[] = {
     {0x0, 0x0, NULL, NULL},
 };
 
+static struct page_code_desc pc_desc_zbc[] = {
+    {0x1, 0x0, "rw", "Read-Write error recovery"},
+    {0x7, 0x0, "ve", "Verify error recovery"},
+    {0x8, 0x0, "ca", "Caching"},
+    {0xa, 0x2, "atag", "Application tag"},
+    {0xa, 0xf, "zbdct", "Zoned block device control"},  /* zbc2r04a */
+    {0x1c, 0x1, "bc", "Background control"},
+    {0x0, 0x0, NULL, NULL},
+};
+
 struct pc_desc_group pcd_gr_arr[] = {
     {pc_desc_common, "common"},
     {pc_desc_disk, "disk"},
@@ -259,6 +273,7 @@ struct pc_desc_group pcd_gr_arr[] = {
     {pc_desc_ses, "enclosure"},
     {pc_desc_rbc, "reduced block"},
     {pc_desc_adc, "adc"},
+    {pc_desc_zbc, "zbc"},
     {pc_desc_t_fcp, "transport: FCP"},
     {pc_desc_t_spi4, "transport: SPI"},
     {pc_desc_t_sas, "transport: SAS"},
@@ -817,6 +832,9 @@ get_mpage_tbl_size(int scsi_ptype, int * sizep)
         case PDT_ADC:       /* automation device/interface */
             *sizep = count_desc_elems(pc_desc_adc);
             return &pc_desc_adc[0];
+        case PDT_ZBC:
+            *sizep = count_desc_elems(pc_desc_zbc);
+            return &pc_desc_zbc[0];
     }
     *sizep = 0;
     return NULL;
@@ -1417,7 +1435,7 @@ main(int argc, char * argv[])
             hex2stdout(rsp_buff, headerlen, 1);
             goto fini;
         }
-        if (0 == inq_pdt)
+        if ((PDT_DISK == inq_pdt) || (PDT_ZBC == inq_pdt))
             printf("  Mode data length=%d, medium type=0x%.2x, WP=%d,"
                    " DpoFua=%d, longlba=%d\n", md_len, medium_type,
                    !!(specific & 0x80), !!(specific & 0x10), (int)longlba);
@@ -1444,7 +1462,7 @@ main(int argc, char * argv[])
                     len = 16;
                     density_code_off = 8;
                 }
-                else if (0 == inq_pdt) {
+                else if ((PDT_DISK == inq_pdt) || (PDT_ZBC == inq_pdt)) {
                     printf("> Direct access device block descriptors:\n");
                     density_code_off = 4;
                 }
