@@ -1,13 +1,15 @@
 /*
  * A utility program originally written for the Linux OS SCSI subsystem.
  *
- * Copyright (C) 2000-2018 Ingo van Lil <inguin@gmx.de>
+ * Copyright (C) 2000-2019 Ingo van Lil <inguin@gmx.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  *
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
  * This program can be used to send raw SCSI commands (with an optional
  * data phase) through a Generic SCSI interface.
  */
@@ -37,7 +39,7 @@
 #include "sg_pr2serr.h"
 #include "sg_unaligned.h"
 
-#define SG_RAW_VERSION "0.4.27 (2018-06-27)"
+#define SG_RAW_VERSION "0.4.32 (2019-12-26)"
 
 #define DEFAULT_TIMEOUT 20
 #define MIN_SCSI_CDBSZ 6
@@ -150,199 +152,6 @@ usage()
             "  sg_raw -r 1k /dev/sg0 12 00 00 00 60 00\n");
 }
 
-/* Read ASCII hex bytes or binary from fname (a file named '-' taken as
- * stdin). If reading ASCII hex then there should be either one entry per
- * line or a comma, space or tab separated list of bytes. If no_space is
- * true then a string of ACSII hex digits is expected, 2 per byte. Everything
- * from and including a '#' on a line is ignored. Returns true if ok, or
- * false if error. */
-static bool
-f2hex_arr(const char * fname, bool as_binary, bool no_space,
-          uint8_t * mp_arr, int * mp_arr_len, int max_arr_len)
-{
-    int fn_len, in_len, k, j, m, fd;
-    bool has_stdin, split_line;
-    unsigned int h;
-    const char * lcp;
-    FILE * fp;
-    char line[512];
-    char carry_over[4];
-    int off = 0;
-    struct stat a_stat;
-
-    if ((NULL == fname) || (NULL == mp_arr) || (NULL == mp_arr_len))
-        return false;
-    fn_len = strlen(fname);
-    if (0 == fn_len)
-        return false;
-    has_stdin = ((1 == fn_len) && ('-' == fname[0]));   /* read from stdin */
-    if (as_binary) {
-        if (has_stdin)
-            fd = STDIN_FILENO;
-        else {
-            fd = open(fname, O_RDONLY);
-            if (fd < 0) {
-                pr2serr("unable to open binary file %s: %s\n", fname,
-                         safe_strerror(errno));
-                return false;
-            }
-        }
-        k = read(fd, mp_arr, max_arr_len);
-        if (k <= 0) {
-            if (0 == k)
-                pr2serr("read 0 bytes from binary file %s\n", fname);
-            else
-                pr2serr("read from binary file %s: %s\n", fname,
-                        safe_strerror(errno));
-            if (! has_stdin)
-                close(fd);
-            return false;
-        }
-        if ((0 == fstat(fd, &a_stat)) && S_ISFIFO(a_stat.st_mode)) {
-            /* pipe; keep reading till error or 0 read */
-            while (k < max_arr_len) {
-                m = read(fd, mp_arr + k, max_arr_len - k);
-                if (0 == m)
-                   break;
-                if (m < 0) {
-                    pr2serr("read from binary pipe %s: %s\n", fname,
-                            safe_strerror(errno));
-                    if (! has_stdin)
-                        close(fd);
-                    return false;
-                }
-                k += m;
-            }
-        }
-        *mp_arr_len = k;
-        if (! has_stdin)
-            close(fd);
-        return true;
-    } else {    /* So read the file as ASCII hex */
-        if (has_stdin)
-            fp = stdin;
-        else {
-            fp = fopen(fname, "r");
-            if (NULL == fp) {
-                pr2serr("Unable to open %s for reading\n", fname);
-                return false;
-            }
-        }
-    }
-
-    carry_over[0] = 0;
-    for (j = 0; j < 512; ++j) {
-        if (NULL == fgets(line, sizeof(line), fp))
-            break;
-        in_len = strlen(line);
-        if (in_len > 0) {
-            if ('\n' == line[in_len - 1]) {
-                --in_len;
-                line[in_len] = '\0';
-                split_line = false;
-            } else
-                split_line = true;
-        }
-        if (in_len < 1) {
-            carry_over[0] = 0;
-            continue;
-        }
-        if (carry_over[0]) {
-            if (isxdigit(line[0])) {
-                carry_over[1] = line[0];
-                carry_over[2] = '\0';
-                if (1 == sscanf(carry_over, "%4x", &h))
-                    mp_arr[off - 1] = h;       /* back up and overwrite */
-                else {
-                    pr2serr("%s: carry_over error ['%s'] around line %d\n",
-                            __func__, carry_over, j + 1);
-                    goto bad;
-                }
-                lcp = line + 1;
-                --in_len;
-            } else
-                lcp = line;
-            carry_over[0] = 0;
-        } else
-            lcp = line;
-
-        m = strspn(lcp, " \t");
-        if (m == in_len)
-            continue;
-        lcp += m;
-        in_len -= m;
-        if ('#' == *lcp)
-            continue;
-        k = strspn(lcp, "0123456789aAbBcCdDeEfF ,\t");
-        if ((k < in_len) && ('#' != lcp[k]) && ('\r' != lcp[k])) {
-            pr2serr("%s: syntax error at line %d, pos %d\n", __func__,
-                    j + 1, m + k + 1);
-            goto bad;
-        }
-        if (no_space) {
-            for (k = 0; isxdigit(*lcp) && isxdigit(*(lcp + 1));
-                 ++k, lcp += 2) {
-                if (1 != sscanf(lcp, "%2x", &h)) {
-                    pr2serr("%s: bad hex number in line %d, pos %d\n",
-                            __func__, j + 1, (int)(lcp - line + 1));
-                    goto bad;
-                }
-                if ((off + k) >= max_arr_len) {
-                    pr2serr("%s: array length exceeded\n", __func__);
-                    goto bad;
-                }
-                mp_arr[off + k] = h;
-            }
-            if (isxdigit(*lcp) && (! isxdigit(*(lcp + 1))))
-                carry_over[0] = *lcp;
-            off += k;
-        } else {
-            for (k = 0; k < 1024; ++k) {
-                if (1 == sscanf(lcp, "%10x", &h)) {
-                    if (h > 0xff) {
-                        pr2serr("%s: hex number larger than 0xff in line "
-                                "%d, pos %d\n", __func__, j + 1,
-                                (int)(lcp - line + 1));
-                        goto bad;
-                    }
-                    if (split_line && (1 == strlen(lcp))) {
-                        /* single trailing hex digit might be a split pair */
-                        carry_over[0] = *lcp;
-                    }
-                    if ((off + k) >= max_arr_len) {
-                        pr2serr("%s: array length exceeded\n", __func__);
-                        goto bad;
-                    }
-                    mp_arr[off + k] = h;
-                    lcp = strpbrk(lcp, " ,\t");
-                    if (NULL == lcp)
-                        break;
-                    lcp += strspn(lcp, " ,\t");
-                    if ('\0' == *lcp)
-                        break;
-                } else {
-                    if (('#' == *lcp) || ('\r' == *lcp)) {
-                        --k;
-                        break;
-                    }
-                    pr2serr("%s: error in line %d, at pos %d\n", __func__,
-                            j + 1, (int)(lcp - line + 1));
-                    goto bad;
-                }
-            }
-            off += (k + 1);
-        }
-    }
-    *mp_arr_len = off;
-    if (stdin != fp)
-        fclose(fp);
-    return true;
-bad:
-    if (stdin != fp)
-        fclose(fp);
-    return false;
-}
-
 static int
 parse_cmd_line(struct opts_t * op, int argc, char *argv[])
 {
@@ -465,9 +274,9 @@ parse_cmd_line(struct opts_t * op, int argc, char *argv[])
     if (op->cmdfile_given) {
         bool ok;
 
-        ok = f2hex_arr(op->cmd_file, (op->raw > 0) /* as_binary */,
-                       false /* no_space */, op->cdb, &op->cdb_length,
-                       MAX_SCSI_CDBSZ);
+        ok = sg_f2hex_arr(op->cmd_file, (op->raw > 0) /* as_binary */,
+                          false /* no_space */, op->cdb, &op->cdb_length,
+                          MAX_SCSI_CDBSZ);
         if (! ok)
             return SG_LIB_SYNTAX_ERROR;
         if (op->verbose > 2) {
@@ -538,7 +347,7 @@ static uint8_t *
 fetch_dataout(struct opts_t * op, uint8_t ** free_buf, int * errp)
 {
     bool ok = false;
-    int fd, len, err;
+    int fd, len, tot_len, boff, err;
     uint8_t *buf = NULL;
 
     *free_buf = NULL;
@@ -572,28 +381,29 @@ fetch_dataout(struct opts_t * op, uint8_t ** free_buf, int * errp)
         }
     }
 
-    buf = sg_memalign(op->dataout_len, 0 /* page_size */, free_buf,
-                      op->verbose > 3);
+    tot_len = op->dataout_len;
+    buf = sg_memalign(tot_len, 0 /* page_size */, free_buf, false);
     if (buf == NULL) {
-        pr2serr("sg_memalign: failed to get %d bytes of memory\n",
-                op->dataout_len);
+        pr2serr("sg_memalign: failed to get %d bytes of memory\n", tot_len);
         if (errp)
             *errp = sg_convert_errno(ENOMEM);
         goto bail;
     }
 
-    len = read(fd, buf, op->dataout_len);
-    if (len < 0) {
-        err = errno;
-        if (errp)
-            *errp = sg_convert_errno(err);
-        perror("Failed to read input data");
-        goto bail;
-    } else if (len < op->dataout_len) {
-        if (errp)
-            *errp = SG_LIB_FILE_ERROR;
-        pr2serr("EOF on input file/stream\n");
-        goto bail;
+    for (boff = 0; boff < tot_len; boff += len) {
+        len = read(fd, buf + boff , tot_len - boff);
+        if (len < 0) {
+            err = errno;
+            if (errp)
+                *errp = sg_convert_errno(err);
+            perror("Failed to read input data");
+            goto bail;
+        } else if (0 == len) {
+            if (errp)
+                *errp = SG_LIB_FILE_ERROR;
+            pr2serr("EOF on input file/stream at buffer offset %d\n", boff);
+            goto bail;
+        }
     }
     ok = true;
 
@@ -745,8 +555,7 @@ main(int argc, char *argv[])
     if (op->do_datain) {
         uint32_t din_len = op->datain_len;
 
-        dinp = sg_memalign(din_len, 0 /* page_size */, &wrkBuf,
-                           op->verbose > 3);
+        dinp = sg_memalign(din_len, 0 /* page_size */, &wrkBuf, false);
         if (dinp == NULL) {
             pr2serr("sg_memalign: failed to get %d bytes of memory\n",
                     din_len);
@@ -768,16 +577,13 @@ main(int argc, char *argv[])
         }
     }
     if (op->verbose) {
+        char d[128];
+
         pr2serr("    %s to send: ", is_scsi_cdb ? "cdb" : "cmd");
         if (is_scsi_cdb) {
-            for (k = 0; k < op->cdb_length; ++k)
-                pr2serr("%02x ", op->cdb[k]);
-            pr2serr("\n");
-            if (op->verbose > 1) {
-                sg_get_command_name(op->cdb, 0, b_len - 1, b);
-                b[b_len - 1] = '\0';
-                pr2serr("    Command name: %s\n", b);
-            }
+            pr2serr("%s\n", sg_get_command_str(op->cdb, op->cdb_length,
+                                               op->verbose > 1,
+                                               sizeof(d), d));
         } else {        /* If not SCSI cdb then treat as NVMe command */
             pr2serr("\n");
             hex2stderr(op->cdb, op->cdb_length, -1);
