@@ -1,10 +1,12 @@
 /* A utility program originally written for the Linux OS SCSI subsystem.
- *  Copyright (C) 1999-2018 D. Gilbert
+ *  Copyright (C) 1999-2019 D. Gilbert
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
  * This program uses the SCSI command READ BUFFER on the given
  * device, first to find out how big it is and then to read that
  * buffer (data mode, buffer id 0).
@@ -55,7 +57,7 @@
 #endif
 
 
-static const char * version_str = "5.05 20180627";
+static const char * version_str = "5.07 20191226";
 
 static struct option long_options[] = {
         {"buffer", required_argument, 0, 'b'},
@@ -357,7 +359,7 @@ main(int argc, char * argv[])
     bool clear = true;
 #endif
     bool dio_incomplete = false;
-    int sg_fd, res, j, err;
+    int sg_fd, res, err;
     int buf_capacity = 0;
     int buf_size = 0;
     size_t psz;
@@ -449,11 +451,11 @@ main(int argc, char * argv[])
     io_hdr.sbp = sense_buffer;
     io_hdr.timeout = 60000;     /* 60000 millisecs == 60 seconds */
     if (op->verbose) {
-        pr2serr("    Read buffer (%sdescriptor) cdb: ",
-                (op->do_echo ? "echo " : ""));
-        for (k = 0; k < RB_CMD_LEN; ++k)
-            pr2serr("%02x ", rb_cdb[k]);
-        pr2serr("\n");
+        char b[128];
+
+        pr2serr("    Read buffer (%sdescriptor) cdb: %s\n",
+                (op->do_echo ? "echo " : ""),
+                sg_get_command_str(rb_cdb, RB_CMD_LEN, false, sizeof(b), b));
     }
 
     /* do normal IO to find RB size (not dio or mmap-ed at this stage) */
@@ -585,13 +587,13 @@ main(int argc, char * argv[])
         else if (op->do_quick)
             io_hdr.flags |= SG_FLAG_NO_DXFER;
         if (op->verbose > 1) {
-            pr2serr("    Read buffer (%sdata) cdb: ",
-                    (op->do_echo ? "echo " : ""));
-            for (j = 0; j < RB_CMD_LEN; ++j)
-                pr2serr("%02x ", rb_cdb[j]);
-            pr2serr("\n");
-        }
+            char b[128];
 
+            pr2serr("    Read buffer (%sdata) cdb: %s\n",
+                    (op->do_echo ? "echo " : ""),
+                    sg_get_command_str(rb_cdb, RB_CMD_LEN, false,
+                                       sizeof(b), b));
+        }
         if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
             if (ENOMEM == errno) {
                 pr2serr("SG_IO data: out of memory, try a smaller buffer "
@@ -681,5 +683,5 @@ main(int argc, char * argv[])
     else
         printf("read buffer non-zero\n");
 #endif
-    return (res >= 0) ? res : SG_LIB_CAT_OTHER;
+    return res;
 }
