@@ -1,17 +1,20 @@
-/* A utility program for the Linux OS SCSI generic ("sg") device driver.
-*  Copyright (C) 2001 - 2018 D. Gilbert
-*  This program is free software; you can redistribute it and/or modify
-*  it under the terms of the GNU General Public License as published by
-*  the Free Software Foundation; either version 2, or (at your option)
-*  any later version.
+/*
+ *  A utility program for the Linux OS SCSI generic ("sg") device driver.
+ *    Copyright (C) 2001 - 2019 D. Gilbert
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later
 
    This program reads data from the given SCSI device (typically a disk
    or cdrom) and discards that data. Its primary goal is to time
    multiple reads all starting from the same logical address. Its interface
    is a subset of another member of this package: sg_dd which is a
    "dd" variant. The input file can be a scsi generic device, a block device,
-   a raw device or a seekable file. Streams such as stdin are not acceptable.
-   The block size ('bs') is assumed to be 512 if not given.
+   or a seekable file. Streams such as stdin are not acceptable. The block
+   size ('bs') is assumed to be 512 if not given.
 
    This version should compile with Linux sg drivers with version numbers
    >= 30000 . For mmap-ed IO the sg version number >= 30122 .
@@ -55,7 +58,7 @@
 #include "sg_pr2serr.h"
 
 
-static const char * version_str = "1.34 20180811";
+static const char * version_str = "1.36 20191220";
 
 #define DEF_BLOCK_SIZE 512
 #define DEF_BLOCKS_PER_TRANSFER 128
@@ -75,7 +78,7 @@ static const char * version_str = "1.34 20180811";
 #define RAW_MAJOR 255   /*unlikey value */
 #endif
 
-#define FT_OTHER 1              /* filetype other than sg or raw device */
+#define FT_OTHER 1              /* filetype other than sg and ... */
 #define FT_SG 2                 /* filetype is sg char device */
 #define FT_RAW 4                /* filetype is raw char device */
 #define FT_BLOCK 8              /* filetype is block device */
@@ -292,7 +295,6 @@ sg_bread(int sg_fd, uint8_t * buff, int blocks, int64_t from_block, int bs,
          int cdbsz, bool fua, bool dpo, bool * diop, bool do_mmap,
          bool no_dxfer)
 {
-    int k;
     uint8_t rdCmd[MAX_SCSI_CDBSZ];
     uint8_t senseBuff[SENSE_BUFF_LEN];
     struct sg_io_hdr io_hdr;
@@ -326,10 +328,10 @@ sg_bread(int sg_fd, uint8_t * buff, int blocks, int64_t from_block, int bs,
     io_hdr.timeout = DEF_TIMEOUT;
     io_hdr.pack_id = pack_id_count++;
     if (verbose > 1) {
-        pr2serr( "    read cdb: ");
-        for (k = 0; k < cdbsz; ++k)
-            pr2serr( "%02x ", rdCmd[k]);
-        pr2serr( "\n");
+        char b[128];
+
+        pr2serr("    READ cdb: %s\n",
+                sg_get_command_str(rdCmd, cdbsz, false, sizeof(b), b));
     }
 
     if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
@@ -488,17 +490,19 @@ main(int argc, char * argv[])
             dpo = !! sg_get_num(buf);
         else if (0 == strcmp(key,"fua"))
             fua = !! sg_get_num(buf);
-        else if (strcmp(key,"if") == 0)
-            strncpy(inf, buf, INF_SZ - 1);
-        else if (0 == strcmp(key,"mmap"))
+        else if (strcmp(key,"if") == 0) {
+            memcpy(inf, buf, INF_SZ - 1);
+            inf[INF_SZ - 1] = '\0';
+        } else if (0 == strcmp(key,"mmap"))
             do_mmap = !! sg_get_num(buf);
         else if (0 == strcmp(key,"no_dxfer"))
             no_dxfer = !! sg_get_num(buf);
         else if (0 == strcmp(key,"odir"))
             do_odir = !! sg_get_num(buf);
-        else if (strcmp(key,"of") == 0)
-            strncpy(outf, buf, INF_SZ - 1);
-        else if (0 == strcmp(key,"skip")) {
+        else if (strcmp(key,"of") == 0) {
+            memcpy(outf, buf, INF_SZ - 1);
+            outf[INF_SZ - 1] = '\0';
+        } else if (0 == strcmp(key,"skip")) {
             skip = sg_get_llnum(buf);
             if (-1 == skip) {
                 pr2serr( ME "bad argument to 'skip'\n");
