@@ -1,16 +1,20 @@
 /*
- * Copyright (c) 2016 Douglas Gilbert.
+ * Copyright (c) 2016-2019 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <unistd.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <stdbool.h>
 #include <string.h>
+#include <errno.h>
 #include <ctype.h>
 #include <getopt.h>
 #define __STDC_FORMAT_MACROS 1
@@ -20,6 +24,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
 #include "sg_lib.h"
 #include "sg_lib_data.h"
 #include "sg_pt.h"
@@ -34,7 +39,7 @@
  * and decodes the response. Based on spc5r08.pdf
  */
 
-static const char * version_str = "1.00 20160207";
+static const char * version_str = "1.13 20191220";
 
 #define MAX_RATTR_BUFF_LEN (1024 * 1024)
 #define DEF_RATTR_BUFF_LEN (1024 * 8)
@@ -60,9 +65,13 @@ static const char * version_str = "1.00 20160207";
 #define DEF_PT_TIMEOUT  60      /* 60 seconds */
 
 struct opts_t {
-    int cache;
-    int ea;
-    int enumerate;
+    bool cache;
+    bool enumerate;
+    bool do_raw;
+    bool o_readonly;
+    bool verbose_given;
+    bool version_given;
+    int elem_addr;
     int filter;
     int fai;
     int do_hex;
@@ -70,8 +79,6 @@ struct opts_t {
     int maxlen;
     int pn;
     int quiet;
-    int do_raw;
-    int o_readonly;
     int sa;
     int verbose;
 };
@@ -189,7 +196,7 @@ usage()
     pr2serr("Usage: sg_read_attr [--cache] [--element=EA] [--enumerate] "
             "[--filter=FL]\n"
             "                    [--first=FAI] [--help] [--hex] [--in=FN] "
-            "[--lvn-LVN]\n"
+            "[--lvn=LVN]\n"
             "                    [--maxlen=LEN] [--partition=PN] [--quiet] "
             "[--raw]\n"
             "                    [--readonly] [--sa=SA] [--verbose] "
@@ -227,41 +234,42 @@ usage()
             "    --sa=SA|-s SA      SA is service action (def: 0)\n"
             "    --verbose|-v       increase verbosity\n"
             "    --version|-V       print version string and exit\n\n"
-            "Performs a SCSI READ ATTRIBUTE command. It is typically used "
-            "on tape\nsystems.\n");
+            "Performs a SCSI READ ATTRIBUTE command. Even though it is "
+            "defined in\nSPC-3 and later it is typically used on tape "
+            "systems.\n");
 }
 
 /* Invokes a SCSI READ ATTRIBUTE command (SPC+SMC).  Return of 0 -> success,
  * various SG_LIB_CAT_* positive values or -1 -> other errors */
 static int
-sg_ll_read_attr(int sg_fd, void * resp, int * residp,
+sg_ll_read_attr(int sg_fd, void * resp, int * residp, bool noisy,
                 const struct opts_t * op)
 {
-    int k, ret, res, sense_cat;
-    int noisy = 1;
-    unsigned char raCmdBlk[SG_READ_ATTRIBUTE_CMDLEN] =
+    int ret, res, sense_cat;
+    uint8_t ra_cdb[SG_READ_ATTRIBUTE_CMDLEN] =
           {SG_READ_ATTRIBUTE_CMD, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,
            0, 0, 0, 0};
-    unsigned char sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN];
     struct sg_pt_base * ptvp;
 
-    raCmdBlk[1] = 0x1f & op->sa;
-    if (op->ea)
-        sg_put_unaligned_be16(op->ea, raCmdBlk + 2);
+    ra_cdb[1] = 0x1f & op->sa;
+    if (op->elem_addr)
+        sg_put_unaligned_be16(op->elem_addr, ra_cdb + 2);
     if (op->lvn)
-        raCmdBlk[5] = 0xff & op->lvn;
+        ra_cdb[5] = 0xff & op->lvn;
     if (op->pn)
-        raCmdBlk[7] = 0xff & op->pn;
+        ra_cdb[7] = 0xff & op->pn;
     if (op->fai)
-        sg_put_unaligned_be16(op->fai, raCmdBlk + 8);
-    sg_put_unaligned_be32((uint32_t)op->maxlen, raCmdBlk + 10);
+        sg_put_unaligned_be16(op->fai, ra_cdb + 8);
+    sg_put_unaligned_be32((uint32_t)op->maxlen, ra_cdb + 10);
     if (op->cache)
-        raCmdBlk[14] |= 0x1;
+        ra_cdb[14] |= 0x1;
     if (op->verbose) {
-        pr2serr("    Read attribute cdb: ");
-        for (k = 0; k < SG_READ_ATTRIBUTE_CMDLEN; ++k)
-            pr2serr("%02x ", raCmdBlk[k]);
-        pr2serr("\n");
+        char b[128];
+
+        pr2serr("    Read attribute cdb: %s\n",
+                sg_get_command_str(ra_cdb, SG_READ_ATTRIBUTE_CMDLEN, false,
+                                   sizeof(b), b));
     }
 
     ptvp = construct_scsi_pt_obj();
@@ -269,14 +277,14 @@ sg_ll_read_attr(int sg_fd, void * resp, int * residp,
         pr2serr("%s: out of memory\n", __func__);
         return -1;
     }
-    set_scsi_pt_cdb(ptvp, raCmdBlk, sizeof(raCmdBlk));
+    set_scsi_pt_cdb(ptvp, ra_cdb, sizeof(ra_cdb));
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
-    set_scsi_pt_data_in(ptvp, (unsigned char *)resp, op->maxlen);
+    set_scsi_pt_data_in(ptvp, (uint8_t *)resp, op->maxlen);
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, op->verbose);
-    ret = sg_cmds_process_resp(ptvp, "read attribute", res, op->maxlen,
-                               sense_b, noisy, op->verbose, &sense_cat);
+    ret = sg_cmds_process_resp(ptvp, "read attribute", res, noisy,
+                               op->verbose, &sense_cat);
     if (-1 == ret)
-        ;
+        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
     else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
@@ -296,11 +304,11 @@ sg_ll_read_attr(int sg_fd, void * resp, int * residp,
 }
 
 static void
-dStrRaw(const char* str, int len)
+dStrRaw(const char * str, int len)
 {
     int k;
 
-    for (k = 0 ; k < len; ++k)
+    for (k = 0; k < len; ++k)
         printf("%c", str[k]);
 }
 
@@ -366,193 +374,15 @@ enum_sa_acrons(void)
         printf("  %d:\t\t%s\t%s\n", anvp->val, anvp->acron, anvp->name);
 }
 
-/* Read ASCII hex bytes or binary from fname (a file named '-' taken as
- * stdin). If reading ASCII hex then there should be either one entry per
- * line or a comma, space or tab separated list of bytes. If no_space is
- * set then a string of ACSII hex digits is expected, 2 per byte. Everything
- * from and including a '#' on a line is ignored. Returns 0 if ok, or 1 if
- * error. */
-static int
-f2hex_arr(const char * fname, int as_binary, int no_space,
-          uint8_t * mp_arr, int * mp_arr_len, int max_arr_len)
-{
-    int fn_len, in_len, k, j, m, split_line, fd, has_stdin;
-    unsigned int h;
-    const char * lcp;
-    FILE * fp;
-    char line[512];
-    char carry_over[4];
-    int off = 0;
-
-    if ((NULL == fname) || (NULL == mp_arr) || (NULL == mp_arr_len))
-        return 1;
-    fn_len = strlen(fname);
-    if (0 == fn_len)
-        return 1;
-    has_stdin = ((1 == fn_len) && ('-' == fname[0]));  /* read from stdin */
-    if (as_binary) {
-        if (has_stdin) {
-            fd = STDIN_FILENO;
-                if (sg_set_binary_mode(STDIN_FILENO) < 0)
-                    perror("sg_set_binary_mode");
-        } else {
-            fd = open(fname, O_RDONLY);
-            if (fd < 0) {
-                pr2serr("unable to open binary file %s: %s\n", fname,
-                         safe_strerror(errno));
-                return 1;
-            } else if (sg_set_binary_mode(fd) < 0)
-                perror("sg_set_binary_mode");
-        }
-        k = read(fd, mp_arr, max_arr_len);
-        if (k <= 0) {
-            if (0 == k)
-                pr2serr("read 0 bytes from binary file %s\n", fname);
-            else
-                pr2serr("read from binary file %s: %s\n", fname,
-                        safe_strerror(errno));
-            if (! has_stdin)
-                close(fd);
-            return 1;
-        }
-        *mp_arr_len = k;
-        if (! has_stdin)
-            close(fd);
-        return 0;
-    } else {    /* So read the file as ASCII hex */
-        if (has_stdin)
-            fp = stdin;
-        else {
-            fp = fopen(fname, "r");
-            if (NULL == fp) {
-                pr2serr("Unable to open %s for reading\n", fname);
-                return 1;
-            }
-        }
-    }
-
-    carry_over[0] = 0;
-    for (j = 0; j < 512; ++j) {
-        if (NULL == fgets(line, sizeof(line), fp))
-            break;
-        in_len = strlen(line);
-        if (in_len > 0) {
-            if ('\n' == line[in_len - 1]) {
-                --in_len;
-                line[in_len] = '\0';
-                split_line = 0;
-            } else
-                split_line = 1;
-        }
-        if (in_len < 1) {
-            carry_over[0] = 0;
-            continue;
-        }
-        if (carry_over[0]) {
-            if (isxdigit(line[0])) {
-                carry_over[1] = line[0];
-                carry_over[2] = '\0';
-                if (1 == sscanf(carry_over, "%4x", &h))
-                    mp_arr[off - 1] = h;       /* back up and overwrite */
-                else {
-                    pr2serr("%s: carry_over error ['%s'] around line %d\n",
-                            __func__, carry_over, j + 1);
-                    goto bad;
-                }
-                lcp = line + 1;
-                --in_len;
-            } else
-                lcp = line;
-            carry_over[0] = 0;
-        } else
-            lcp = line;
-
-        m = strspn(lcp, " \t");
-        if (m == in_len)
-            continue;
-        lcp += m;
-        in_len -= m;
-        if ('#' == *lcp)
-            continue;
-        k = strspn(lcp, "0123456789aAbBcCdDeEfF ,\t");
-        if ((k < in_len) && ('#' != lcp[k]) && ('\r' != lcp[k])) {
-            pr2serr("%s: syntax error at line %d, pos %d\n", __func__,
-                    j + 1, m + k + 1);
-            goto bad;
-        }
-        if (no_space) {
-            for (k = 0; isxdigit(*lcp) && isxdigit(*(lcp + 1));
-                 ++k, lcp += 2) {
-                if (1 != sscanf(lcp, "%2x", &h)) {
-                    pr2serr("%s: bad hex number in line %d, pos %d\n",
-                            __func__, j + 1, (int)(lcp - line + 1));
-                    goto bad;
-                }
-                if ((off + k) >= max_arr_len) {
-                    pr2serr("%s: array length exceeded\n", __func__);
-                    goto bad;
-                }
-                mp_arr[off + k] = h;
-            }
-            if (isxdigit(*lcp) && (! isxdigit(*(lcp + 1))))
-                carry_over[0] = *lcp;
-            off += k;
-        } else {
-            for (k = 0; k < 1024; ++k) {
-                if (1 == sscanf(lcp, "%4x", &h)) {
-                    if (h > 0xff) {
-                        pr2serr("%s: hex number larger than 0xff in line %d, "
-                                "pos %d\n", __func__, j + 1,
-                                (int)(lcp - line + 1));
-                        goto bad;
-                    }
-                    if (split_line && (1 == strlen(lcp))) {
-                        /* single trailing hex digit might be a split pair */
-                        carry_over[0] = *lcp;
-                    }
-                    if ((off + k) >= max_arr_len) {
-                        pr2serr("%s: array length exceeded\n", __func__);
-                        goto bad;
-                    }
-                    mp_arr[off + k] = h;
-                    lcp = strpbrk(lcp, " ,\t");
-                    if (NULL == lcp)
-                        break;
-                    lcp += strspn(lcp, " ,\t");
-                    if ('\0' == *lcp)
-                        break;
-                } else {
-                    if (('#' == *lcp) || ('\r' == *lcp)) {
-                        --k;
-                        break;
-                    }
-                    pr2serr("%s: error in line %d, at pos %d\n", __func__,
-                            j + 1, (int)(lcp - line + 1));
-                    goto bad;
-                }
-            }
-            off += (k + 1);
-        }
-    }
-    *mp_arr_len = off;
-    if (stdin != fp)
-        fclose(fp);
-    return 0;
-bad:
-    if (stdin != fp)
-        fclose(fp);
-    return 1;
-}
-
-/* Returns 1 if 'ucp' all 0xff bytes, returns 2 is all 0xff bytes apart
+/* Returns 1 if 'bp' all 0xff bytes, returns 2 is all 0xff bytes apart
  * from last being 0xfe; otherwise returns 0. */
 static int
-all_ffs_or_last_fe(const unsigned char * ucp, int len)
+all_ffs_or_last_fe(const uint8_t * bp, int len)
 {
-    for ( ; len > 0; ++ucp, --len) {
-        if (*ucp < 0xfe)
+    for ( ; len > 0; ++bp, --len) {
+        if (*bp < 0xfe)
             return 0;
-        if (0xfe == *ucp)
+        if (0xfe == *bp)
             return (1 == len) ? 2 : 0;
 
     }
@@ -595,7 +425,7 @@ attr_id_lookup(unsigned int id, const struct attr_name_info_t ** anipp,
 }
 
 static void
-decode_attr_list(const unsigned char * alp, int len, bool supported,
+decode_attr_list(const uint8_t * alp, int len, bool supported,
                  const struct opts_t * op)
 {
     int id;
@@ -609,7 +439,7 @@ decode_attr_list(const unsigned char * alp, int len, bool supported,
     else if (0 == op->quiet)
         printf("%sttribute list:\n", leadin);
     if (op->do_hex) {
-        dStrHex((const char *)alp, len, 0);
+        hex2stdout(alp, len, 0);
         return;
     }
     for ( ; len > 0; alp += 2, len -= 2) {
@@ -632,12 +462,12 @@ decode_attr_list(const unsigned char * alp, int len, bool supported,
 }
 
 static void
-helper_full_attr(const unsigned char * alp, int len, int id,
+helper_full_attr(const uint8_t * alp, int len, int id,
                  const struct attr_name_info_t * anip,
                  const struct opts_t * op)
 {
     int k;
-    const unsigned char * ucp;
+    const uint8_t * bp;
 
     if (op->verbose)
         printf("[r%c] ", (0x80 & alp[2]) ? 'o' : 'w');
@@ -656,7 +486,7 @@ helper_full_attr(const unsigned char * alp, int len, int id,
                 printf("%" PRIx64, sg_get_unaligned_be(len - 5, alp + 5));
             else {
                 printf("\n");
-                dStrHex((const char *)(alp + 5), len - 5, 0);
+                hex2stdout((alp + 5), len - 5, 0);
             }
         }
         break;
@@ -672,96 +502,96 @@ helper_full_attr(const unsigned char * alp, int len, int id,
                 printf("%" PRIx64, sg_get_unaligned_be(len - 5, alp + 5));
             else {
                 printf("\n");
-                dStrHex((const char *)(alp + 5), len - 5, 0);
+                hex2stdout(alp + 5, len - 5, 0);
             }
         }
         break;
     case 0x340:         /* Medium Usage history */
-        ucp = alp + 5;
+        bp = alp + 5;
         printf("\n");
         if ((len - 5) < 90) {
             pr2serr("%s: expected 90 bytes, got %d\n", __func__, len - 5);
             break;
         }
         printf("    Current amount of data written [MiB]: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 0));
+               sg_get_unaligned_be48(bp + 0));
         printf("    Current write retry count: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 6));
+               sg_get_unaligned_be48(bp + 6));
         printf("    Current amount of data read [MiB]: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 12));
+               sg_get_unaligned_be48(bp + 12));
         printf("    Current read retry count: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 18));
+               sg_get_unaligned_be48(bp + 18));
         printf("    Previous amount of data written [MiB]: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 24));
+               sg_get_unaligned_be48(bp + 24));
         printf("    Previous write retry count: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 30));
+               sg_get_unaligned_be48(bp + 30));
         printf("    Previous amount of data read [MiB]: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 36));
+               sg_get_unaligned_be48(bp + 36));
         printf("    Previous read retry count: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 42));
+               sg_get_unaligned_be48(bp + 42));
         printf("    Total amount of data written [MiB]: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 48));
+               sg_get_unaligned_be48(bp + 48));
         printf("    Total write retry count: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 54));
+               sg_get_unaligned_be48(bp + 54));
         printf("    Total amount of data read [MiB]: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 60));
+               sg_get_unaligned_be48(bp + 60));
         printf("    Total read retry count: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 66));
+               sg_get_unaligned_be48(bp + 66));
         printf("    Load count: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 72));
+               sg_get_unaligned_be48(bp + 72));
         printf("    Total change partition count: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 78));
+               sg_get_unaligned_be48(bp + 78));
         printf("    Total partition initialization count: %" PRIu64 "\n",
-               sg_get_unaligned_be48(ucp + 84));
+               sg_get_unaligned_be48(bp + 84));
         break;
     case 0x341:         /* Partition Usage history */
-        ucp = alp + 5;
+        bp = alp + 5;
         printf("\n");
         if ((len - 5) < 60) {
             pr2serr("%s: expected 60 bytes, got %d\n", __func__, len - 5);
             break;
         }
         printf("    Current amount of data written [MiB]: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 0));
+               sg_get_unaligned_be32(bp + 0));
         printf("    Current write retry count: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 4));
+               sg_get_unaligned_be32(bp + 4));
         printf("    Current amount of data read [MiB]: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 8));
+               sg_get_unaligned_be32(bp + 8));
         printf("    Current read retry count: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 12));
+               sg_get_unaligned_be32(bp + 12));
         printf("    Previous amount of data written [MiB]: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 16));
+               sg_get_unaligned_be32(bp + 16));
         printf("    Previous write retry count: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 20));
+               sg_get_unaligned_be32(bp + 20));
         printf("    Previous amount of data read [MiB]: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 24));
+               sg_get_unaligned_be32(bp + 24));
         printf("    Previous read retry count: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 28));
+               sg_get_unaligned_be32(bp + 28));
         printf("    Total amount of data written [MiB]: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 32));
+               sg_get_unaligned_be32(bp + 32));
         printf("    Total write retry count: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 36));
+               sg_get_unaligned_be32(bp + 36));
         printf("    Total amount of data read [MiB]: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 40));
+               sg_get_unaligned_be32(bp + 40));
         printf("    Total read retry count: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 44));
+               sg_get_unaligned_be32(bp + 44));
         printf("    Load count: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 48));
+               sg_get_unaligned_be32(bp + 48));
         printf("    change partition count: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 52));
+               sg_get_unaligned_be32(bp + 52));
         printf("    partition initialization count: %" PRIu32 "\n",
-               sg_get_unaligned_be32(ucp + 56));
+               sg_get_unaligned_be32(bp + 56));
         break;
     default:
         pr2serr("%s: unknown attribute id: 0x%x\n", __func__, id);
         printf("  In hex:\n");
-        dStrHex((const char *)alp, len, 0);
+        hex2stdout(alp, len, 0);
         break;
     }
 }
 
 static void
-decode_attr_vals(const unsigned char * alp, int len, const struct opts_t * op)
+decode_attr_vals(const uint8_t * alp, int len, const struct opts_t * op)
 {
     int bump, id, alen;
     uint64_t ull;
@@ -776,7 +606,7 @@ decode_attr_vals(const unsigned char * alp, int len, const struct opts_t * op)
         if (0 == op->quiet)
             printf("Attribute values:\n");
         if (op->do_hex) {       /* only expect -HH to get through here */
-            dStrHex((const char *)alp, len, 0);
+            hex2stdout(alp, len, 0);
             return;
         }
     }
@@ -821,7 +651,7 @@ decode_attr_vals(const unsigned char * alp, int len, const struct opts_t * op)
                     helper_full_attr(alp, bump, id, anip, op);
                 else {
                     printf("\n");
-                    dStrHex((const char *)(alp + 5), alen, 0);
+                    hex2stdout(alp + 5, alen, 0);
                 }
            } else {
                 if (2 == anip->process)
@@ -841,7 +671,7 @@ decode_attr_vals(const unsigned char * alp, int len, const struct opts_t * op)
                 printf("Attribute id lookup failed, in hex:\n");
             else
                 printf("\n");
-            dStrHex((const char *)(alp + 5), alen, 0);
+            hex2stdout(alp + 5, alen, 0);
         }
     }
     if (op->verbose && (len > 0) && (len <= 4))
@@ -850,10 +680,10 @@ decode_attr_vals(const unsigned char * alp, int len, const struct opts_t * op)
 }
 
 static void
-decode_all_sa_s(const unsigned char * rabp, int len, const struct opts_t * op)
+decode_all_sa_s(const uint8_t * rabp, int len, const struct opts_t * op)
 {
     if (op->do_hex && (2 != op->do_hex)) {
-        dStrHex((const char *)rabp, len, ((1 == op->do_hex) ? 1 : -1));
+        hex2stdout(rabp, len, ((1 == op->do_hex) ? 1 : -1));
         return;
     }
     switch (op->sa) {
@@ -887,7 +717,7 @@ decode_all_sa_s(const unsigned char * rabp, int len, const struct opts_t * op)
         break;
     case RA_SMC2_SA:
         printf("Used by SMC-2, not information, output in hex:\n");
-        dStrHex((const char *)rabp, len, 0);
+        hex2stdout(rabp, len, 0);
         break;
     case RA_SUP_ATTR_SA:
         decode_attr_list(rabp + 4, len - 4, true, op);
@@ -895,7 +725,7 @@ decode_all_sa_s(const unsigned char * rabp, int len, const struct opts_t * op)
     default:
         printf("Unrecognized service action [0x%x], response in hex:\n",
                op->sa);
-        dStrHex((const char *)rabp, len, 0);
+        hex2stdout(rabp, len, 0);
         break;
     }
 }
@@ -903,12 +733,14 @@ decode_all_sa_s(const unsigned char * rabp, int len, const struct opts_t * op)
 int
 main(int argc, char * argv[])
 {
-    int sg_fd, res, c, len, resid, rlen, in_len;
+    int sg_fd, res, c, len, resid, rlen;
     unsigned int ra_len;
+    int in_len = 0;
     int ret = 0;
     const char * device_name = NULL;
     const char * fname = NULL;
-    unsigned char * rabp = NULL;
+    uint8_t * rabp = NULL;
+    uint8_t * free_rabp = NULL;
     struct opts_t opts;
     struct opts_t * op;
     char b[80];
@@ -926,15 +758,15 @@ main(int argc, char * argv[])
 
         switch (c) {
         case 'c':
-            ++op->cache;
+            op->cache = true;
             break;
         case 'e':
-            ++op->enumerate;
+            op->enumerate = true;
             break;
         case 'E':
-           op->ea = sg_get_num(optarg);
-           if ((op->ea < 0) || (op->ea > 65535)) {
-                pr2serr("bad argument to '--ea=EA', expect 0 to 65535\n");
+           op->elem_addr = sg_get_num(optarg);
+           if ((op->elem_addr < 0) || (op->elem_addr > 65535)) {
+                pr2serr("bad argument to '--element=EA', expect 0 to 65535\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
             break;
@@ -989,10 +821,10 @@ main(int argc, char * argv[])
             ++op->quiet;
             break;
         case 'r':
-            ++op->do_raw;
+            op->do_raw = true;
             break;
         case 'R':
-            ++op->o_readonly;
+            op->o_readonly = true;
             break;
         case 's':
            if (isdigit(*optarg)) {
@@ -1011,11 +843,12 @@ main(int argc, char * argv[])
             }
             break;
         case 'v':
+            op->verbose_given = true;
             ++op->verbose;
             break;
         case 'V':
-            pr2serr("version: %s\n", version_str);
-            return 0;
+            op->version_given = true;
+            break;
         default:
             pr2serr("unrecognised option code 0x%x ??\n", c);
             usage();
@@ -1034,6 +867,26 @@ main(int argc, char * argv[])
             return SG_LIB_SYNTAX_ERROR;
         }
     }
+#ifdef DEBUG
+    pr2serr("In DEBUG mode, ");
+    if (op->verbose_given && op->version_given) {
+        pr2serr("but override: '-vV' given, zero verbose and continue\n");
+        op->verbose_given = false;
+        op->version_given = false;
+        op->verbose = 0;
+    } else if (! op->verbose_given) {
+        pr2serr("set '-vv'\n");
+        op->verbose = 2;
+    } else
+        pr2serr("keep verbose=%d\n", op->verbose);
+#else
+    if (op->verbose_given && op->version_given)
+        pr2serr("Not in DEBUG mode, so '-vV' has no special action\n");
+#endif
+    if (op->version_given) {
+        pr2serr("version: %s\n", version_str);
+        return 0;
+    }
 
     if (op->enumerate) {
         enum_attributes();
@@ -1049,35 +902,39 @@ main(int argc, char * argv[])
 
     if (0 == op->maxlen)
         op->maxlen = DEF_RATTR_BUFF_LEN;
-    rabp = (unsigned char *)calloc(1, op->maxlen);
+    rabp = (uint8_t *)sg_memalign(op->maxlen, 0, &free_rabp, op->verbose > 3);
     if (NULL == rabp) {
-        pr2serr("unable to calloc %d bytes\n", op->maxlen);
-        return SG_LIB_CAT_OTHER;
+        pr2serr("unable to sg_memalign %d bytes\n", op->maxlen);
+        return sg_convert_errno(ENOMEM);
     }
 
     if (NULL == device_name) {
         if (fname) {
-            if (f2hex_arr(fname, op->do_raw, 0, rabp, &in_len, op->maxlen))
-                return SG_LIB_FILE_ERROR;
+            if ((ret = sg_f2hex_arr(fname, op->do_raw, false /* no space */,
+                                    rabp, &in_len, op->maxlen)))
+                goto clean_up;
             if (op->do_raw)
-                op->do_raw = 0;    /* can interfere on decode */
+                op->do_raw = false;    /* can interfere on decode */
             if (in_len < 4) {
                 pr2serr("--in=%s only decoded %d bytes (needs 4 at least)\n",
                         fname, in_len);
-                return SG_LIB_SYNTAX_ERROR;
+                ret = SG_LIB_SYNTAX_ERROR;
+                goto clean_up;
             }
             decode_all_sa_s(rabp, in_len, op);
             goto clean_up;
         }
         pr2serr("missing device name!\n");
         usage();
-        return SG_LIB_SYNTAX_ERROR;
+        ret = SG_LIB_SYNTAX_ERROR;
+        goto clean_up;
     }
 
     if (op->do_raw) {
         if (sg_set_binary_mode(STDOUT_FILENO) < 0) {
             perror("sg_set_binary_mode");
-            return SG_LIB_FILE_ERROR;
+            ret = SG_LIB_FILE_ERROR;
+                goto clean_up;
         }
     }
 
@@ -1085,10 +942,11 @@ main(int argc, char * argv[])
     if (sg_fd < 0) {
         pr2serr("open error: %s: %s\n", device_name,
                 safe_strerror(-sg_fd));
-        return SG_LIB_FILE_ERROR;
+        ret = sg_convert_errno(-sg_fd);
+        goto clean_up;
     }
 
-    res = sg_ll_read_attr(sg_fd, rabp, &resid, op);
+    res = sg_ll_read_attr(sg_fd, rabp, &resid, op->verbose > 0, op);
     ret = res;
     if (0 == res) {
         rlen = op->maxlen - resid;
@@ -1129,10 +987,15 @@ close_then_end:
     if (res < 0) {
         pr2serr("close error: %s\n", safe_strerror(-res));
         if (0 == ret)
-            ret = SG_LIB_FILE_ERROR;
+            ret = sg_convert_errno(-res);
     }
 clean_up:
-    if (rabp)
-        free(rabp);
+    if (free_rabp)
+        free(free_rabp);
+    if (0 == op->verbose) {
+        if (! sg_if_can2stderr("sg_read_attr failed: ", ret))
+            pr2serr("Some error occurred, try again with '-v' or '-vv' for "
+                    "more information\n");
+    }
     return (ret >= 0) ? ret : SG_LIB_CAT_OTHER;
 }
