@@ -1,8 +1,10 @@
 /*
- * Copyright (c) 2014-2018 Douglas Gilbert.
+ * Copyright (c) 2014-2020 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <unistd.h>
@@ -36,7 +38,7 @@
  * and decodes the response. Based on zbc-r02.pdf
  */
 
-static const char * version_str = "1.17 20180628";
+static const char * version_str = "1.21 20200220";
 
 #define MAX_RZONES_BUFF_LEN (1024 * 1024)
 #define DEF_RZONES_BUFF_LEN (1024 * 8)
@@ -53,6 +55,7 @@ static struct option long_options[] = {
         {"help", no_argument, 0, 'h'},
         {"hex", no_argument, 0, 'H'},
         {"maxlen", required_argument, 0, 'm'},
+        {"num", required_argument, 0, 'n'},
         {"partial", no_argument, 0, 'p'},
         {"raw", no_argument, 0, 'r'},
         {"readonly", no_argument, 0, 'R'},
@@ -60,6 +63,7 @@ static struct option long_options[] = {
         {"start", required_argument, 0, 's'},
         {"verbose", no_argument, 0, 'v'},
         {"version", no_argument, 0, 'V'},
+        {"wp", no_argument, 0, 'w'},
         {0, 0, 0, 0},
 };
 
@@ -82,6 +86,8 @@ usage(int h)
             "    --maxlen=LEN|-m LEN    max response length (allocation "
             "length in cdb)\n"
             "                           (def: 0 -> 8192 bytes)\n"
+            "    --num=NUM|-n NUM    number of zones to output (def: 0 -> "
+            "all)\n"
             "    --partial|-p       sets PARTIAL bit in cdb (def: 0 -> "
             "zone list\n"
             "                       length not altered by allocation length "
@@ -93,7 +99,8 @@ usage(int h)
             "    --start=LBA|-s LBA    report zones from the LBA (def: 0)\n"
             "                          need not be a zone starting LBA\n"
             "    --verbose|-v       increase verbosity\n"
-            "    --version|-V       print version string and exit\n\n"
+            "    --version|-V       print version string and exit\n"
+            "    --wp|-w            output write pointer only\n\n"
             "Sends a SCSI REPORT ZONES command and decodes the response. "
             "Give\nhelp option twice (e.g. '-hh') to see reporting options "
             "enumerated.\n");
@@ -110,6 +117,7 @@ h_twoormore:
             "    0x5    list zones with a zone condition of FULL\n"
             "    0x6    list zones with a zone condition of READ ONLY\n"
             "    0x7    list zones with a zone condition of OFFLINE\n"
+            "    0x8    list zones with a zone condition of INACTIVE\n"
             "    0x10   list zones with RWP Recommended set to true\n"
             "    0x11   list zones with Non-sequential write resources "
             "active set to true\n"
@@ -124,7 +132,7 @@ sg_ll_report_zones(int sg_fd, uint64_t zs_lba, bool partial, int report_opts,
                    void * resp, int mx_resp_len, int * residp, bool noisy,
                    int verbose)
 {
-    int k, ret, res, sense_cat;
+    int ret, res, sense_cat;
     uint8_t rz_cdb[SG_ZONING_IN_CMDLEN] =
           {SG_ZONING_IN, REPORT_ZONES_SA, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,
            0, 0, 0, 0};
@@ -137,12 +145,12 @@ sg_ll_report_zones(int sg_fd, uint64_t zs_lba, bool partial, int report_opts,
     if (partial)
         rz_cdb[14] |= 0x80;
     if (verbose) {
-        pr2serr("    Report zones cdb: ");
-        for (k = 0; k < SG_ZONING_IN_CMDLEN; ++k)
-            pr2serr("%02x ", rz_cdb[k]);
-        pr2serr("\n");
-    }
+        char b[128];
 
+        pr2serr("    Report zones cdb: %s\n",
+                sg_get_command_str(rz_cdb, SG_ZONING_IN_CMDLEN, false,
+                                   sizeof(b), b));
+    }
     ptvp = construct_scsi_pt_obj();
     if (NULL == ptvp) {
         pr2serr("%s: out of memory\n", __func__);
@@ -152,8 +160,8 @@ sg_ll_report_zones(int sg_fd, uint64_t zs_lba, bool partial, int report_opts,
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     set_scsi_pt_data_in(ptvp, (uint8_t *)resp, mx_resp_len);
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
-    ret = sg_cmds_process_resp(ptvp, "report zones", res, mx_resp_len,
-                               sense_b, noisy, verbose, &sense_cat);
+    ret = sg_cmds_process_resp(ptvp, "report zones", res, noisy, verbose,
+                               &sense_cat);
     if (-1 == ret)
         ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
     else if (-2 == ret) {
@@ -200,6 +208,12 @@ zone_type_str(int zt, char * b, int blen, int vb)
     case 3:
         cp = "Sequential write preferred";
         break;
+    case 4:
+        cp = "Sequential or before required";
+        break;
+    case 5:
+        cp = "Gap";
+        break;
     default:
         cp = NULL;
         break;
@@ -237,6 +251,9 @@ zone_condition_str(int zc, char * b, int blen, int vb)
     case 4:
         cp = "Closed";
         break;
+    case 5:
+        cp = "Inactive";
+        break;
     case 0xd:
         cp = "Read only";
         break;
@@ -279,10 +296,12 @@ main(int argc, char * argv[])
     bool o_readonly = false;
     bool verbose_given = false;
     bool version_given = false;
+    bool wp_only = false;
     int k, res, c, zl_len, len, zones, resid, rlen, zt, zc, same;
     int sg_fd = -1;
     int do_help = 0;
     int do_hex = 0;
+    int do_num = 0;
     int maxlen = 0;
     int reporting_opt = 0;
     int ret = 0;
@@ -298,7 +317,7 @@ main(int argc, char * argv[])
     while (1) {
         int option_index = 0;
 
-        c = getopt_long(argc, argv, "hHm:o:prRs:vV", long_options,
+        c = getopt_long(argc, argv, "hHm:n:o:prRs:vVw", long_options,
                         &option_index);
         if (c == -1)
             break;
@@ -319,6 +338,13 @@ main(int argc, char * argv[])
                 return SG_LIB_SYNTAX_ERROR;
             }
             break;
+        case 'n':
+            do_num = sg_get_num(optarg);
+            if (do_num < 0) {
+                pr2serr("argument to '--num' should be zero or more\n");
+                return SG_LIB_SYNTAX_ERROR;
+            }
+            break;
         case 'o':
            reporting_opt = sg_get_num_nomult(optarg);
            if ((reporting_opt < 0) || (reporting_opt > 63)) {
@@ -351,6 +377,9 @@ main(int argc, char * argv[])
         case 'V':
             version_given = true;
             break;
+        case 'w':
+            wp_only = true;
+            break;
         default:
             pr2serr("unrecognised option code 0x%x ??\n", c);
             usage(1);
@@ -452,7 +481,8 @@ main(int argc, char * argv[])
                     ((1 == do_hex) ? 1 : -1));
             goto the_end;
         }
-        printf("Report zones response:\n");
+        if (! wp_only)
+            printf("Report zones response:\n");
         if (len < 64) {
             pr2serr("Zone length [%d] too short (perhaps after truncation\n)",
                     len);
@@ -460,16 +490,25 @@ main(int argc, char * argv[])
             goto the_end;
         }
         same = reportZonesBuff[4] & 0xf;
-        printf("  Same=%d: %s\n\n", same, same_desc_arr[same]);
-        printf("  Maximum LBA: 0x%" PRIx64 "\n",
-               sg_get_unaligned_be64(reportZonesBuff + 8));
+        if (! wp_only) {
+            printf("  Same=%d: %s\n", same, same_desc_arr[same]);
+            printf("  Maximum LBA: 0x%" PRIx64 "\n\n",
+                   sg_get_unaligned_be64(reportZonesBuff + 8));
+        }
         zones = (len - 64) / 64;
+        if (do_num > 0)
+                zones = (zones > do_num) ? do_num : zones;
         for (k = 0, bp = reportZonesBuff + 64; k < zones; ++k, bp += 64) {
-            printf(" Zone descriptor: %d\n", k);
+            if (! wp_only)
+                printf(" Zone descriptor: %d\n", k);
             if (do_hex) {
                 hex2stdout(bp, len, -1);
                 continue;
             }
+            if (wp_only) {
+                printf("0x%" PRIx64 "\n", sg_get_unaligned_be64(bp + 24));
+                continue;
+            }
             zt = bp[0] & 0xf;
             zc = (bp[1] >> 4) & 0xf;
             printf("   Zone type: %s\n", zone_type_str(zt, b, sizeof(b),
@@ -485,9 +524,11 @@ main(int argc, char * argv[])
             printf("   Write pointer LBA: 0x%" PRIx64 "\n",
                    sg_get_unaligned_be64(bp + 24));
         }
-        if ((64 + (64 * zones)) < zl_len)
-            printf("\n>>> Beware: Zone list truncated, may need another "
-                   "call\n");
+        if ((do_num == 0) && (! wp_only)) {
+            if ((64 + (64 * zones)) < zl_len)
+                printf("\n>>> Beware: Zone list truncated, may need another "
+                       "call\n");
+        }
     } else if (SG_LIB_CAT_INVALID_OP == res)
         pr2serr("Report zones command not supported\n");
     else {
