@@ -1,14 +1,18 @@
 /*
- * Copyright (c) 2011-2015 Douglas Gilbert.
+ * Copyright (c) 2011-2019 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <unistd.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
+#include <stdbool.h>
 #include <string.h>
 #include <errno.h>
 #include <limits.h>
@@ -21,6 +25,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
 #include "sg_lib.h"
 #include "sg_pt.h"
 #include "sg_cmds_basic.h"
@@ -28,7 +33,7 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
-static const char * version_str = "1.00 20151219";
+static const char * version_str = "1.14 20191220";
 
 /* Not all environments support the Unix sleep() */
 #if defined(MSC_VER) || defined(__MINGW32__)
@@ -67,6 +72,8 @@ static struct option long_options[] = {
     {"count", required_argument, 0, 'c'},
     {"crypto", no_argument, 0, 'C'},
     {"desc", no_argument, 0, 'd'},
+    {"dry-run", no_argument, 0, 'D'},
+    {"dry_run", no_argument, 0, 'D'},
     {"early", no_argument, 0, 'e'},
     {"fail", no_argument, 0, 'F'},
     {"help", no_argument, 0, 'h'},
@@ -76,6 +83,7 @@ static struct option long_options[] = {
     {"pattern", required_argument, 0, 'p'},
     {"quick", no_argument, 0, 'Q'},
     {"test", required_argument, 0, 'T'},
+    {"timeout", required_argument, 0, 't'},
     {"verbose", no_argument, 0, 'v'},
     {"version", no_argument, 0, 'V'},
     {"wait", no_argument, 0, 'w'},
@@ -84,22 +92,26 @@ static struct option long_options[] = {
 };
 
 struct opts_t {
-    int ause;
-    int block;
+    bool ause;
+    bool block;
+    bool crypto;
+    bool desc;
+    bool dry_run;
+    bool early;
+    bool fail;
+    bool invert;
+    bool overwrite;
+    bool quick;
+    bool verbose_given;
+    bool version_given;
+    bool wait;
+    bool znr;
     int count;
-    int crypto;
-    int desc;
-    int early;
-    int fail;
-    int invert;
     int ipl;    /* initialization pattern length */
-    int overwrite;
     int test;
-    int quick;
+    int timeout;        /* in seconds */
     int verbose;
-    int wait;
     int zero;
-    int znr;
     const char * pattern_fn;
 };
 
@@ -108,12 +120,14 @@ static void
 usage()
 {
   pr2serr("Usage: sg_sanitize [--ause] [--block] [--count=OC] [--crypto] "
-          "[--early]\n"
-          "                   [--fail] [--help] [--invert] [--ipl=LEN] "
-          "[--overwrite]\n"
-          "                   [--pattern=PF] [--quick] [--test=TE] "
-          "[--verbose]\n"
-          "                   [--version] [--wait] [--zero] [--znr] DEVICE\n"
+          "[--dry-run]\n"
+          "                   [--early] [--fail] [--help] [--invert] "
+          "[--ipl=LEN]\n"
+          "                   [--overwrite] [--pattern=PF] [--quick] "
+          "[--test=TE]\n"
+          "                   [--timeout=SECS] [--verbose] [--version] "
+          "[--wait]\n"
+          "                   [--zero] [--znr] DEVICE\n"
           "  where:\n"
           "    --ause|-A            set AUSE bit in cdb\n"
           "    --block|-B           do BLOCK ERASE sanitize\n"
@@ -123,6 +137,8 @@ usage()
           "    --desc|-d            polling request sense sets 'desc' "
           "field\n"
           "                         (def: clear 'desc' field)\n"
+          "    --dry-run|-D         to preparation but bypass SANITIZE "
+          "commnd\n"
           "    --early|-e           exit once sanitize started (IMMED set "
           "in cdb)\n"
           "                         user can monitor progress with REQUEST "
@@ -143,6 +159,8 @@ usage()
           "    --test=TE|-T TE      TE is placed in TEST field of "
           "OVERWRITE\n"
           "                         parameter list (def: 0)\n"
+          "    --timeout=SECS|-t SECS    SANITIZE command timeout in "
+          "seconds\n"
           "    --verbose|-v         increase verbosity\n"
           "    --version|-V         print version string then exit\n"
           "    --wait|-w            wait for command to finish (could "
@@ -164,59 +182,71 @@ static int
 do_sanitize(int sg_fd, const struct opts_t * op, const void * param_lstp,
             int param_lst_len)
 {
-    int k, ret, res, sense_cat, immed;
-    unsigned char sanCmdBlk[SANITIZE_OP_LEN];
-    unsigned char sense_b[SENSE_BUFF_LEN];
+    bool immed;
+    int ret, res, sense_cat, timeout;
+    uint8_t san_cdb[SANITIZE_OP_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN];
     struct sg_pt_base * ptvp;
 
     if (op->early || op->wait)
-        immed = op->early ? 1 : 0;
+        immed = op->early;
     else
-        immed = 1;
-    memset(sanCmdBlk, 0, sizeof(sanCmdBlk));
-    sanCmdBlk[0] = SANITIZE_OP;
+        immed = true;
+    timeout = (immed ? SHORT_TIMEOUT : LONG_TIMEOUT);
+    /* only use command line timeout if it exceeds previous defaults */
+    if (op->timeout > timeout)
+        timeout = op->timeout;
+    memset(san_cdb, 0, sizeof(san_cdb));
+    san_cdb[0] = SANITIZE_OP;
     if (op->overwrite)
-        sanCmdBlk[1] = SANITIZE_SA_OVERWRITE;
+        san_cdb[1] = SANITIZE_SA_OVERWRITE;
     else if (op->block)
-        sanCmdBlk[1] = SANITIZE_SA_BLOCK_ERASE;
+        san_cdb[1] = SANITIZE_SA_BLOCK_ERASE;
     else if (op->crypto)
-        sanCmdBlk[1] = SANITIZE_SA_CRYPTO_ERASE;
+        san_cdb[1] = SANITIZE_SA_CRYPTO_ERASE;
     else if (op->fail)
-        sanCmdBlk[1] = SANITIZE_SA_EXIT_FAIL_MODE;
+        san_cdb[1] = SANITIZE_SA_EXIT_FAIL_MODE;
     else
         return SG_LIB_SYNTAX_ERROR;
     if (immed)
-        sanCmdBlk[1] |= 0x80;
+        san_cdb[1] |= 0x80;
     if (op->znr)        /* added sbc4r07 */
-        sanCmdBlk[1] |= 0x40;
+        san_cdb[1] |= 0x40;
     if (op->ause)
-        sanCmdBlk[1] |= 0x20;
-    sg_put_unaligned_be16((uint16_t)param_lst_len, sanCmdBlk + 7);
+        san_cdb[1] |= 0x20;
+    sg_put_unaligned_be16((uint16_t)param_lst_len, san_cdb + 7);
 
     if (op->verbose > 1) {
-        pr2serr("    Sanitize cmd: ");
-        for (k = 0; k < SANITIZE_OP_LEN; ++k)
-            pr2serr("%02x ", sanCmdBlk[k]);
-        pr2serr("\n");
+        char b[128];
+
+        pr2serr("    Sanitize cdb: %s\n",
+                sg_get_command_str(san_cdb, SANITIZE_OP_LEN, false,
+                                   sizeof(b), b));
+        if (op->verbose > 2) {
+            if (param_lst_len > 0) {
+                pr2serr("    Parameter list contents:\n");
+                hex2stderr((const uint8_t *)param_lstp, param_lst_len, -1);
+            }
+            pr2serr("    Sanitize command timeout: %d seconds\n", timeout);
+        }
     }
-    if ((op->verbose > 2) && (param_lst_len > 0)) {
-        pr2serr("    Parameter list contents:\n");
-        dStrHexErr((const char *)param_lstp, param_lst_len, 1);
+    if (op->dry_run) {
+        pr2serr("Due to --dry-run option, bypassing SANITIZE command\n");
+        return 0;
     }
     ptvp = construct_scsi_pt_obj();
     if (NULL == ptvp) {
         pr2serr("Sanitize: out of memory\n");
         return -1;
     }
-    set_scsi_pt_cdb(ptvp, sanCmdBlk, sizeof(sanCmdBlk));
+    set_scsi_pt_cdb(ptvp, san_cdb, sizeof(san_cdb));
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
-    set_scsi_pt_data_out(ptvp, (unsigned char *)param_lstp, param_lst_len);
-    res = do_scsi_pt(ptvp, sg_fd, (immed ? SHORT_TIMEOUT : LONG_TIMEOUT),
-                     op->verbose);
-    ret = sg_cmds_process_resp(ptvp, "Sanitize", res, 0, sense_b,
-                               1 /*noisy */, op->verbose, &sense_cat);
+    set_scsi_pt_data_out(ptvp, (uint8_t *)param_lstp, param_lst_len);
+    res = do_scsi_pt(ptvp, sg_fd, timeout, op->verbose);
+    ret = sg_cmds_process_resp(ptvp, "Sanitize", res, true /*noisy */,
+                               op->verbose, &sense_cat);
     if (-1 == ret)
-        ;
+        ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
     else if (-2 == ret) {
         switch (sense_cat) {
         case SG_LIB_CAT_RECOVERED:
@@ -225,7 +255,8 @@ do_sanitize(int sg_fd, const struct opts_t * op, const void * param_lstp,
             break;
         case SG_LIB_CAT_MEDIUM_HARD:
             {
-                int valid, slen;
+                bool valid;
+                int slen;
                 uint64_t ull = 0;
 
                 slen = get_scsi_pt_sense_len(ptvp);
@@ -253,51 +284,51 @@ do_sanitize(int sg_fd, const struct opts_t * op, const void * param_lstp,
 #define TPROTO_ISCSI 5
 
 static char *
-get_lu_name(const unsigned char * ucp, int u_len, char * b, int b_len)
+get_lu_name(const uint8_t * bp, int u_len, char * b, int b_len)
 {
     int len, off, sns_dlen, dlen, k;
-    unsigned char u_sns[512];
+    uint8_t u_sns[512];
     char * cp;
 
     len = u_len - 4;
-    ucp += 4;
+    bp += 4;
     off = -1;
-    if (0 == sg_vpd_dev_id_iter(ucp, len, &off, VPD_ASSOC_LU,
+    if (0 == sg_vpd_dev_id_iter(bp, len, &off, VPD_ASSOC_LU,
                                 8 /* SCSI name string (sns) */,
                                 3 /* UTF-8 */)) {
-        sns_dlen = ucp[off + 3];
-        memcpy(u_sns, ucp + off + 4, sns_dlen);
+        sns_dlen = bp[off + 3];
+        memcpy(u_sns, bp + off + 4, sns_dlen);
         /* now want to check if this is iSCSI */
         off = -1;
-        if (0 == sg_vpd_dev_id_iter(ucp, len, &off, VPD_ASSOC_TPORT,
+        if (0 == sg_vpd_dev_id_iter(bp, len, &off, VPD_ASSOC_TPORT,
                                     8 /* SCSI name string (sns) */,
                                     3 /* UTF-8 */)) {
-            if ((0x80 & ucp[1]) && (TPROTO_ISCSI == (ucp[0] >> 4))) {
+            if ((0x80 & bp[1]) && (TPROTO_ISCSI == (bp[0] >> 4))) {
                 snprintf(b, b_len, "%.*s", sns_dlen, u_sns);
                 return b;
             }
         }
     } else
         sns_dlen = 0;
-    if (0 == sg_vpd_dev_id_iter(ucp, len, &off, VPD_ASSOC_LU,
+    if (0 == sg_vpd_dev_id_iter(bp, len, &off, VPD_ASSOC_LU,
                                 3 /* NAA */, 1 /* binary */)) {
-        dlen = ucp[off + 3];
+        dlen = bp[off + 3];
         if (! ((8 == dlen) || (16 ==dlen)))
             return b;
         cp = b;
         for (k = 0; ((k < dlen) && (b_len > 1)); ++k) {
-            snprintf(cp, b_len, "%02x", ucp[off + 4 + k]);
+            snprintf(cp, b_len, "%02x", bp[off + 4 + k]);
             cp += 2;
             b_len -= 2;
         }
-    } else if (0 == sg_vpd_dev_id_iter(ucp, len, &off, VPD_ASSOC_LU,
+    } else if (0 == sg_vpd_dev_id_iter(bp, len, &off, VPD_ASSOC_LU,
                                        2 /* EUI */, 1 /* binary */)) {
-        dlen = ucp[off + 3];
+        dlen = bp[off + 3];
         if (! ((8 == dlen) || (12 == dlen) || (16 ==dlen)))
             return b;
         cp = b;
         for (k = 0; ((k < dlen) && (b_len > 1)); ++k) {
-            snprintf(cp, b_len, "%02x", ucp[off + 4 + k]);
+            snprintf(cp, b_len, "%02x", bp[off + 4 + k]);
             cp += 2;
             b_len -= 2;
         }
@@ -312,16 +343,16 @@ get_lu_name(const unsigned char * ucp, int u_len, char * b, int b_len)
 #define VPD_DEVICE_ID 0x83
 
 static int
-print_dev_id(int fd, unsigned char * sinq_resp, int max_rlen, int verbose)
+print_dev_id(int fd, uint8_t * sinq_resp, int max_rlen, int verbose)
 {
     int res, k, n, verb, pdt, has_sn, has_di;
-    unsigned char b[256];
+    uint8_t b[256];
     char a[256];
     char pdt_name[64];
 
     verb = (verbose > 1) ? verbose - 1 : 0;
     memset(sinq_resp, 0, max_rlen);
-    res = sg_ll_inquiry(fd, 0, 0 /* evpd */, 0 /* pg_op */, b,
+    res = sg_ll_inquiry(fd, false, false /* evpd */, 0 /* pg_op */, b,
                         SAFE_STD_INQ_RESP_LEN, 1, verb);
     if (res)
         return res;
@@ -343,7 +374,7 @@ print_dev_id(int fd, unsigned char * sinq_resp, int max_rlen, int verbose)
         pr2serr("Short INQUIRY response: %d bytes, expect at least 36\n", n);
         return SG_LIB_CAT_OTHER;
     }
-    res = sg_ll_inquiry(fd, 0, 1 /* evpd */, VPD_SUPPORTED_VPDS, b,
+    res = sg_ll_inquiry(fd, false, true /* evpd */, VPD_SUPPORTED_VPDS, b,
                         SAFE_STD_INQ_RESP_LEN, 1, verb);
     if (res) {
         if (verbose)
@@ -359,21 +390,16 @@ print_dev_id(int fd, unsigned char * sinq_resp, int max_rlen, int verbose)
     if (n > (SAFE_STD_INQ_RESP_LEN - 4))
         n = (SAFE_STD_INQ_RESP_LEN - 4);
     for (k = 0, has_sn = 0, has_di = 0; k < n; ++k) {
-        if (VPD_UNIT_SERIAL_NUM == b[4 + k]) {
-            if (has_di) {
-                if (verbose)
-                    pr2serr("VPD_SUPPORTED_VPDS dis-ordered\n");
-                return 0;
-            }
+        if (VPD_UNIT_SERIAL_NUM == b[4 + k])
             ++has_sn;
-        } else if (VPD_DEVICE_ID == b[4 + k]) {
+        else if (VPD_DEVICE_ID == b[4 + k]) {
             ++has_di;
             break;
         }
     }
     if (has_sn) {
-        res = sg_ll_inquiry(fd, 0, 1 /* evpd */, VPD_UNIT_SERIAL_NUM, b,
-                            sizeof(b), 1, verb);
+        res = sg_ll_inquiry(fd, false, true /* evpd */, VPD_UNIT_SERIAL_NUM,
+                            b, sizeof(b), 1, verb);
         if (res) {
             if (verbose)
                 pr2serr("VPD_UNIT_SERIAL_NUM gave res=%d\n", res);
@@ -390,7 +416,7 @@ print_dev_id(int fd, unsigned char * sinq_resp, int max_rlen, int verbose)
         printf("      Unit serial number: %.*s\n", n, (const char *)(b + 4));
     }
     if (has_di) {
-        res = sg_ll_inquiry(fd, 0, 1 /* evpd */, VPD_DEVICE_ID, b,
+        res = sg_ll_inquiry(fd, false, true /* evpd */, VPD_DEVICE_ID, b,
                             sizeof(b), 1, verb);
         if (res) {
             if (verbose)
@@ -416,19 +442,21 @@ print_dev_id(int fd, unsigned char * sinq_resp, int max_rlen, int verbose)
 int
 main(int argc, char * argv[])
 {
-    int sg_fd, k, res, c, infd, progress, vb, n, resp_len;
-    int got_stdin = 0;
+    bool got_stdin = false;
+    int k, res, c, infd, progress, vb, n, resp_len, err;
+    int sg_fd = -1;
     int param_lst_len = 0;
+    int ret = -1;
     const char * device_name = NULL;
     char ebuff[EBUFF_SZ];
     char b[80];
-    unsigned char requestSenseBuff[DEF_REQS_RESP_LEN];
-    unsigned char * wBuff = NULL;
-    int ret = -1;
+    uint8_t rsBuff[DEF_REQS_RESP_LEN];
+    uint8_t * wBuff = NULL;
+    uint8_t * free_wBuff = NULL;
     struct opts_t opts;
     struct opts_t * op;
     struct stat a_stat;
-    unsigned char inq_resp[SAFE_STD_INQ_RESP_LEN];
+    uint8_t inq_resp[SAFE_STD_INQ_RESP_LEN];
 
     op = &opts;
     memset(op, 0, sizeof(opts));
@@ -436,17 +464,17 @@ main(int argc, char * argv[])
     while (1) {
         int option_index = 0;
 
-        c = getopt_long(argc, argv, "ABc:CdeFhi:IOp:QT:vVwzZ", long_options,
-                        &option_index);
+        c = getopt_long(argc, argv, "ABc:CdDeFhi:IOp:Qt:T:vVwzZ",
+                        long_options, &option_index);
         if (c == -1)
             break;
 
         switch (c) {
         case 'A':
-            ++op->ause;
+            op->ause = true;
             break;
         case 'B':
-            ++op->block;
+            op->block = true;
             break;
         case 'c':
             op->count = sg_get_num(optarg);
@@ -456,16 +484,19 @@ main(int argc, char * argv[])
             }
             break;
         case 'C':
-            ++op->crypto;
+            op->crypto = true;
             break;
         case 'd':
-            ++op->desc;
+            op->desc = true;
+            break;
+        case 'D':
+            op->dry_run = true;
             break;
         case 'e':
-            ++op->early;
+            op->early = true;
             break;
         case 'F':
-            ++op->fail;
+            op->fail = true;
             break;
         case 'h':
         case '?':
@@ -479,16 +510,23 @@ main(int argc, char * argv[])
             }
             break;
         case 'I':
-            ++op->invert;
+            op->invert = true;
             break;
         case 'O':
-            ++op->overwrite;
+            op->overwrite = true;
             break;
         case 'p':
             op->pattern_fn = optarg;
             break;
         case 'Q':
-            ++op->quick;
+            op->quick = true;
+            break;
+        case 't':
+            op->timeout = sg_get_num(optarg);
+            if (op->timeout < 0) {
+                pr2serr("bad argument to '--timeout=SECS', want 0 or more\n");
+                return SG_LIB_SYNTAX_ERROR;
+            }
             break;
         case 'T':
             op->test = sg_get_num(optarg);
@@ -498,19 +536,20 @@ main(int argc, char * argv[])
             }
             break;
         case 'v':
+            op->verbose_given = true;
             ++op->verbose;
             break;
         case 'V':
-            pr2serr(ME "version: %s\n", version_str);
-            return 0;
+            op->version_given = true;
+            break;
         case 'w':
-            ++op->wait;
+            op->wait = true;
             break;
         case 'z':
             ++op->zero;
             break;
         case 'Z':
-            ++op->znr;
+            op->znr = true;
             break;
         default:
             pr2serr("unrecognised option code 0x%x ??\n", c);
@@ -530,39 +569,62 @@ main(int argc, char * argv[])
             return SG_LIB_SYNTAX_ERROR;
         }
     }
+#ifdef DEBUG
+    pr2serr("In DEBUG mode, ");
+    if (op->verbose_given && op->version_given) {
+        pr2serr("but override: '-vV' given, zero verbose and continue\n");
+        op->verbose_given = false;
+        op->version_given = false;
+        op->verbose = 0;
+    } else if (! op->verbose_given) {
+        pr2serr("set '-vv'\n");
+        op->verbose = 2;
+    } else
+        pr2serr("keep verbose=%d\n", op->verbose);
+#else
+    if (op->verbose_given && op->version_given)
+        pr2serr("Not in DEBUG mode, so '-vV' has no special action\n");
+#endif
+    if (op->version_given) {
+        pr2serr(ME "version: %s\n", version_str);
+        return 0;
+    }
+
     if (NULL == device_name) {
-        pr2serr("missing device name!\n");
+        pr2serr("Missing device name!\n\n");
         usage();
         return SG_LIB_SYNTAX_ERROR;
     }
     vb = op->verbose;
-    n = !!op->block + !!op->crypto + !!op->fail + !!op->overwrite;
+    n = (int)op->block + (int)op->crypto + (int)op->fail + (int)op->overwrite;
     if (1 != n) {
         pr2serr("one and only one of '--block', '--crypto', '--fail' or "
                 "'--overwrite' please\n");
-        return SG_LIB_SYNTAX_ERROR;
+        return SG_LIB_CONTRADICT;
     }
     if (op->overwrite) {
         if (op->zero) {
             if (op->pattern_fn) {
                 pr2serr("confused: both '--pattern=PF' and '--zero' "
                         "options\n");
-                return SG_LIB_SYNTAX_ERROR;
+                return SG_LIB_CONTRADICT;
             }
             op->ipl = 4;
         } else {
             if (NULL == op->pattern_fn) {
                 pr2serr("'--overwrite' requires '--pattern=PF' or '--zero' "
                         "option\n");
-                return SG_LIB_SYNTAX_ERROR;
+                return SG_LIB_CONTRADICT;
             }
-            got_stdin = (0 == strcmp(op->pattern_fn, "-")) ? 1 : 0;
+            got_stdin = (0 == strcmp(op->pattern_fn, "-"));
             if (! got_stdin) {
                 memset(&a_stat, 0, sizeof(a_stat));
                 if (stat(op->pattern_fn, &a_stat) < 0) {
+                    err = errno;
                     pr2serr("pattern file: unable to stat(%s): %s\n",
-                            op->pattern_fn, safe_strerror(errno));
-                    return SG_LIB_FILE_ERROR;
+                            op->pattern_fn, safe_strerror(err));
+                    ret = sg_convert_errno(err);
+                    goto err_out;
                 }
                 if (op->ipl <= 0) {
                     op->ipl = (int)a_stat.st_size;
@@ -576,16 +638,18 @@ main(int argc, char * argv[])
             if (op->ipl < 1) {
                 pr2serr("'--overwrite' requires '--ipl=LEN' option if can't "
                         "get PF length\n");
-                return SG_LIB_SYNTAX_ERROR;
+                return SG_LIB_CONTRADICT;
             }
         }
     }
 
-    sg_fd = sg_cmds_open_device(device_name, 0 /* rw */, vb);
+    sg_fd = sg_cmds_open_device(device_name, false /* rw */, vb);
     if (sg_fd < 0) {
-        pr2serr(ME "open error: %s: %s\n", device_name,
-                safe_strerror(-sg_fd));
-        return SG_LIB_FILE_ERROR;
+        if (op->verbose)
+            pr2serr(ME "open error: %s: %s\n", device_name,
+                    safe_strerror(-sg_fd));
+        ret = sg_convert_errno(-sg_fd);
+        goto err_out;
     }
 
     ret = print_dev_id(sg_fd, inq_resp, sizeof(inq_resp), op->verbose);
@@ -594,11 +658,11 @@ main(int argc, char * argv[])
 
     if (op->overwrite) {
         param_lst_len = op->ipl + 4;
-        wBuff = (unsigned char*)calloc(op->ipl + 4, 1);
+        wBuff = (uint8_t*)sg_memalign(op->ipl + 4, 0, &free_wBuff, false);
         if (NULL == wBuff) {
             pr2serr("unable to allocate %d bytes of memory with calloc()\n",
                     op->ipl + 4);
-            ret = SG_LIB_SYNTAX_ERROR;
+            ret = sg_convert_errno(ENOMEM);
             goto err_out;
         }
         if (op->zero) {
@@ -613,22 +677,24 @@ main(int argc, char * argv[])
                     perror("sg_set_binary_mode");
             } else {
                 if ((infd = open(op->pattern_fn, O_RDONLY)) < 0) {
+                    err = errno;
                     snprintf(ebuff, EBUFF_SZ, ME "could not open %s for "
                              "reading", op->pattern_fn);
                     perror(ebuff);
-                    ret = SG_LIB_FILE_ERROR;
+                    ret = sg_convert_errno(err);
                     goto err_out;
                 } else if (sg_set_binary_mode(infd) < 0)
                     perror("sg_set_binary_mode");
             }
             res = read(infd, wBuff + 4, op->ipl);
             if (res < 0) {
+                err = errno;
                 snprintf(ebuff, EBUFF_SZ, ME "couldn't read from %s",
                          op->pattern_fn);
                 perror(ebuff);
                 if (! got_stdin)
                     close(infd);
-                ret = SG_LIB_FILE_ERROR;
+                ret = sg_convert_errno(err);
                 goto err_out;
             }
             if (res < op->ipl) {
@@ -639,7 +705,7 @@ main(int argc, char * argv[])
             if (! got_stdin)
                 close(infd);
         }
-        wBuff[0] = op->count & 0x1f;;
+        wBuff[0] = op->count & 0x1f;
         if (op->test)
             wBuff[0] |= ((op->test & 0x3) << 5);
         if (op->invert)
@@ -647,7 +713,7 @@ main(int argc, char * argv[])
         sg_put_unaligned_be16((uint16_t)op->ipl, wBuff + 2);
     }
 
-    if ((0 == op->quick) && (! op->fail)) {
+    if ((! op->quick) && (! op->fail)) {
         printf("\nA SANITIZE will commence in 15 seconds\n");
         printf("    ALL data on %s will be DESTROYED\n", device_name);
         printf("        Press control-C to abort\n");
@@ -668,22 +734,26 @@ main(int argc, char * argv[])
         pr2serr("Sanitize failed: %s\n", b);
     }
 
-    if ((0 == ret) && (0 == op->early) && (0 == op->wait)) {
-        for (k = 0 ;; ++k) {
+    if ((0 == ret) && (! op->early) && (! op->wait)) {
+        for (k = 0; ;++k) {     /* unbounded, exits via break */
+            if (op->dry_run && (k > 0)) {
+                pr2serr("Due to --dry-run option, leave poll loop\n");
+                break;
+            }
             sleep_for(POLL_DURATION_SECS);
-            memset(requestSenseBuff, 0x0, sizeof(requestSenseBuff));
-            res = sg_ll_request_sense(sg_fd, op->desc, requestSenseBuff,
-                                      sizeof(requestSenseBuff), 1, vb);
+            memset(rsBuff, 0x0, sizeof(rsBuff));
+            res = sg_ll_request_sense(sg_fd, op->desc, rsBuff, sizeof(rsBuff),
+                                      1, vb);
             if (res) {
                 ret = res;
                 if (SG_LIB_CAT_INVALID_OP == res)
                     pr2serr("Request Sense command not supported\n");
                 else if (SG_LIB_CAT_ILLEGAL_REQ == res) {
                     pr2serr("bad field in Request Sense cdb\n");
-                    if (1 == op->desc) {
+                    if (op->desc) {
                         pr2serr("Descriptor type sense may not be supported, "
                                 "try again with fixed type\n");
-                        op->desc = 0;
+                        op->desc = false;
                         continue;
                     }
                 } else {
@@ -696,14 +766,13 @@ main(int argc, char * argv[])
                 break;
             }
             /* "Additional sense length" same in descriptor and fixed */
-            resp_len = requestSenseBuff[7] + 8;
+            resp_len = rsBuff[7] + 8;
             if (vb > 2) {
                 pr2serr("Parameter data in hex\n");
-                dStrHexErr((const char *)requestSenseBuff, resp_len, 1);
+                hex2stderr(rsBuff, resp_len, -1);
             }
             progress = -1;
-            sg_get_sense_progress_fld(requestSenseBuff, resp_len,
-                                      &progress);
+            sg_get_sense_progress_fld(rsBuff, resp_len, &progress);
             if (progress < 0) {
                 ret = res;
                 if (vb > 1)
@@ -718,13 +787,20 @@ main(int argc, char * argv[])
     }
 
 err_out:
-    if (wBuff)
-        free(wBuff);
-    res = sg_cmds_close_device(sg_fd);
-    if (res < 0) {
-        pr2serr("close error: %s\n", safe_strerror(-res));
-        if (0 == ret)
-            return SG_LIB_FILE_ERROR;
+    if (free_wBuff)
+        free(free_wBuff);
+    if (sg_fd >= 0) {
+        res = sg_cmds_close_device(sg_fd);
+        if (res < 0) {
+            pr2serr("close error: %s\n", safe_strerror(-res));
+            if (0 == ret)
+                ret = sg_convert_errno(-res);
+        }
+    }
+    if (0 == op->verbose) {
+        if (! sg_if_can2stderr("sg_sanitize failed: ", ret))
+            pr2serr("Some error occurred, try again with '-v' "
+                    "or '-vv' for more information\n");
     }
     return (ret >= 0) ? ret : SG_LIB_CAT_OTHER;
 }
