@@ -1,7 +1,7 @@
 /*
  * (c) 2000 Kurt Garloff <garloff at suse dot de>
  * heavily based on Douglas Gilbert's sg_rbuf program.
- * (c) 1999-2018 Douglas Gilbert
+ * (c) 1999-2019 Douglas Gilbert
  *
  * Program to test the SCSI host adapter by issuing
  * write and read operations on a device's buffer
@@ -16,6 +16,8 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
  * $Id: sg_test_rwbuf.c,v 1.1 2000/03/02 13:50:03 garloff Exp $
  *
  *   2003/11/11  switch sg3_utils version to use SG_IO ioctl [dpg]
@@ -45,7 +47,7 @@
 #include "sg_pr2serr.h"
 
 
-static const char * version_str = "1.18 20180628";
+static const char * version_str = "1.20 20191220";
 
 #define BPI (signed)(sizeof(int))
 
@@ -94,7 +96,7 @@ find_out_about_buffer(int sg_fd)
         uint8_t rbBuff[RB_DESC_LEN];
         uint8_t sense_buffer[32];
         struct sg_io_hdr io_hdr;
-        int k, res;
+        int res;
 
         rb_cdb[1] = RB_MODE_DESC;
         rb_cdb[8] = RB_DESC_LEN;
@@ -110,10 +112,11 @@ find_out_about_buffer(int sg_fd)
         io_hdr.timeout = 60000;     /* 60000 millisecs == 60 seconds */
 
         if (verbose) {
-                pr2serr("    read buffer [mode desc] cdb: ");
-                for (k = 0; k < (int)sizeof(rb_cdb); ++k)
-                        pr2serr("%02x ", rb_cdb[k]);
-                pr2serr("\n");
+                char b[128];
+
+                pr2serr("    read buffer [mode desc] cdb: %s\n",
+                        sg_get_command_str(rb_cdb, (int)sizeof(rb_cdb), false,
+                                           sizeof(b), b));
         }
         if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
                 perror(ME "SG_IO READ BUFFER descriptor error");
@@ -228,7 +231,7 @@ void do_fill_buffer (int *buf, int len)
 
 int read_buffer (int sg_fd, unsigned ssize)
 {
-        int res, k;
+        int res;
         uint8_t rb_cdb[] = {READ_BUFFER, 0, 0, 0, 0, 0, 0, 0, 0, 0};
         int bufSize = ssize + addread;
         uint8_t * free_rbBuff = NULL;
@@ -253,10 +256,11 @@ int read_buffer (int sg_fd, unsigned ssize)
         io_hdr.pack_id = 2;
         io_hdr.timeout = 60000;     /* 60000 millisecs == 60 seconds */
         if (verbose) {
-                pr2serr("    read buffer [mode data] cdb: ");
-                for (k = 0; k < (int)sizeof(rb_cdb); ++k)
-                        pr2serr("%02x ", rb_cdb[k]);
-                pr2serr("\n");
+                char b[128];
+
+                pr2serr("    read buffer [mode data] cdb: %s\n",
+                        sg_get_command_str(rb_cdb, (int)sizeof(rb_cdb), false,
+                                           sizeof(b), b));
         }
 
         if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
@@ -298,7 +302,7 @@ int write_buffer (int sg_fd, unsigned ssize)
                                                   false);
         uint8_t sense_buffer[32];
         struct sg_io_hdr io_hdr;
-        int k, res;
+        int res;
 
         if (NULL == wbBuff)
                 return -1;
@@ -318,10 +322,11 @@ int write_buffer (int sg_fd, unsigned ssize)
         io_hdr.pack_id = 1;
         io_hdr.timeout = 60000;     /* 60000 millisecs == 60 seconds */
         if (verbose) {
-                pr2serr("    write buffer [mode data] cdb: ");
-                for (k = 0; k < (int)sizeof(wb_cdb); ++k)
-                        pr2serr("%02x ", wb_cdb[k]);
-                pr2serr("\n");
+                char b[128];
+
+                pr2serr("    write buffer [mode data] cdb: %s\n",
+                        sg_get_command_str(wb_cdb, (int)sizeof(wb_cdb), false,
+                                           sizeof(b), b));
         }
 
         if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
@@ -386,8 +391,8 @@ void usage ()
 
 int main (int argc, char * argv[])
 {
-        bool verbose_given = true;
-        bool version_given = true;
+        bool verbose_given = false;
+        bool version_given = false;
         int sg_fd, res;
         const char * device_name = NULL;
         int times = 1;
