@@ -1,8 +1,10 @@
 /*
- * Copyright (c) 2006-2016 Douglas Gilbert.
+ * Copyright (c) 2006-2019 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <unistd.h>
@@ -51,9 +53,13 @@
 #define VPD_VP_HP3PAR 4
 #define VPD_VP_IBM_LTO 5
 #define VPD_VP_HP_LTO 6
+#define VPD_VP_WDC_HITACHI 7
+#define VPD_VP_NVME 8
+#define VPD_VP_SG 9     /* this package/library as a vendor */
 
 
 /* vendor VPD pages */
+#define VPD_V_HIT_PG3 0x3
 #define VPD_V_HP3PAR 0xc0
 #define VPD_V_FIRM_SEA  0xc0
 #define VPD_V_UPR_EMC  0xc0
@@ -80,6 +86,12 @@
 #define VPD_V_VAC_RDAC 0xc9
 #define VPD_V_RVSI_RDAC 0xca
 #define VPD_V_SAID_RDAC 0xd0
+#define VPD_V_HIT_PG_D1 0xd1
+#define VPD_V_HIT_PG_D2 0xd2
+
+#ifndef SG_NVME_VPD_NICR
+#define SG_NVME_VPD_NICR 0xde   /* NVME Identify Controller Response */
+#endif
 
 
 #define DEF_ALLOC_LEN 252
@@ -87,15 +99,17 @@
 
 /* These structures are duplicates of those of the same name in
  * sg_vpd.c . Take care that both are the same. */
+
 struct opts_t {
-    int do_all;
-    int do_enum;
+    bool do_all;
+    bool do_enum;
+    bool do_force;
+    bool do_long;
+    bool do_quiet;
     int do_hex;
-    int num_vpd;
+    int vpd_pn;
     int do_ident;
-    int do_long;
     int maxlen;
-    int do_quiet;
     int do_raw;
     int vend_prod_num;
     int verbose;
@@ -114,11 +128,11 @@ struct svpd_values_name_t {
     const char * name;
 };
 
-int vpd_fetch_page_from_dev(int sg_fd, unsigned char * rp, int page,
-                            int mxlen, int vb, int * rlenp);
+int vpd_fetch_page(int sg_fd, uint8_t * rp, int page, int mxlen,
+                   bool qt, int vb, int * rlenp);
 
 /* sharing large global buffer, defined in sg_vpd.c */
-extern unsigned char rsp_buff[];
+extern uint8_t * rsp_buff;
 
 /* end of section copied from sg_vpd.c . Maybe sg_vpd.h is needed */
 
@@ -134,11 +148,15 @@ struct svpd_vp_name_t {
 static struct svpd_vp_name_t vp_arr[] = {
     {VPD_VP_DDS, "dds", "DDS tape family from IBM"},
     {VPD_VP_EMC, "emc", "EMC (company)"},
+    {VPD_VP_WDC_HITACHI, "hit", "WDC/Hitachi disk"},
     {VPD_VP_HP3PAR, "hp3par", "3PAR array (HP was Left Hand)"},
-    {VPD_VP_IBM_LTO, "ibm_lto", "IBM LTO tape/systems"},
     {VPD_VP_HP_LTO, "hp_lto", "HP LTO tape/systems"},
+    {VPD_VP_IBM_LTO, "ibm_lto", "IBM LTO tape/systems"},
+    {VPD_VP_NVME, "nvme", "NVMe related"},
     {VPD_VP_RDAC, "rdac", "RDAC array (NetApp E-Series)"},
     {VPD_VP_SEAGATE, "sea", "Seagate disk"},
+    {VPD_VP_SG, "sg", "sg3_utils extensions"},
+    {VPD_VP_WDC_HITACHI, "wdc", "WDC/Hitachi disk"},
     {0, NULL, NULL},
 };
 
@@ -148,21 +166,20 @@ static struct svpd_vp_name_t vp_arr[] = {
 static struct svpd_values_name_t vendor_vpd_pg[] = {
     {VPD_V_ACI_LTO, VPD_VP_HP_LTO, 1, "aci", "ACI revision level (HP LTO)"},
     {VPD_V_DATC_SEA, VPD_VP_SEAGATE, 0, "datc", "Date code (Seagate)"},
-    {VPD_V_DCRL_LTO, VPD_VP_IBM_LTO, 1, "dcrl" , "Drive component revision "
+    {VPD_V_DCRL_LTO, VPD_VP_IBM_LTO, 1, "dcrl", "Drive component revision "
      "levels (IBM LTO)"},
     {VPD_V_FVER_DDS, VPD_VP_DDS, 1, "ddsver", "Firmware revision (DDS)"},
     {VPD_V_DEV_BEH_SEA, VPD_VP_SEAGATE, 0, "devb", "Device behavior "
      "(Seagate)"},
-    {VPD_V_DSN_LTO, VPD_VP_IBM_LTO, 1, "dsn" , "Drive serial numbers (IBM "
+    {VPD_V_DSN_LTO, VPD_VP_IBM_LTO, 1, "dsn", "Drive serial numbers (IBM "
      "LTO)"},
-    {VPD_V_DUCD_LTO, VPD_VP_IBM_LTO, 1, "ducd" , "Device unique "
+    {VPD_V_DUCD_LTO, VPD_VP_IBM_LTO, 1, "ducd", "Device unique "
      "configuration data (IBM LTO)"},
     {VPD_V_EDID_RDAC, VPD_VP_RDAC, 0, "edid", "Extended device "
      "identification (RDAC)"},
-    {VPD_V_FEAT_RDAC, VPD_VP_RDAC, 0, "prm4", "Feature Parameters (RDAC)"},
     {VPD_V_FIRM_SEA, VPD_VP_SEAGATE, 0, "firm", "Firmware numbers "
      "(Seagate)"},
-    {VPD_V_FVER_LTO, VPD_VP_HP_LTO, 0, "frl" , "Firmware revision level "
+    {VPD_V_FVER_LTO, VPD_VP_HP_LTO, 0, "frl", "Firmware revision level "
      "(HP LTO)"},
     {VPD_V_FVER_RDAC, VPD_VP_RDAC, 0, "fwr4", "Firmware version (RDAC)"},
     {VPD_V_HEAD_LTO, VPD_VP_HP_LTO, 1, "head", "Head Assy revision level "
@@ -175,9 +192,12 @@ static struct svpd_values_name_t vendor_vpd_pg[] = {
     {VPD_V_JUMP_SEA, VPD_VP_SEAGATE, 0, "jump", "Jump setting (Seagate)"},
     {VPD_V_MECH_LTO, VPD_VP_HP_LTO, 1, "mech", "Mechanism revision level "
      "(HP LTO)"},
-    {VPD_V_MPDS_LTO, VPD_VP_IBM_LTO, 1, "mpds" , "Mode parameter default "
+    {VPD_V_MPDS_LTO, VPD_VP_IBM_LTO, 1, "mpds", "Mode parameter default "
      "settings (IBM LTO)"},
+    {SG_NVME_VPD_NICR, VPD_VP_SG, 0, "nicr",
+     "NVMe Identify Controller Response (sg3_utils)"},
     {VPD_V_PCA_LTO, VPD_VP_HP_LTO, 1, "pca", "PCA revision level (HP LTO)"},
+    {VPD_V_FEAT_RDAC, VPD_VP_RDAC, 0, "prm4", "Feature Parameters (RDAC)"},
     {VPD_V_RVSI_RDAC, VPD_VP_RDAC, 0, "rvsi", "Replicated volume source "
      "identifier (RDAC)"},
     {VPD_V_SAID_RDAC, VPD_VP_RDAC, 0, "said", "Storage array world wide "
@@ -186,10 +206,30 @@ static struct svpd_values_name_t vendor_vpd_pg[] = {
     {VPD_V_SVER_RDAC, VPD_VP_RDAC, 0, "swr4", "Software version (RDAC)"},
     {VPD_V_UPR_EMC, VPD_VP_EMC, 0, "upr", "Unit path report (EMC)"},
     {VPD_V_VAC_RDAC, VPD_VP_RDAC, 0, "vac1", "Volume access control (RDAC)"},
+    {VPD_V_HIT_PG3, VPD_VP_WDC_HITACHI, 0, "wp3", "Page 0x3 (WDC/Hitachi)"},
+    {VPD_V_HIT_PG_D1, VPD_VP_WDC_HITACHI, 0, "wpd1",
+     "Page 0xd1 (WDC/Hitachi)"},
+    {VPD_V_HIT_PG_D2, VPD_VP_WDC_HITACHI, 0, "wpd2",
+     "Page 0xd2 (WDC/Hitachi)"},
     {0, 0, 0, NULL, NULL},
 };
 
 
+void
+dup_sanity_chk(int sz_opts_t, int sz_values_name_t)
+{
+    const size_t my_sz_opts_t = sizeof(struct opts_t);
+    const size_t my_sz_values_name_t = sizeof(struct svpd_values_name_t);
+
+    if (sz_opts_t != (int)my_sz_opts_t)
+        pr2serr(">>> struct opts_t differs in size from sg_vpd.c [%d != "
+                "%d]\n", (int)my_sz_opts_t, sz_opts_t);
+    if (sz_values_name_t != (int)my_sz_values_name_t)
+        pr2serr(">>> struct svpd_values_name_t differs in size from "
+                "sg_vpd.c [%d != %d]\n", (int)my_sz_values_name_t,
+                sz_values_name_t);
+}
+
 static int
 is_like_pdt(int actual_pdt, const struct svpd_values_name_t * vnp)
 {
@@ -288,16 +328,16 @@ svpd_find_vendor_by_acron(const char * ap)
 void
 svpd_enumerate_vendor(int vend_prod_num)
 {
+    bool seen;
     const struct svpd_vp_name_t * vpp;
     const struct svpd_values_name_t * vnp;
-    int seen;
 
     if (vend_prod_num < 0) {
-        for (seen = 0, vpp = vp_arr; vpp->acron; ++vpp) {
+        for (seen = false, vpp = vp_arr; vpp->acron; ++vpp) {
             if (vpp->name) {
                 if (! seen) {
                     printf("\nVendor/product identifiers:\n");
-                    seen = 1;
+                    seen = true;
                 }
                 printf("  %-10s %d      %s\n", vpp->acron,
                        vpp->vend_prod_num, vpp->name);
@@ -306,13 +346,13 @@ svpd_enumerate_vendor(int vend_prod_num)
     }
     if (-1 == vend_prod_num)
         return;
-    for (seen = 0, vnp = vendor_vpd_pg; vnp->acron; ++vnp) {
+    for (seen = false, vnp = vendor_vpd_pg; vnp->acron; ++vnp) {
         if ((vend_prod_num >= 0) && (vend_prod_num != vnp->subvalue))
             continue;
         if (vnp->name) {
             if (! seen) {
                 printf("\nVendor specific VPD pages:\n");
-                seen = 1;
+                seen = true;
             }
             printf("  %-10s 0x%02x,%d      %s\n", vnp->acron,
                    vnp->value, vnp->subvalue, vnp->name);
@@ -321,13 +361,13 @@ svpd_enumerate_vendor(int vend_prod_num)
 }
 
 int
-svpd_count_vendor_vpds(int num_vpd, int vend_prod_num)
+svpd_count_vendor_vpds(int vpd_pn, int vend_prod_num)
 {
     const struct svpd_values_name_t * vnp;
     int matches;
 
     for (vnp = vendor_vpd_pg, matches = 0; vnp->acron; ++vnp) {
-        if ((num_vpd == vnp->value) && vnp->name) {
+        if ((vpd_pn == vnp->value) && vnp->name) {
             if ((vend_prod_num < 0) || (vend_prod_num == vnp->subvalue)) {
                 if (0 == matches)
                     printf("Matching vendor specific VPD pages:\n");
@@ -341,16 +381,16 @@ svpd_count_vendor_vpds(int num_vpd, int vend_prod_num)
 }
 
 static void
-dStrRaw(const char* str, int len)
+dStrRaw(const uint8_t * str, int len)
 {
     int k;
 
-    for (k = 0 ; k < len; ++k)
+    for (k = 0; k < len; ++k)
         printf("%c", str[k]);
 }
 
 static void
-decode_vpd_c0_hp3par(unsigned char * buff, int len)
+decode_vpd_c0_hp3par(uint8_t * buff, int len)
 {
     int rev;
     long offset;
@@ -410,7 +450,7 @@ decode_vpd_c0_hp3par(unsigned char * buff, int len)
 
 
 static void
-decode_firm_vpd_c0_sea(unsigned char * buff, int len)
+decode_firm_vpd_c0_sea(uint8_t * buff, int len)
 {
     if (len < 28) {
         pr2serr("Seagate firmware numbers VPD page length too short=%d\n",
@@ -444,7 +484,7 @@ decode_firm_vpd_c0_sea(unsigned char * buff, int len)
 }
 
 static void
-decode_date_code_vpd_c1_sea(unsigned char * buff, int len)
+decode_date_code_vpd_c1_sea(uint8_t * buff, int len)
 {
     if (len < 20) {
         pr2serr("Seagate Data code VPD page length too short=%d\n",
@@ -456,7 +496,7 @@ decode_date_code_vpd_c1_sea(unsigned char * buff, int len)
 }
 
 static void
-decode_dev_beh_vpd_c3_sea(unsigned char * buff, int len)
+decode_dev_beh_vpd_c3_sea(uint8_t * buff, int len)
 {
     if (len < 25) {
         pr2serr("Seagate Device behaviour VPD page length too short=%d\n",
@@ -519,7 +559,7 @@ static const char * failover_mode_arr[] =
 };
 
 static void
-decode_upr_vpd_c0_emc(unsigned char * buff, int len)
+decode_upr_vpd_c0_emc(uint8_t * buff, int len)
 {
     int k, ip_mgmt, vpp80, lun_z;
 
@@ -537,7 +577,7 @@ decode_upr_vpd_c0_emc(unsigned char * buff, int len)
         printf("%02x", buff[10 + k]);
     printf("\n");
     printf("  Array Serial Number: ");
-    dStrRaw((const char *)&buff[50], buff[49]);
+    dStrRaw(&buff[50], buff[49]);
     printf("\n");
 
     printf("  LUN State: ");
@@ -593,7 +633,7 @@ decode_upr_vpd_c0_emc(unsigned char * buff, int len)
 }
 
 static void
-decode_rdac_vpd_c0(unsigned char * buff, int len)
+decode_rdac_vpd_c0(uint8_t * buff, int len)
 {
     int memsize;
     char name[65];
@@ -639,7 +679,7 @@ decode_rdac_vpd_c0(unsigned char * buff, int len)
 }
 
 static void
-decode_rdac_vpd_c1(unsigned char * buff, int len)
+decode_rdac_vpd_c1(uint8_t * buff, int len)
 {
     int i, n, v, r, m, p, d, y, num_part;
     char part[5];
@@ -681,7 +721,7 @@ decode_rdac_vpd_c1(unsigned char * buff, int len)
 }
 
 static void
-decode_rdac_vpd_c2(unsigned char * buff, int len)
+decode_rdac_vpd_c2(uint8_t * buff, int len)
 {
     int i, n, v, r, m, p, d, y, num_part;
     char part[5];
@@ -736,7 +776,7 @@ decode_rdac_vpd_c2(unsigned char * buff, int len)
 }
 
 static void
-decode_rdac_vpd_c3(unsigned char * buff, int len)
+decode_rdac_vpd_c3(uint8_t * buff, int len)
 {
     if (len < 0x2c) {
         pr2serr("Feature parameters VPD page length too short=%d\n", len);
@@ -758,7 +798,7 @@ decode_rdac_vpd_c3(unsigned char * buff, int len)
 }
 
 static void
-decode_rdac_vpd_c4(unsigned char * buff, int len)
+decode_rdac_vpd_c4(uint8_t * buff, int len)
 {
     char subsystem_id[17];
     char subsystem_rev[5];
@@ -828,7 +868,7 @@ decode_rdac_vpd_c4(unsigned char * buff, int len)
 }
 
 static void
-convert_binary_to_ascii(unsigned char * src, unsigned char * dst,  int len)
+convert_binary_to_ascii(uint8_t * src, uint8_t * dst,  int len)
 {
     int i;
 
@@ -838,7 +878,7 @@ convert_binary_to_ascii(unsigned char * src, unsigned char * dst,  int len)
 }
 
 static void
-decode_rdac_vpd_c8(unsigned char * buff, int len)
+decode_rdac_vpd_c8(uint8_t * buff, int len)
 {
     int i;
 #ifndef SG_LIB_MINGW
@@ -849,7 +889,7 @@ decode_rdac_vpd_c8(unsigned char * buff, int len)
     int label_len;
     char uuid[33];
     int uuid_len;
-    unsigned char port_id[128];
+    uint8_t port_id[128];
     int n;
 
     if (len < 0xab) {
@@ -949,7 +989,7 @@ decode_rdac_vpd_c8(unsigned char * buff, int len)
 }
 
 static void
-decode_rdac_vpd_c9_rtpg_data(unsigned char aas, unsigned char vendor)
+decode_rdac_vpd_c9_rtpg_data(uint8_t aas, uint8_t vendor)
 {
     printf("  Asymmetric Access State:");
     switch(aas & 0x0F) {
@@ -1011,7 +1051,7 @@ decode_rdac_vpd_c9_rtpg_data(unsigned char aas, unsigned char vendor)
 }
 
 static void
-decode_rdac_vpd_c9(unsigned char * buff, int len)
+decode_rdac_vpd_c9(uint8_t * buff, int len)
 {
     if (len < 3) {
         pr2serr("Volume Access Control VPD page length too short=%d\n", len);
@@ -1115,7 +1155,7 @@ decode_rdac_vpd_c9(unsigned char * buff, int len)
 }
 
 static void
-decode_rdac_vpd_ca(unsigned char * buff, int len)
+decode_rdac_vpd_ca(uint8_t * buff, int len)
 {
     int i;
 
@@ -1148,7 +1188,7 @@ decode_rdac_vpd_ca(unsigned char * buff, int len)
 }
 
 static void
-decode_rdac_vpd_d0(unsigned char * buff, int len)
+decode_rdac_vpd_d0(uint8_t * buff, int len)
 {
     int i;
 
@@ -1167,7 +1207,7 @@ decode_rdac_vpd_d0(unsigned char * buff, int len)
 
 
 static void
-decode_dds_vpd_c0(unsigned char * buff, int len)
+decode_dds_vpd_c0(uint8_t * buff, int len)
 {
     char firmware_rev[25];
     char build_date[43];
@@ -1200,7 +1240,7 @@ decode_dds_vpd_c0(unsigned char * buff, int len)
 }
 
 static void
-decode_hp_lto_vpd_cx(unsigned char * buff, int len, int page)
+decode_hp_lto_vpd_cx(uint8_t * buff, int len, int page)
 {
     char str[32];
     const char *comp = NULL;
@@ -1255,7 +1295,7 @@ decode_hp_lto_vpd_cx(unsigned char * buff, int len, int page)
 }
 
 static void
-decode_ibm_lto_dcrl(unsigned char * buff, int len)
+decode_ibm_lto_dcrl(uint8_t * buff, int len)
 {
     if (len < 0x2b) {
         pr2serr("Driver Component Revision Levels page (IBM LTO) invalid "
@@ -1269,7 +1309,7 @@ decode_ibm_lto_dcrl(unsigned char * buff, int len)
 }
 
 static void
-decode_ibm_lto_dsn(unsigned char * buff, int len)
+decode_ibm_lto_dsn(uint8_t * buff, int len)
 {
     if (len < 0x1c) {
         pr2serr("Driver Serial Numbers page (IBM LTO) invalid "
@@ -1280,7 +1320,79 @@ decode_ibm_lto_dsn(unsigned char * buff, int len)
     printf("  Reported serial number: %.12s\n", buff + 16);
 }
 
-/* Returns 0 if successful, see sg_ll_inquiry() plus SG_LIB_SYNTAX_ERROR for
+static void
+decode_vpd_3_hit(uint8_t * b, int blen)
+{
+    uint16_t plen = sg_get_unaligned_be16(b + 2);
+
+    if ((plen < 184) || (blen < 184)) {
+        pr2serr("Hitachi VPD page 0x3 length (%u) shorter than %u\n",
+                plen + 4, 184 + 4);
+        return;
+    }
+    printf("  ASCII uCode Identifier: %.12s\n", b + 24);
+    printf("  ASCII servo P/N: %.4s\n", b + 36);
+    printf("  Major Version: %.2s\n", b + 40);
+    printf("  Minor Version: %.2s\n", b + 42);
+    printf("  User Count: %.4s\n", b + 44);
+    printf("  Build Number: %.4s\n", b + 48);
+    printf("  Build Date String: %.32s\n", b + 52);
+    printf("  Product ID: %.8s\n", b + 84);
+    printf("  Interface ID: %.8s\n", b + 92);
+    printf("  Code Type: %.8s\n", b + 100);
+    printf("  User Name: %.12s\n", b + 108);
+    printf("  Machine Name: %.16s\n", b + 120);
+    printf("  Directory Name: %.32s\n", b + 136);
+    printf("  Operating state: %u\n", sg_get_unaligned_be32(b + 168));
+    printf("  Functional Mode: %u\n", sg_get_unaligned_be32(b + 172));
+    printf("  Degraded Reason: %u\n", sg_get_unaligned_be32(b + 176));
+    printf("  Broken Reason: %u\n", sg_get_unaligned_be32(b + 180));
+    printf("  Code Mode: %u\n", sg_get_unaligned_be32(b + 184));
+    printf("  Revision: %.4s\n", b + 188);
+}
+
+static void
+decode_vpd_d1_hit(uint8_t * b, int blen)
+{
+    uint16_t plen = sg_get_unaligned_be16(b + 2);
+
+    if ((plen < 80) || (blen < 80)) {
+        pr2serr("Hitachi VPD page 0xd1 length (%u) shorter than %u\n",
+                plen + 4, 80 + 4);
+        return;
+    }
+    printf("  ASCII Media Disk Definition: %.16s\n", b + 4);
+    printf("  ASCII Motor Serial Number: %.16s\n", b + 20);
+    printf("  ASCII Flex Assembly Serial Number: %.16s\n", b + 36);
+    printf("  ASCII Actuator Serial Number: %.16s\n", b + 52);
+    printf("  ASCII Device Enclosure Serial Number: %.16s\n", b + 68);
+}
+
+static void
+decode_vpd_d2_hit(uint8_t * b, int blen)
+{
+    uint16_t plen = sg_get_unaligned_be16(b + 2);
+
+    if ((plen < 52) || (blen < 52)) {
+        pr2serr("Hitachi VPD page 0xd2 length (%u) shorter than %u\n",
+                plen + 4, 52 + 4);
+        return;
+    }
+    if ((blen - 4) == 120) {
+        printf("  HDC Version: %.*s\n", b[4], b + 5);
+        printf("  Card Serial Number: %.*s\n", b[24], b + 25);
+        printf("  NAND Flash Version: %.*s\n", b[44], b + 45);
+        printf("  Card Assembly Part Number: %.*s\n", b[64], b + 65);
+        printf("  Second Card Serial Number: %.*s\n", b[84], b + 85);
+        printf("  Second Card Assembly Part Number: %.*s\n", b[104], b + 105);
+    } else {
+        printf("  ASCII HDC Version: %.16s\n", b + 5);
+        printf("  ASCII Card Serial Number: %.16s\n", b + 22);
+        printf("  ASCII Card Assembly Part Number: %.16s\n", b + 39);
+    }
+}
+
+/* Returns 0 if successful, see sg_ll_inquiry() plus SG_LIB_CAT_OTHER for
    unsupported page */
 int
 svpd_decode_vendor(int sg_fd, struct opts_t * op, int off)
@@ -1289,31 +1401,54 @@ svpd_decode_vendor(int sg_fd, struct opts_t * op, int off)
     char name[64];
     const struct svpd_values_name_t * vnp;
     int alloc_len = op->maxlen;
-    unsigned char * rp;
+    uint8_t * rp;
 
+    switch (op->vpd_pn) {
+    case 0x3:
+    case 0xc0:
+    case 0xc1:
+    case 0xc2:
+    case 0xc3:
+    case 0xc4:
+    case 0xc5:
+    case 0xc8:
+    case 0xc9:
+    case 0xca:
+    case 0xd0:
+    case 0xd1:
+    case 0xd2:
+        break;
+    default:    /* not known so return prior to fetching page */
+        return SG_LIB_CAT_OTHER;
+    }
     rp = rsp_buff + off;
     if (sg_fd >= 0) {
         if (0 == alloc_len)
             alloc_len = DEF_ALLOC_LEN;
     }
-    res = vpd_fetch_page_from_dev(sg_fd, rp, op->num_vpd, alloc_len,
+    res = vpd_fetch_page(sg_fd, rp, op->vpd_pn, alloc_len, op->do_quiet,
                                   op->verbose, &len);
     if (0 == res) {
-        vnp = svpd_get_v_detail(op->num_vpd, op->vend_prod_num, 0xf & rp[0]);
+        vnp = svpd_get_v_detail(op->vpd_pn, op->vend_prod_num, 0xf & rp[0]);
         if (vnp && vnp->name)
-            strcpy(name, vnp->name);
+            snprintf(name, sizeof(name), "%s", vnp->name);
         else
             snprintf(name, sizeof(name) - 1, "Vendor VPD page=0x%x",
-                     op->num_vpd);
+                     op->vpd_pn);
         if ((! op->do_raw) && (! op->do_quiet) && (op->do_hex < 2))
             printf("%s VPD Page:\n", name);
         if (op->do_raw)
-            dStrRaw((const char *)rp, len);
+            dStrRaw(rp, len);
         else if (op->do_hex)
-            dStrHex((const char *)rp, len,
-                    ((1 == op->do_hex) ? 0 : -1));
+            hex2stdout(rp, len, ((1 == op->do_hex) ? 0 : -1));
         else {
-            switch(op->num_vpd) {
+            switch(op->vpd_pn) {
+                case 0x3:
+                    if (VPD_VP_WDC_HITACHI == op->vend_prod_num)
+                        decode_vpd_3_hit(rp, len);
+                    else
+                        hex2stdout(rp, len, 0);
+                    break;
                 case 0xc0:
                     if (VPD_VP_SEAGATE == op->vend_prod_num)
                         decode_firm_vpd_c0_sea(rp, len);
@@ -1328,9 +1463,9 @@ svpd_decode_vendor(int sg_fd, struct opts_t * op, int off)
                     else if (VPD_VP_IBM_LTO == op->vend_prod_num)
                         decode_ibm_lto_dcrl(rp, len);
                     else if (VPD_VP_HP_LTO == op->vend_prod_num)
-                        decode_hp_lto_vpd_cx(rp, len, op->num_vpd);
+                        decode_hp_lto_vpd_cx(rp, len, op->vpd_pn);
                     else
-                        dStrHex((const char *)rp, len, 0);
+                        hex2stdout(rp, len, 0);
                     break;
                 case 0xc1:
                     if (VPD_VP_SEAGATE == op->vend_prod_num)
@@ -1340,17 +1475,17 @@ svpd_decode_vendor(int sg_fd, struct opts_t * op, int off)
                     else if (VPD_VP_IBM_LTO == op->vend_prod_num)
                         decode_ibm_lto_dsn(rp, len);
                     else if (VPD_VP_HP_LTO == op->vend_prod_num)
-                        decode_hp_lto_vpd_cx(rp, len, op->num_vpd);
+                        decode_hp_lto_vpd_cx(rp, len, op->vpd_pn);
                     else
-                        dStrHex((const char *)rp, len, 0);
+                        hex2stdout(rp, len, 0);
                     break;
                 case 0xc2:
                     if (VPD_VP_RDAC == op->vend_prod_num)
                         decode_rdac_vpd_c2(rp, len);
                     else if (VPD_VP_HP_LTO == op->vend_prod_num)
-                        decode_hp_lto_vpd_cx(rp, len, op->num_vpd);
+                        decode_hp_lto_vpd_cx(rp, len, op->vpd_pn);
                     else
-                        dStrHex((const char *)rp, len, 0);
+                        hex2stdout(rp, len, 0);
                     break;
                 case 0xc3:
                     if (VPD_VP_SEAGATE == op->vend_prod_num)
@@ -1358,54 +1493,68 @@ svpd_decode_vendor(int sg_fd, struct opts_t * op, int off)
                     else if (VPD_VP_RDAC == op->vend_prod_num)
                         decode_rdac_vpd_c3(rp, len);
                     else if (VPD_VP_HP_LTO == op->vend_prod_num)
-                        decode_hp_lto_vpd_cx(rp, len, op->num_vpd);
+                        decode_hp_lto_vpd_cx(rp, len, op->vpd_pn);
                     else
-                        dStrHex((const char *)rp, len, 0);
+                        hex2stdout(rp, len, 0);
                     break;
                 case 0xc4:
                     if (VPD_VP_RDAC == op->vend_prod_num)
                         decode_rdac_vpd_c4(rp, len);
                     else if (VPD_VP_HP_LTO == op->vend_prod_num)
-                        decode_hp_lto_vpd_cx(rp, len, op->num_vpd);
+                        decode_hp_lto_vpd_cx(rp, len, op->vpd_pn);
                     else
-                        dStrHex((const char *)rp, len, 0);
+                        hex2stdout(rp, len, 0);
                     break;
                 case 0xc5:
                     if (VPD_VP_HP_LTO == op->vend_prod_num)
-                        decode_hp_lto_vpd_cx(rp, len, op->num_vpd);
+                        decode_hp_lto_vpd_cx(rp, len, op->vpd_pn);
                     else
-                        dStrHex((const char *)rp, len, 0);
+                        hex2stdout(rp, len, 0);
                     break;
                 case 0xc8:
                     if (VPD_VP_RDAC == op->vend_prod_num)
                         decode_rdac_vpd_c8(rp, len);
                     else
-                        dStrHex((const char *)rp, len, 0);
+                        hex2stdout(rp, len, 0);
                     break;
                 case 0xc9:
                     if (VPD_VP_RDAC == op->vend_prod_num)
                         decode_rdac_vpd_c9(rp, len);
                     else
-                        dStrHex((const char *)rp, len, 0);
+                        hex2stdout(rp, len, 0);
                     break;
                 case 0xca:
                     if (VPD_VP_RDAC == op->vend_prod_num)
                         decode_rdac_vpd_ca(rp, len);
                     else
-                        dStrHex((const char *)rp, len, 0);
+                        hex2stdout(rp, len, 0);
                     break;
                 case 0xd0:
                     if (VPD_VP_RDAC == op->vend_prod_num)
                         decode_rdac_vpd_d0(rp, len);
                     else
-                        dStrHex((const char *)rp, len, 0);
+                        hex2stdout(rp, len, 0);
+                    break;
+                case 0xd1:
+                    if (VPD_VP_WDC_HITACHI == op->vend_prod_num)
+                        decode_vpd_d1_hit(rp, len);
+                    else
+                        hex2stdout(rp, len, 0);
+                    break;
+                case 0xd2:
+                    if (VPD_VP_WDC_HITACHI == op->vend_prod_num)
+                        decode_vpd_d2_hit(rp, len);
+                    else
+                        hex2stdout(rp, len, 0);
                     break;
                 default:
-                    return SG_LIB_SYNTAX_ERROR;
+                    pr2serr("%s: logic error, should know can't decode "
+                            "pn=0x%x\n", __func__, op->vpd_pn);
+                    return SG_LIB_CAT_OTHER;
             }
             return 0;
         }
     } else
-        pr2serr("Vendor VPD page=0x%x  failed to fetch", op->num_vpd);
+        pr2serr("Vendor VPD page=0x%x  failed to fetch", op->vpd_pn);
     return res;
 }
