@@ -1,9 +1,11 @@
 /*
- * Copyright (c) 2014-2018 Douglas Gilbert
+ * Copyright (c) 2014-2019 Douglas Gilbert
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * This program issues the SCSI command WRITE AND VERIFY to a given SCSI
  * device. It sends the command with the logical block address passed as the
  * LBA argument, for the given number of blocks. The number of bytes sent is
@@ -38,7 +40,7 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
-static const char * version_str = "1.15 20180628";
+static const char * version_str = "1.17 20191220";
 
 
 #define ME "sg_write_verify: "
@@ -125,18 +127,18 @@ run_scsi_transaction(int sg_fd, const uint8_t *cdbp, int cdb_len,
                      uint8_t *dop, int do_len, int timeout,
                      bool noisy, int verbose)
 {
-    int res, k, sense_cat, ret;
+    int res, sense_cat, ret;
     struct sg_pt_base * ptvp;
     uint8_t sense_b[SENSE_BUFF_LEN];
     char b[32];
 
     snprintf(b, sizeof(b), "Write and verify(%d)", cdb_len);
     if (verbose) {
-       pr2serr("    %s cdb: ", b);
-       for (k = 0; k < cdb_len; ++k)
-           pr2serr("%02x ", cdbp[k]);
-       pr2serr("\n");
-       if ((verbose > 2) && dop && do_len) {
+        char d[128];
+
+        pr2serr("    %s cdb: %s\n", b,
+                sg_get_command_str(cdbp, cdb_len, false, sizeof(d), d));
+        if ((verbose > 2) && dop && do_len) {
             pr2serr("    Data out buffer [%d bytes]:\n", do_len);
             hex2stderr(dop, do_len, -1);
         }
@@ -150,8 +152,7 @@ run_scsi_transaction(int sg_fd, const uint8_t *cdbp, int cdb_len,
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     set_scsi_pt_data_out(ptvp, dop, do_len);
     res = do_scsi_pt(ptvp, sg_fd, timeout, verbose);
-    ret = sg_cmds_process_resp(ptvp, b, res, SG_NO_DATA_IN, sense_b, noisy,
-                               verbose, &sense_cat);
+    ret = sg_cmds_process_resp(ptvp, b, res, noisy, verbose, &sense_cat);
     if (-1 == ret)
         ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
     else if (-2 == ret) {
