@@ -1,12 +1,14 @@
 /*
- * Copyright (c) 2018 Douglas Gilbert
+ * Copyright (c) 2018-2019 Douglas Gilbert
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * This program issues a NVMe Identify command (controller or namespace)
  * or a Device self-test command via the "SCSI" pass-through interface of
- * this packages sg_utils library. That interface is primarily shown in
+ * this package's sg_utils library. That interface is primarily shown in
  * the ../include/sg_pt.h header file.
  *
  */
@@ -37,7 +39,7 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
-static const char * version_str = "1.04 20180220";
+static const char * version_str = "1.06 20190113";
 
 
 #define ME "sg_tst_nvme: "
@@ -537,8 +539,7 @@ sg_scsi_inquiry(struct sg_pt_base * ptvp, bool evpd, int pg_op, void * resp,
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     set_scsi_pt_data_in(ptvp, (uint8_t *)resp, mx_resp_len);
     res = do_scsi_pt(ptvp, -1, timeout_secs, vb);
-    ret = sg_cmds_process_resp(ptvp, "inquiry", res, mx_resp_len, sense_b,
-                               noisy, vb, &sense_cat);
+    ret = sg_cmds_process_resp(ptvp, "inquiry", res, noisy, vb, &sense_cat);
     resid = get_scsi_pt_resid(ptvp);
     if (residp)
         *residp = resid;
